//------------------------------------------------------------------------------
// Purpose: 
//
// Date   : 12. Sep. 2011
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using Fusym;

namespace Concepts
{
    public class InferenceEngine
    {
        public static List<InferenceRule> RuleBase = new List<InferenceRule>();

        private SemanticNetwork m_SemanticNetwork;
        private Node m_Implies;
        private Node m_Equivalent;

        public InferenceEngine(SemanticNetwork semanticNetwork)
        {
            m_SemanticNetwork = semanticNetwork;
            m_Implies    = m_SemanticNetwork.GetNode("=>");
            m_Equivalent = m_SemanticNetwork.GetNode("<=>");
        }


        /*
        public void Apply(InferenceRule rule, Concept concept)
        {
            Node ruleType = rule.RuleType;

            if (ruleType == m_Implies)
                rule = rule;
            else
            if (ruleType == m_Equivalent)
                rule = rule;
            else
                throw new Exception("Illegal inference rule symbol " + ruleType.ToString());
        }*/


        public List<Binding> Match(Concept leftSideOfRule, Concept instance)
        {
            List<Binding> bindings = new List<Binding>();
            List<Binding> result = Match(leftSideOfRule, instance, bindings);
            return result;
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// (haben (wer ?x) (was hunger)) => (ist (wer ?x) (was hungrig))
        /// (haben (wer hans) (was hunger))
        /// </summary>
        ///---------------------------------------------------------------------
        public List<Binding> Match(Concept leftSideOfRule, Concept instance, List<Binding> bindings)
        {
            // Concept names must match.
            if (leftSideOfRule.ConceptName != instance.ConceptName)
                return null;

            List<SlotValuePair> slotValuePairsLeft = leftSideOfRule.SlotValuePairs;
            int countLeft = slotValuePairsLeft.Count;

            // Walk through all slots on the left side of the rule
            for (int i = 0; i < countLeft; i++)
            {
                SlotValuePair pairLeft = slotValuePairsLeft[i];

                Node    slotNameLeft  = pairLeft.SlotName;
                Concept slotValueLeft = pairLeft.SlotValue;



                // Try to find a slot in the instance with the same name.
                SlotValuePair pairInst = instance.FindSlot(slotNameLeft);

                if (pairInst == null)
                    return null;

                Concept slotValueInst = pairInst.SlotValue;

                if (IsVariable(slotValueLeft))
                {
                    Concept c = FindBinding(slotValueLeft.ConceptName, bindings);

                    // If it is a bound variable, replace variable with its value.
                    if (c != null)
                        slotValueLeft = c;
                    else
                    // Bind the variable.
                    {
                        Binding b = new Binding();
                        b.Variable = slotValueLeft.ConceptName;
                        b.Value    = slotValueInst;
                        bindings.Add(b);
                    }
                }
                else
                    Match(slotValueLeft, slotValueInst, bindings);

            }

            return bindings;
        }






        ///---------------------------------------------------------------------
        /// <summary>
        /// Apply a rule to an instance.
        /// </summary>
        ///---------------------------------------------------------------------
        public Concept Apply(
                             Concept leftSideOfRule,
                             Concept rightSideOfRule,
                             Concept instance, 
                             List<Binding> bindings
                            )
        {
            Concept result = new Concept();
            result.ConceptName = rightSideOfRule.ConceptName;

            //
            // If the name of the concept on the right side of
            // the rule is a variable, replace it by the value bound to it.
            //
            if (IsVariable(result.ConceptName))
                result.ConceptName = FindBinding(result.ConceptName, bindings).ConceptName;

            List<SlotValuePair> slotsFromRightSideOfTheRule = rightSideOfRule.SlotValuePairs;
            int countRight = slotsFromRightSideOfTheRule.Count;

            List<SlotValuePair> slotsFromLeftSideOfTheRule;
            int countLeft;

            if (leftSideOfRule != null)
            {
                slotsFromLeftSideOfTheRule = leftSideOfRule.SlotValuePairs;
                countLeft = slotsFromLeftSideOfTheRule.Count;
            }
            else
            {
                slotsFromLeftSideOfTheRule = null;
                countLeft = 0;
            }

            foreach (SlotValuePair p in slotsFromRightSideOfTheRule)
            {
                Node    n = p.SlotName;
                Concept c = p.SlotValue;
                Concept m;

                if (c == null)
                    m = null;
                else
                if (IsVariable(c))
                {
                    m = FindBinding(c.ConceptName, bindings);
                    if (m == null) // unbound variable
                        m = c; // remain unbound variable.
                }
                else
                    m = Apply(null, c, null, bindings);

                SlotValuePair pair = new SlotValuePair();
                pair.SlotName  = n;
                pair.SlotValue = m;
                result.SlotValuePairs.Add(pair);
            }



            if (instance == null)
                return result;



            List<SlotValuePair> slotsFromInstance = instance.SlotValuePairs;
            countRight = slotsFromInstance.Count;

            foreach (SlotValuePair p in slotsFromInstance)
            {
                //--------------------------------------------------------------
                Node n = p.SlotName;
                Concept c = p.SlotValue;
                Concept m;

                // If the current slot is mentioned in the 
                // right side of the rule, it was already handled.
                bool found = false;
                foreach (SlotValuePair q in slotsFromRightSideOfTheRule)
                {
                    Node n2 = q.SlotName;
                    if (n2 == n)
                    {
                        found = true;
                        break;
                    }
                }

                if (found)
                    continue;

                //--------------------------------------------------------------

                // If the current slot is mentioned in the 
                // left side of the rule, it was already handled.
                found = false;
                foreach (SlotValuePair q in slotsFromLeftSideOfTheRule)
                {
                    Node n2 = q.SlotName;
                    if (n2 == n)
                    {
                        found = true;
                        break;
                    }
                }

                if (found)
                    continue;

                //--------------------------------------------------------------






                // Simply copy the slot-value-pair.
                result.SlotValuePairs.Add(p);
            }









            return result;
        }









        ///---------------------------------------------------------------------
        /// <summary>
        /// Determine whether a concept is a variable.
        /// It should be a symbol and should begin with a '?'.
        /// </summary>
        ///---------------------------------------------------------------------
        private bool IsVariable(Concept c)
        {
            string s = c.ConceptName.ToString();
            if (s.Length > 0)
                return s[0] == '?';
            else
                return false;
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Determine whether a concept is a variable.
        /// It should be a symbol and should begin with a '?'.
        /// </summary>
        ///---------------------------------------------------------------------
        private bool IsVariable(Node n)
        {
            string s = n.ToString();
            if (s.Length > 0)
                return s[0] == '?';
            else
                return false;
        }
        
        ///---------------------------------------------------------------------
        /// <summary>
        /// Find a binding of a variable.
        /// </summary>
        ///---------------------------------------------------------------------
        public Concept FindBinding(Node n, List<Binding> bindings)
        {
            int count = bindings.Count;
            for (int i = 0; i < count; i++)
            {
                Binding b = bindings[i];
                if (n == b.Variable)
                    return b.Value;
            }

            return null;
        }
    }


    ///-------------------------------------------------------------------------
    /// <summary>
    /// Binding of a value to a variable.
    /// Assignments to a variable.
    /// </summary>
    ///-------------------------------------------------------------------------
    public class Binding
    {
        public Node    Variable;
        public Concept Value;
    }

}
