//------------------------------------------------------------------------------
// Purpose: Load
// Date   : 04. Apr. 2012
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Fusym
{
    public interface ILoadingProgressReceiver
    {
        void Progress(string item, int percent);
    }

    public class AssociationFiles
    {
        private SemanticNetwork m_SemanticNetwork;
        private Encoding m_Encoding;
        private List<string> m_Filenames;
        private List<DateTime> m_Dates;
        private int m_Count;
        private ILoadingProgressReceiver m_Receiver;

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public AssociationFiles(IList<string> filenames, Encoding encoding, ILoadingProgressReceiver receiver)
        {
            m_Encoding = encoding;
            m_Receiver = receiver;

            m_Filenames = new List<string>(filenames);
            m_Dates = new List<DateTime>();

            foreach (string fn in m_Filenames)
                m_Dates.Add(DateTime.MinValue);

            m_Count = m_Filenames.Count;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load association files, but only if write date has changed.
        /// </summary>
        ///---------------------------------------------------------------------
        public SemanticNetwork Get()
        {
            bool foundChange = UpdateFileDates();

            if ((foundChange == false) && (m_SemanticNetwork != null))
                return m_SemanticNetwork;

            m_SemanticNetwork = new SemanticNetwork(m_Encoding);


            for (int i = 0; i < m_Count; i++)
            {
                string fn = m_Filenames[i];

                try
                {
                    m_SemanticNetwork.LoadFromFile(fn);
                }
                catch
                {
                }

                if (m_Receiver != null)
                {
                    int percentage = (int)Math.Round(100.0 * i / (double)m_Count);
                    m_Receiver.Progress(fn, percentage);
                }
            }

            // At the end
            m_Receiver.Progress("", 0);

            return m_SemanticNetwork;
        }

        private bool UpdateFileDates()
        {
            bool result = false;

            for (int i = 0; i < m_Count; i++)
            {
                string fn = m_Filenames[i];
                FileInfo fi = new FileInfo(fn);

                DateTime d_old = m_Dates[i];
                DateTime d_new = fi.LastWriteTime;

                if (d_new != d_old)
                {
                    m_Dates[i] = d_new;
                    result = true;
                }
            }

            return result;
        }
    }
}
