//------------------------------------------------------------------------------
// Purpose: Database system based on BlospExpressionSymbol
// Date   : 29. Mar. 2008
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Fusym
{
    public class BlospDatabase : IDocumentList
    {
        List<BlospExpression> m_List = new List<BlospExpression>();

        // Koennte man mal zentralisieren.
        public static Encoding encoding = System.Text.Encoding.GetEncoding(1252);
        private Encoding m_Encoding = encoding;
        private int m_Current;

        public BlospDatabase()
        {
        }


        public int Count
        {
            get
            {
                return m_List.Count;
            }
        }


        public BlospExpression this[int index]
        {
            get
            {
                return m_List[index];
            }
            set
            {
                m_List[index] = value;
            }
        }
        

        public BlospExpression FindOrCreate(string conceptName)
        {
            BlospExpression be; //, nameLine;

            be = GetFirstDocument() as BlospExpression;
            while (be != null)
            {
                if (be.HasName(conceptName))
                   return be;

                be = GetNextDocument() as BlospExpression;
            }

            be = new BlospExpression();
            be.AddName(conceptName);

            Add(be);
            return be;
        }
                


        public BlospExpression Add()
        {
            BlospExpression result = new BlospExpression();
            m_List.Add(result);
            return result;
        }

        public void Add(BlospExpression be)
        {
            m_List.Add(be);
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Appends line from a textfile to the internal list m_Records.
        /// If you want to clear the internal representation, use Clear().
        /// </summary>
        /// <param name="filename">The filename of a text file.</param>
        ///---------------------------------------------------------------------
        public void LoadFromFile(string filename)
        {
            List<string> sl = new List<string>();
            StreamReader sr = null;

            BufferedStream bs = new BufferedStream(new FileStream(filename, FileMode.Open));

            sr = new StreamReader(bs, m_Encoding);
            try
            {
                string line;
                while ((line = sr.ReadLine()) != null)
                {
                    sl.Add(line);
                }
            }
            finally
            {
                sr.Close();
            }

            BlospParser blospParser = new BlospParser(sl);
            BlospExpression root = blospParser.Parse();

            if (root.Count > 0)
            {
                BlospExpressionSymbol be = root[0];
                foreach (BlospExpression bl in be.Children)
                {
                    m_List.Add(bl);
                }
            }

        }



        public void SaveToFile(string filename)
        {
            StringBuilder sb = new StringBuilder();

            foreach (BlospExpression bl in m_List)
            {
                sb.AppendLine(bl.ToString());
                sb.AppendLine();
            }

            WriteFile(filename, sb.ToString());
        }



        private bool WriteFile(string filename, string text)
        {
            try
            {
                FileStream stream = new FileStream(filename, FileMode.Create);
                if (!stream.CanWrite)
                {
                    stream.Close();
                    return false;
                }
                StreamWriter writer = new StreamWriter(stream, m_Encoding);
                writer.Write(text);
                writer.Flush();
                writer.Close();
                stream.Close();
                return true;
            }
            catch
            {
                return false;
            }
        }






        public IDocument GetFirstDocument()
        {
            m_Current = 0;
            return GetNextDocument();
        }

        public IDocument GetNextDocument()
        {
            if (m_Current < m_List.Count)
                return m_List[m_Current++];
            else
                return null;
        }
    }
}
