//------------------------------------------------------------------------------
// Purpose: A BlospExpressionSymbol is a string with 0 or more children
//          in subsequent lines.
// Date   : 27. May 2008
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using Symps;

namespace Fusym
{
    ///-------------------------------------------------------------------------
    /// <summary>
    /// Represents a space-bounded string with its indentation where it
    /// is found in a line. A BlospExpressionSymbol can have children,
    /// which are other BlospExpressions found in subsequent lines.
    /// 
    /// A line is a list of BlospExpressions, therefore the children
    /// are a list of a list of BlospExpressions.
    /// </summary>
    ///-------------------------------------------------------------------------
    public class BlospExpressionSymbol // : IDocumentList, IDocument
    {
        private string m_Name;
        private string m_RestOfTheLine;
        private int m_Indentation;
        private List<BlospExpression> m_Children;
        //        private int m_Current;

        protected static bool m_SearchConceptNames = false;
        protected static bool m_SearchSlotNames = false;
        protected static bool m_SearchSlotValues = false;

        public static bool SearchConceptNames
        {
            get { return m_SearchConceptNames; }
            set { m_SearchConceptNames = value; }
        }

        public static bool SearchSlotNames
        {
            get { return m_SearchSlotNames; }
            set { m_SearchSlotNames = value; }
        }

        public static bool SearchSlotValues
        {
            get { return m_SearchSlotValues; }
            set { m_SearchSlotValues = value; }
        }



        public BlospExpressionSymbol(string s, int i)
        {
            m_Name = s;
            m_Indentation = i;
        }

        public string Name
        {
            get { return m_Name != null ? m_Name : ""; }
            set { m_Name = value; }
        }

        public string RestOfTheLine
        {
            get { return m_RestOfTheLine; }
            set { m_RestOfTheLine = value; }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// The column where the string begins.
        /// </summary>
        ///---------------------------------------------------------------------
        public int Indentation
        {
            get { return m_Indentation; }
            set { m_Indentation = value; }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Geth the children of this BlospExpressionSymbol.
        /// </summary>
        ///---------------------------------------------------------------------
        public List<BlospExpression> Children
        {
            get
            {
                if (m_Children == null)
                    m_Children = new List<BlospExpression>();

                return m_Children;
            }
        }


        public bool HasChildren
        {
            get
            {
                return (m_Children != null) && (m_Children.Count > 0);
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Add a new child to this BlospExpressionSymbol. A child is a line.
        /// </summary>
        /// <param name="line"></param>
        ///---------------------------------------------------------------------
        public void AddChild(BlospExpression line)
        {
            if (m_Children == null)
                m_Children = new List<BlospExpression>();

            m_Children.Add(line);
        }

        public void InsertChild(int index, BlospExpression line)
        {
            if (m_Children == null)
                m_Children = new List<BlospExpression>();

            m_Children.Insert(index, line);
        }






        ///---------------------------------------------------------------------
        /// <summary>
        /// Create a slot from a line that contains at least 2 elements
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        ///---------------------------------------------------------------------
        public Slot ListAsSlot(List<BlospExpressionSymbol> list)
        {
            if ((list == null) || (list.Count < 2))
                throw new Exception("ListAsSlot: not slot list");

            return new Slot(list[0].Name, list[0].RestOfTheLine);
        }







        /*



        public IDocument GetFirstDocument()
        {
            m_Current = 0;
            return GetNextDocument();
        }

        public IDocument GetNextDocument()
        {
//private List<List<BlospExpressionSymbol>> m_Children;

            if (m_Current < m_List.Count)
                return m_List[m_Current++];
            else
                return null;
        }



        public string GetFirstSymbol()
        {
            if (listOfAllWords == null)
                listOfAllWords = new List<string>(1 + m_Slots.Count * 2);
            else
                listOfAllWords.Clear();

            current = 0;

            if (m_SearchConceptNames)
                listOfAllWords.Add(m_Name);

            if (m_SearchSlotNames)
                for (int i = 0; i < m_Slots.Count; i++)
                    listOfAllWords.Add(m_Slots[i].Name);

            if (m_SearchSlotValues)
                for (int i = 0; i < m_Slots.Count; i++)
                    if ((m_Slots[i] is Slot) && ((m_Slots[i] as Slot).Value is string))
                    {
                        string s = (m_Slots[i] as Slot).Value as string;
                        string[] words = s.Split(' ');

                        listOfAllWords.AddRange(words);
                    }


            return GetNextSymbol();
        }

        public string GetNextSymbol()
        {
            if (current < listOfAllWords.Count)
                return listOfAllWords[current++];
            else
                return "";
        }
         * */

    }

}
