//------------------------------------------------------------------------------
// Purpose: A distance measure based on clouds of symbols.
//
// Date   : 8.Nov.2009
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public class CloudMatching
    {
        private SemanticNetwork m_SemanticNetwork;


        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public CloudMatching(SemanticNetwork semanticNetwork)
        {
            m_SemanticNetwork = semanticNetwork;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Calculates the semantic distance between two symbols.
        /// </summary>
        ///---------------------------------------------------------------------
        public int SemanticDistance(
                                    string symbol1, 
                                    string symbol2, 
                                    int maxCount, 
                                    int maxDepth
                                   )
        {
            const int cFactor = 1; // Zu testen zu-tun
            double sumOfCircle1;
            double sumOfCircle2;

            Dictionary<Node, int> dict = new Dictionary<Node, int>();
            List<Node> list;
            

            // Circle around symbol1
            list = m_SemanticNetwork.Circle(symbol1, maxCount, maxDepth);
            int count1 = list.Count;
            sumOfCircle1 = SumOfCircle(list);

            // Add items of first circle to hashtable
            for (int i = 0; i < list.Count; i++)
            {
                Node n = list[i];
                dict.Add(n, n.Mark);
            }

            // Circle around symbol2
            list = m_SemanticNetwork.Circle(symbol2, maxCount, maxDepth);
            int count2 = list.Count;
            sumOfCircle2 = SumOfCircle(list);


            double sumOfMatches = 0.0;
            for (int i = 0; i < list.Count; i++)
            {
                Node n = list[i];
                

                int mark1, mark2;
                if (dict.TryGetValue(n, out mark1))
                {
                    mark2 = n.Mark;
                    int maxMark = Math.Max(mark1, mark2);
                    double value = Math.Exp(-maxMark * cFactor);
                    sumOfMatches += value;
                }
            }



            // Normalization to 0..1

            double result;
            double divisor = Math.Max(sumOfCircle1, sumOfCircle2);

            if (divisor != 0)
                result =  1 - sumOfMatches / divisor;
            else
                result = 0;

            return (int)Math.Round(100 * result);
        }


        private double SumOfCircle(List<Node> list)
        {
            double result = 0.0;
            for (int i = 0; i < list.Count; i++)
            {
                Node n = list[i];
                int mark = n.Mark;
                result += Math.Exp(-mark);
            }

            return result;
        }
    }
}
