//------------------------------------------------------------------------------
// Purpose: 
//
// Date   : 01. Aug. 2010
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Fusym
{
    public class DocumentEvaluatorRandomWalk : IDocumentEvaluator
    {
        private SemanticNetwork m_SemanticNetwork;
        private RandomWalk rw;
        private List<string> m_InputSymbols;
        private int m_InputSymbolCount;
        private int m_Depth;
        private int m_MaxCount;
        private string m_Symbols;


        public string Symbols
        {
            get { return m_Symbols;  }
        }

        public List<string> AssociatedSymbols
        {
            get
            {
                List<Node> nodes = rw.SearchSymbol(m_InputSymbols, m_MaxCount, m_Depth);
                List<string> list = new List<string>();
                foreach (Node n in nodes)
                    list.Add(n.ToString());
                return list;
            }
        } 

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public DocumentEvaluatorRandomWalk(
                                             SemanticNetwork semanticNetwork,
                                             List<string> inputSymbols,
                                             int depth,
                                             int maxCount
                                          )
        {
            m_SemanticNetwork = semanticNetwork;
            m_InputSymbols = inputSymbols;
            m_InputSymbolCount = m_InputSymbols.Count;
            m_Depth = depth;
            m_MaxCount = maxCount;

            rw = new RandomWalk(semanticNetwork);
            rw.Prepare(m_InputSymbols, m_MaxCount, m_Depth);
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Evaluate a document.
        /// </summary>
        ///---------------------------------------------------------------------
        public double Evaluate(IDocument document)
        {
            EvaluationVector v1 = new EvaluationVector(m_InputSymbolCount, 0);

            Dictionary<string, EvaluationVector> beta = rw.DictWordToEvaluationVector;
           
            // Combine vectors for all words in the document.
            string s = document.GetFirstSymbol();
            while (s != "")
            {
                s = s.ToUpper();

                EvaluationVector v2;
                if (beta.TryGetValue(s, out v2))
                    v1.MergeMaxima(v2);

                s = document.GetNextSymbol();
            }








            double sum = 0.0;
            int currentPair = 0;
            
            if (m_InputSymbolCount == 1)
                sum = v1.DistanceList[0];
            else
            for (int i = 0; i < m_InputSymbolCount; i++)
            {
                int dist_i = v1.DistanceList[i];

                
                for (int j = i + 1; j < m_InputSymbolCount; j++)
                {
//                    int dist_j = v1.DistanceList[i];
                    int dist_j = v1.DistanceList[j];

                    int maxDist = Math.Max(dist_i, dist_j);
                    int minDist = Math.Min(dist_i, dist_j);

                    // Distance between Node i and Node j
                    double ratio = rw.PairDistances_0_1[currentPair];

                    //sum += minDist * ratio + maxDist * (1 - ratio);
                    sum += maxDist * ratio + minDist * (1 - ratio);

                    currentPair++;
                }
            }

            m_Symbols = v1.ToString(); // Wohl Info, warum Dokument so bewertet wurde.

            double sehrGrosseZahl = 1000000000;

            return sehrGrosseZahl - sum;
        }

        


          

    }
}
