//------------------------------------------------------------------------------
// Provides a vector whose elements are the semantic distances to each 
// of the words of the query.
//
// Joachim Pimiskern, 01.May.2007, Freeware.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public class EvaluationVector
    {
        public List<int> DistanceList;
        public List<string> Symbols;
        public int Count;


        public int this[int idx]
        {
            get
            {
                return DistanceList[idx];
            }

            set
            {
                DistanceList[idx] = value;
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public EvaluationVector(int countOfElements, int maxDistance)
        {
            Count = countOfElements;


            DistanceList = new List<int>(countOfElements);
            Symbols     = new List<string>(countOfElements);

            // Make sure that the vector has enough elements.
            for (int i = 0; i < countOfElements; i++)
            {
                DistanceList.Add(maxDistance);
                Symbols.Add("???");
            }
        }


        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();

            for (int i = 0; i < Symbols.Count; i++)
            {
                string s = Symbols[i];

                bool alreadyUsed = false;
                for (int j = 0; j < i; j++)
                    if (Symbols[j] == s)
                    {
                        alreadyUsed = true;
                        break;
                    }


                if (alreadyUsed)
                    continue;


                if (i < Symbols.Count - 1)
                    sb.Append(s).Append(" ");
                else
                    sb.Append(s);
            }

            return sb.ToString();
        }







        ///---------------------------------------------------------------------
        /// <summary>
        /// Componentwise minimum vector.
        /// </summary>
        ///---------------------------------------------------------------------
        public void MergeMinima(EvaluationVector other)
        {
            List<int> otherDistanceList = other.DistanceList;
            List<string> otherSymbols   = other.Symbols;

            for (int i = 0; i < Count; i++)
            {
                int thisDistance = DistanceList[i];
                int otherDistance = otherDistanceList[i];

                if (otherDistance < thisDistance)
                {
                    DistanceList[i] = otherDistance;
                    Symbols[i] = otherSymbols[i];
                }
                else
                    if (
                        (otherDistance == thisDistance) &&
                        (Symbols[i] == "???")
                       )
                        Symbols[i] = otherSymbols[i];
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Componentwise maximum vector.
        /// </summary>
        ///---------------------------------------------------------------------
        public void MergeMaxima(EvaluationVector other)
        {
            List<int> otherDistanceList = other.DistanceList;
            List<string> otherSymbols = other.Symbols;

            for (int i = 0; i < Count; i++)
            {
                int thisDistance = DistanceList[i];
                int otherDistance = otherDistanceList[i];

                if (otherDistance > thisDistance)
                {
                    DistanceList[i] = otherDistance;
                    Symbols[i]      = otherSymbols[i];
                }
                else
                if (
                    (otherDistance == thisDistance) &&
                    (Symbols[i] == "???")
                   )
                    Symbols[i] = otherSymbols[i];
            }
        }
        
    }
}
