// Joachim Pimiskern, 19.3.2012. Freeware
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public class GraphClustering
    {
        private Random rnd = new Random();


        public void Test1()
        {
            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            SemanticNetwork sn = gcm.SemanticNetwork;

            string[] strings = new string[]
            {
                "hund",
                "katze",
                "maus",
                "trompete",
                "tuba",
                "violine"
            };

            Test(ToNodes(strings, sn));
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Convert an array of strings into a list of Nodes.
        /// </summary>
        ///---------------------------------------------------------------------
        private List<Node> ToNodes(string[] strings, SemanticNetwork sn)
        {
            List<Node> result = new List<Node>();
            foreach (string s in strings)
                result.Add(sn.AddSymbol(s.ToUpper()));
            return result;                
        }


        public void Test(List<Node> nodes)
        {
            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            SemanticNetwork sn = gcm.SemanticNetwork;

            Dictionary<Node, int> ends = new Dictionary<Node, int>();
            foreach (Node n in nodes)
                ends[n] = 1;

            Dictionary<Node, int> marks = new Dictionary<Node, int>();
            List<Node> list = new List<Node>();

            bool b = true;
            while (b)
            {

                int maxMark = 0;
                Node maxNode = null;

                foreach (Node s1 in nodes)
                    foreach (Node s2 in nodes)
                    {
                        sn.FindPath(s1, s2, list);
                        foreach (Node n in list)
                        {
                            int value;
                            if (!marks.TryGetValue(n, out value))
                            {
                                marks[n] = 0;
                                value = 0;
                            }

                            value++;
                            marks[n] = value;

                            if (!ends.ContainsKey(n) && (value > maxMark))
                            {
                                maxMark = value;
                                maxNode = n;
                            }
                        }
                    }

                if (maxNode != null)
                {
                    sn.Isolate(maxNode);
                    Clusters(nodes, sn);
                }
                else
                {
                    b = false;

                    //foreach (string s1 in strings)
                    //    foreach (string s2 in strings)
                    //    {
                    //        sn.FindPath(s1, s2, list);
                    //    }
                }
            }
        }

        private void Clusters(List<Node> nodes, SemanticNetwork sn)
        {
            Dictionary<Node, int> dict = new Dictionary<Node, int>();
            int count = nodes.Count;

            for (int i = 0; i < count; i++)
                dict[nodes[i]] = i;

            List<Node> list = new List<Node>();

            for (int i = 0; i < count; i++)
            {
                Node n1 = nodes[i];
                for (int j = i + 1; j < count; j++)
                {
                    Node n2 = nodes[j];

                    if (sn.FindPath(n1, n2, list))
                    {
                        int i1 = dict[n1];
                        int i2 = dict[n2];

                        int min = Math.Min(i1, i2);
                        dict[n1] = min;
                        dict[n2] = min;
                    }
                }
            }
        }
    }
}
