//------------------------------------------------------------------------------
// Joachim Pimiskern, 2006. Freeware. 
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    ///-------------------------------------------------------------------------
    /// <summary>
    /// This interface demands a document to be enumerable by Fusym.
    /// Return "" at the end.
    /// </summary>
    ///-------------------------------------------------------------------------
    public interface IDocument
    {
        string GetFirstSymbol();
        string GetNextSymbol();
    }


    ///-------------------------------------------------------------------------
    /// <summary>
    /// This interface provides a list of objects that implement IDocument
    /// </summary>
    ///-------------------------------------------------------------------------
    public interface IDocumentList
    {
        IDocument GetFirstDocument();
        IDocument GetNextDocument();
    }
    
    public interface IClusterProvider
    {
        List<string> GetFirstCluster();
        List<string> GetNextCluster();
    }

    public interface IDocumentEvaluator
    {
        string Symbols { get; }
        List<string> AssociatedSymbols { get; }
        double Evaluate(IDocument document);
    }
}
