//------------------------------------------------------------------------------
// Purpose: A Node type for Fusym technology.
//
// Date   : 26.Apr.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    ///-------------------------------------------------------------------------
    /// <summary>
    /// A node in the associative graph represents one symbol.
    /// </summary>
    ///-------------------------------------------------------------------------
    public class Node
    {
        private string m_symbol;
        public List<Node> Neighbours = new List<Node>();
        List<int> m_EdgeValue; 
        int CurrentNeighbour; // For enumeration
        public int Mark;
        private int m_Index;


        public int Index
        {
            get
            {
                return m_Index;
            }

            set
            {
                m_Index = value;
            }
        }


        public Node(string symbol)
        {
            m_symbol = symbol;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// A list that stores an integer value assigned to
        /// each edge of 'this' node.
        /// </summary>
        ///---------------------------------------------------------------------
        public List<int> EdgeValue
        {
            get {
                    if (m_EdgeValue == null)
                        m_EdgeValue = new List<int>(Neighbours.Count);
                        
                    return m_EdgeValue;
                }
        }
        
        
        public void InitEdgeValues(int value)
        {
            if (m_EdgeValue == null)
                m_EdgeValue = new List<int>(Neighbours.Count);

            m_EdgeValue.Clear();

            for (int i = 0; i < Neighbours.Count; i++)
                m_EdgeValue.Add(value);
        }
        

        ///---------------------------------------------------------------------
        /// <summary>
        /// Add a new neighbour to a node.
        /// </summary>
        ///---------------------------------------------------------------------
//        public void AddNeighbour(Node n)
  //      {
    //        if (!Neighbours.Contains(n))
      //          Neighbours.Add(n);
       // }

        public override string ToString()
        {
            return m_symbol;
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Enumeration of neighbour symbols.
        /// </summary>
        /// <returns>The first neighour, if there is one, null otherwise</returns>
        ///---------------------------------------------------------------------
        public Node GetFirstNeighbour()
        {
            CurrentNeighbour = 0;
            return GetNextNeighbour();
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Enumeration of neighbour symbols.
        /// </summary>
        /// <returns>The next neighour, if there is one, null otherwise</returns>
        ///---------------------------------------------------------------------
        public Node GetNextNeighbour()
        {
            if (CurrentNeighbour < Neighbours.Count)
                return Neighbours[CurrentNeighbour++];
            else
                return null;
        }

        public double ClusterCoefficient()
        {
            int n = Neighbours.Count;
            int k = 0;

            for (int i = 1; i < n; i++)
            {
                Node n_i = Neighbours[i];
                for (int j = 0; j < i; j++)
                {
                    Node n_j = Neighbours[j];
                    if (n_i.DirectNeighbor(n_j))
                        k++;
                }
            }

            if (k > 1)
                return 2 * n / (double)(k * (k - 1));
            else
                return 0;
        }

        private bool DirectNeighbor(Node n)
        {
            return Neighbours.Contains(n);
        }
    }

}
