//------------------------------------------------------------------------------
// Purpose: Organizes associative queries.
//
// Date   : 10. Sep. 2010
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public interface ISearchProgressReceiver
    {
        void SetCountOfDocuments(int count);
        void SetIndexOfDocument(int idx);
    }

    public class SearchManager
    {
        private ISearchProgressReceiver m_SearchProgressReceiver;
        private SemanticNetwork    m_SemanticNetwork;
        private IDocumentList      m_DocumentList;
        private IDocumentEvaluator m_DocumentEvaluator;
        private ValuedDocList      m_ValuedDocList;

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public SearchManager(
                             ISearchProgressReceiver searchProgressReceiver,
                             IDocumentList           documentList,
                             ValuedDocList           valuedDocList,
                             IDocumentEvaluator      documentEvaluator,
                             SemanticNetwork         semanticNetwork
                            )
        {
            m_SearchProgressReceiver  = searchProgressReceiver;
            m_DocumentList            = documentList;
            m_ValuedDocList           = valuedDocList;
            m_DocumentEvaluator       = documentEvaluator;
            m_SemanticNetwork         = semanticNetwork;
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Start the work.
        /// </summary>
        ///---------------------------------------------------------------------
        public void Run()
        {
            IDocument doc;



            // Determine the count of all records to consider, for the progressbar
            int Count = 0;
            doc = m_DocumentList.GetFirstDocument();
            while (doc != null)
            {
                Count++;
                doc = m_DocumentList.GetNextDocument();
            }

            m_SearchProgressReceiver.SetCountOfDocuments(Count);


            ///---------------------------------------------------------
            // The documentEvaluator can calculate a value of a document
            // because it can loop through all of the document's words.
            // A document is of type IDocument.
            //----------------------------------------------------------
            int i = 0;
            m_ValuedDocList.Clear();
            m_ValuedDocList.Capacity = Count;


            doc = m_DocumentList.GetFirstDocument();
            while (doc != null)
            {
                double value = m_DocumentEvaluator.Evaluate(doc);
                m_ValuedDocList.Add(
                                    doc,
                                    m_DocumentEvaluator.Symbols,
                                    value
                                   );

                i++;

                m_SearchProgressReceiver.SetIndexOfDocument(i);
                   

                doc = m_DocumentList.GetNextDocument();
            }

        }


    }
}
