//------------------------------------------------------------------------------
// Purpose: 
// Date   : 28. May 2008
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public class SlotValue
    {
        public virtual string TheLine()
        {
            throw new Exception("TheLine() must be overrided.");
        }

        public virtual BlospExpression ToBlospExpression()
        {
            BlospExpression result = new BlospExpression();
            result.Add(new BlospExpressionSymbol(TheLine(), 0));
            return result;
        }
    }

    public class SlotValueString : SlotValue
    {
        private string m_Value;

        public SlotValueString(string value)
        {
            m_Value = value;
        }

        public override string TheLine()
        {
            return m_Value;
        }


        public override string ToString()
        {
            return m_Value;
        }
    }

    public class SlotValueInteger : SlotValue
    {
        private int m_Value;

        public SlotValueInteger(int value)
        {
            m_Value = value;
        }

        public override string TheLine()
        {
            return m_Value.ToString();
        }

        public override string ToString()
        {
            return m_Value.ToString();
        }
    }

    public class SlotValueDouble : SlotValue
    {
        private double m_Value;

        public SlotValueDouble(double value)
        {
            m_Value = value;
        }

        public override string TheLine()
        {
            return m_Value.ToString();
        }

        public override string ToString()
        {
            return m_Value.ToString();
        }
    }

    public class SlotValueBoolean : SlotValue
    {
        private bool m_Value;

        public SlotValueBoolean(bool value)
        {
            m_Value = value;
        }

        public override string TheLine()
        {
            return m_Value.ToString();
        }

        public override string ToString()
        {
            return m_Value.ToString();
        }
    }

    public class SlotValueBlospExpression : SlotValue
    {
        private BlospExpression m_Value;

        public SlotValueBlospExpression(BlospExpression value)
        {
            m_Value = value;
        }

        public override string TheLine()
        {
            string result = "";

            for (int i = 0; i < m_Value.Count; i++)
            {
                result += m_Value[i].Name;
                if (i < m_Value.Count - 1)
                    result += " ";
            }

            return result;
        }

        public override BlospExpression ToBlospExpression()
        {
            return m_Value;
        }

        public override string ToString()
        {
            return m_Value.ToString();
        }
    }




}
