// Joachim Pimiskern, 11.4.2012
using System;
using System.Collections.Generic;
using System.Text;

namespace Fusym
{
    public class SymFndRndBetweenness
    {
        private SemanticNetwork m_SemanticNetwork;
        private RandomWalk m_RandomWalk;
        private Dictionary<Node, int> m_Dict = new Dictionary<Node, int>();
        private Random m_Rnd = new Random();

        public SymFndRndBetweenness(SemanticNetwork semanticNetwork)
        {
            m_SemanticNetwork = semanticNetwork;
            m_RandomWalk = new RandomWalk(m_SemanticNetwork);
        }

        //public List<NodeAndMark> Search(string queryString)
        //{
        //    List<string> input = StringUtils.GetQuerySymbols(queryString);

            

        //    List<List<NodeAndMark>> clouds = new List<List<NodeAndMark>>();
        //    m_Dict.Clear();


        //    foreach (string symbol in input)
        //    {
        //        List<NodeAndMark> cloud = Cloud(symbol);
        //        clouds.Add(cloud);
        //    }

        //    ProcessAllPairsOfClouds(clouds);

        //    List<NodeAndMark> result = EvaluateDictionary();

        //    return result;
        //}

        public List<NodeAndMark> Search(string queryString)
        {
            List<NodeAndMark> cloud = CloudMultipleSymbols(queryString);
            m_Dict.Clear();
            Paths(cloud, cloud);
            List<NodeAndMark> result = EvaluateDictionary();

            return result;
        }

        public List<Node> SearchNodes(string queryString)
        {
            List<NodeAndMark> nodesAndMarks = Search(queryString);
            List<Node> result = new List<Node>(nodesAndMarks.Count);

            foreach (NodeAndMark nm in nodesAndMarks)
            {
                nm.Node.Mark = nm.Mark;
                result.Add(nm.Node);
            }

            return result;
        }

        private List<Node> ToNodeList(Node n)
        {
            List<Node> result = new List<Node>();
            result.Add(n);
            return result;
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Find the cloud of symbols in the neighborhood of a central symbol.
        /// </summary>
        ///---------------------------------------------------------------------
        private List<NodeAndMark> Cloud(string symbol)
        {
            Node n = m_SemanticNetwork.AddSymbol(symbol); // symbol is known to be in capitals.
            //List<Node> circle = m_RandomWalk.GetCircleRandomWalk(n, 10000, 5);
            List<Node> circle = m_RandomWalk.RW_AllAllowed(ToNodeList(n), 10000,5, true);

            List<NodeAndMark> result = new List<NodeAndMark>(circle.Count);

            int max = 10;

            for (int i = 0; i < circle.Count; i++)
            {
                if (i > max)
                   break;

                Node m = circle[i];
                NodeAndMark nm = new NodeAndMark();
                nm.Node = m;
                nm.Mark = m.Mark;
                result.Add(nm);
            }

            return result;
        }

        private List<NodeAndMark> CloudMultipleSymbols(string symbols)
        {
            List<string> symbolList = StringUtils.GetQuerySymbols(symbols);
            List<Node> circle = m_RandomWalk.GetCircleRandomWalk(symbolList, 10000, 5, true);

            List<NodeAndMark> result = new List<NodeAndMark>(circle.Count);

            int max = 10;

            for (int i = 0; i < circle.Count; i++)
            {
                if (i > max)
                    break;

                Node m = circle[i];
                NodeAndMark nm = new NodeAndMark();
                nm.Node = m;
                nm.Mark = m.Mark;
                result.Add(nm);
            }

            return result;
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Process all pairs of clouds
        /// </summary>
        ///---------------------------------------------------------------------
        private void ProcessAllPairsOfClouds(List<List<NodeAndMark>> clouds)
        {
            int count = clouds.Count;

            for (int i = 0; i < count; i++)
            {
                List<NodeAndMark> cloud1 = clouds[i];
                for (int j = i + 1; j < count; j++)
                {
                    List<NodeAndMark> cloud2 = clouds[j];

                    Paths(cloud1, cloud2);
                }
            }
        }

        private void Paths(List<NodeAndMark> cloud1, List<NodeAndMark> cloud2)
        {
            List<Node> path = new List<Node>();

            int count1 = cloud1.Count;
            int count2 = cloud2.Count;

            for (int i = 0; i < count1; i++)
            {
                NodeAndMark nm1 = cloud1[i];

                for (int j = 0; j < count2; j++)
                {
                    //if (m_Rnd.NextDouble() > 0.99)
                    //    continue;

                    NodeAndMark nm2 = cloud2[j];

                    if (m_SemanticNetwork.FindPath(nm1.Node, nm2.Node, path))
                        StorePath(path);
                }
            }
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Store path in dictionary.
        /// </summary>
        ///---------------------------------------------------------------------
        private void StorePath(List<Node> path)
        {
            int count = path.Count;
            int value;

            foreach (Node n in path)
            {
                if (!m_Dict.TryGetValue(n, out value))
                    value = 0;

                value++;
                m_Dict[n] = value;
            }
        }


        private List<NodeAndMark> EvaluateDictionary()
        {
            List<NodeAndMark> result = new List<NodeAndMark>();

            foreach (KeyValuePair<Node, int> pair in m_Dict)
            {
                NodeAndMark nm = new NodeAndMark();

                nm.Node = pair.Key;
                nm.Mark = pair.Value;
                result.Add(nm);
            }


            result.Sort(NodeAndMark.CmpNodeAndMarkDesc);

            return result;
        }
    }
}
