// 7.5.2007
// Joachim Pimiskern
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Fusym;

namespace Fusym
{
    public class ValuedDocument
    {
        public IDocument Object;
        public double    Value;
        public string    theString;
        public bool      Mark;

        public ValuedDocument(IDocument obj, string theString, double value)
        {
            Object    = obj;
            this.theString = theString;
            Value     = value;
            Mark      = false;
        }
    }


    internal class MyComparer : IComparer<double>
    {
        int IComparer<double>.Compare(double x, double y)
        {
            if (x > y)
                return 1;
            else
                if (x < y)
                    return -1;
                else
                    return 1; // Trick
        }

    }








    public class ValuedDocList
    {
        private bool m_Prepared = false;

        private SortedList<double, ValuedDocument> m_SortedList = new SortedList<double, ValuedDocument>(new MyComparer());
        private List<ValuedDocument> list = new List<ValuedDocument>();


        ///---------------------------------------------------------------------
        /// <summary>
        /// Return the count of elements
        /// </summary>
        ///---------------------------------------------------------------------
        public int Count
        {
            get
            {
                Prepare();
                return list.Count;
            }
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Copy from m_SortedList to list, if not already done.
        /// This is necessary since m_SortedList has no integer indexer.
        /// </summary>
        ///---------------------------------------------------------------------
        private void Prepare()
        {
            if (m_Prepared)
                return;

            System.Collections.Generic.IEnumerator<System.Collections.Generic.KeyValuePair<double,ValuedDocument>> ide = m_SortedList.GetEnumerator();
            list.Clear();
            list.Capacity = m_SortedList.Count;
            while (ide.MoveNext())
            {
                ValuedDocument doc = ide.Current.Value;
                list.Add(doc);
            }

            m_Prepared = true;
        }





        public void Clear()
        {
            m_SortedList.Clear();
            list.Clear();
            m_Prepared = false;
        }

        public int Capacity
        {
            set
            {
                m_SortedList.Capacity = value;
                list.Capacity         = value;
            }
        }


        public void Add(IDocument obj, double value)
        {
            Add(obj,null,value);
        }


        public void Add(IDocument obj, string s, double value)
        {
            m_Prepared = false;

            ValuedDocument doc = new ValuedDocument(obj,s,value);
            m_SortedList.Add(value,doc);
        }




        public ValuedDocument this[int i]
        {
            get
            {
                Prepare();
                if ((i >= 0) && (i < list.Count))
                    return list[i];
                else
                    return null;
            }
        }



        public void SetMarks(bool value)
        {
            Prepare();
           
            for (int i = 0; i < list.Count; i++)
            {
                list[i].Mark = value;
            }
        }




    }
}
