// 11.12.2011
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace Fusym
{
    public class VioletRule
    {
        public string LeftSide;
        public List<string> RightSide = new List<string>();
        public string Value;
        
        public override string ToString()
        {
            string result = LeftSide + " := ";
            for (int i = 0; i < RightSide.Count; i++)
            {
                result += RightSide[i];
                if (i < RightSide.Count - 1)
                    result += " ";
            }

            return result;
        }
    }

    public class Violet
    {
        private static string m_RegexRuleString = @"^\s*(?<leftside>[^\s]+)\s+:=(\s+(?<symbol>[^\s]+))*.*?$";
        private static Regex m_RegexRule = new Regex(m_RegexRuleString);
        public List<VioletRule> Rules = new List<VioletRule>();
        private Random m_Rnd = new Random();

        public Violet()
        {
            Load();
        }

        private void Load()
        {
            string fn = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\violet.txt";
            string[] lines = File.ReadAllLines(fn);

            Rules.Clear();
            foreach (string s in lines)
            {
                VioletRule rule = ParseLine(s);
                if (rule != null)
                    Rules.Add(rule);
            }
        }

        private VioletRule ParseLine(string s)
        {
            Match m = m_RegexRule.Match(s);
            if (m.Success)
            {
                VioletRule result = new VioletRule();

                result.LeftSide = m.Groups["leftside"].Value.ToUpper();

                Group g = m.Groups["symbol"];
                CaptureCollection cc = g.Captures;
                for (int j = 0; j < cc.Count; j++)
                {
                    Capture c = cc[j];
                    result.RightSide.Add(c.Value.ToUpper());
                }

                return result;
            }
            else
                return null;
        }



        public void Run()
        {
            List<VioletRule> rules = new List<VioletRule>(Rules);
            for (int i = 0; i < rules.Count; i++)
            {
                int r = m_Rnd.Next(rules.Count);
                VioletRule h = rules[i];
                rules[i] = rules[r];
                rules[r] = h;
            }



            for (int i = 0; i < rules.Count; i++)
            {
                VioletRule rule = rules[i];


                List<string> sl = new List<string>();
                for (int j = 0; j < rule.RightSide.Count; j++)
                {
                    string s = rule.RightSide[j];
                    if (!string.IsNullOrEmpty(s) && (s[0] == '#'))
                        s = FindValue(s);

                    if (! string.IsNullOrEmpty(s))
                       sl.Add(s);
                }


                rule.Value = Search(sl);
            }
        }

        private string FindValue(string s)
        {
            foreach (VioletRule rule in Rules)
                if (rule.LeftSide == s)
                    return rule.Value;
            return "";
        }


        private string Search(List<string> inputSymbols)
        {
            int maxSym   = 1000000;
            int maxDepth = 5;

            GlobalClusterManager gm = GlobalClusterManager.GetInstance();
            RandomWalk rw = new RandomWalk(gm.SemanticNetwork);
            List<Node> nodes = rw.SearchSymbol(inputSymbols, maxSym, maxDepth);

            if (nodes.Count > 0)
                return nodes[0].ToString();
            else
                return "";
        }













    }
}
