//------------------------------------------------------------------------------
// Purpose: 
// Date   : 29.Jun.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using System.Windows.Forms;
using System.Drawing;
using Fusym;

namespace Unarch5
{

    ///-------------------------------------------------------------------------
    /// <summary>
    /// A client of the Blosp databases services.
    /// </summary>
    ///-------------------------------------------------------------------------
    public interface IBlospConfigurationFileClient
    {
     void Load(BlospDatabase db);
     void Save(BlospDatabase db);
    }

#region Classes for each form.

    


    //--------------------------------------------------------------------------
    // Classes for each form.
    //--------------------------------------------------------------------------
    public class FormOptionenSettingsClass : IBlospConfigurationFileClient
    {
        private Size  m_Size     = new Size(641, 548);
        private Point m_Location = new Point(100, 100);


        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormOptionen");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormOptionen");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }
    }
    
    
    
    //--------------------------------------------------------------------------
    public class FormMainSettingsClass : IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(800, 500);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }

        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormMain");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormMain");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }


    }
    //--------------------------------------------------------------------------
    public class FormGraphSettingsClass : IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(800, 500);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormGraph");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormGraph");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }
        

    }
    //--------------------------------------------------------------------------
    public class FormStandardEditorSettingsClass: IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(800, 500);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormStandardEditor");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormStandardEditor");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }
        

    }
    //--------------------------------------------------------------------------
    public class FormStandardSettingsClass : IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(800, 500);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormStandard");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormStandard");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }        

    }
    //--------------------------------------------------------------------------
    public class FormSearchSymbolSettingsClass : IBlospConfigurationFileClient
    {
        private Size  m_Size = new Size(800, 500);
        private Point m_Location = new Point(100, 100);
        private int   m_MaxDepth = 3;
        private int   m_MaxSym = 10000;
        private bool  m_DisplayValue      = false;
        private bool  m_NeighbourCount = false;
        private bool  m_GroupChange = false;
        private int   m_EvaluationMethod = 2;

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }

        public int MaxDepth
        {
            get { return m_MaxDepth; }
            set { m_MaxDepth = value; }
        }

        public int MaxSym
        {
            get { return m_MaxSym; }
            set { m_MaxSym = value; }
        }

        public bool DisplayValue
        {
            get { return m_DisplayValue; }
            set { m_DisplayValue = value; }
        }

        public bool NeighbourCount
        {
            get { return m_NeighbourCount; }
            set { m_NeighbourCount = value; }
        }

        public bool GroupChange
        {
            get { return m_GroupChange; }
            set { m_GroupChange = value; }
        }

        public int EvaluationMethod
        {
            get { return m_EvaluationMethod; }
            set { m_EvaluationMethod = value; }
        }
        
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchSymbol");
            m_Size.Width       = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height      = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X       = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y       = be.SlotAsInteger("Top",m_Location.Y);

            m_MaxDepth         = be.SlotAsInteger("MaxDepth",m_MaxDepth);
            m_MaxSym           = be.SlotAsInteger("MaxSym",m_MaxSym);
            m_DisplayValue     = be.SlotAsBool("DisplayValue",m_DisplayValue);
            m_NeighbourCount   = be.SlotAsBool("NeighbourCount",m_NeighbourCount);
            m_GroupChange      = be.SlotAsBool("GroupChange",m_GroupChange);
            m_EvaluationMethod = be.SlotAsInteger("EvaluationMethod",m_EvaluationMethod);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchSymbol");
            be.SetSlotValue("Width"           ,m_Size.Width);
            be.SetSlotValue("Height"          ,m_Size.Height);
            be.SetSlotValue("Left"            ,m_Location.X);
            be.SetSlotValue("Top"             ,m_Location.Y);
            be.SetSlotValue("MaxDepth"        ,m_MaxDepth);
            be.SetSlotValue("MaxSym"          ,m_MaxSym);
            be.SetSlotValue("DisplayValue"    ,m_DisplayValue);
            be.SetSlotValue("NeighbourCount"  ,m_NeighbourCount);
            be.SetSlotValue("GroupChange"     ,m_GroupChange);
            be.SetSlotValue("EvaluationMethod",m_EvaluationMethod);
        }         
    }
    //--------------------------------------------------------------------------
    public class FormSearchAssociativeSettingsClass : IBlospConfigurationFileClient
    {
        private Point m_Location = new Point(100, 100);
        private int   m_MaxDepth = 7;
        private int   m_MaxSym = 100000;
        private int   m_EvaluationMethod = 2;
        private bool  m_FileYieldsAssociations = true;
        private bool  m_FullTextSearch = true;

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }

        public int MaxDepth
        {
            get { return m_MaxDepth; }
            set { m_MaxDepth = value; }
        }

        public int MaxSym
        {
            get { return m_MaxSym; }
            set { m_MaxSym = value; }
        }

        public int EvaluationMethod
        {
            get { return m_EvaluationMethod; }
            set { m_EvaluationMethod = value; }
        }

        public bool FileYieldsAssociations
        {
            get { return m_FileYieldsAssociations; }
            set { m_FileYieldsAssociations = value; }
        }

        public bool FullTextSearch
        {
            get { return m_FullTextSearch; }
            set { m_FullTextSearch = value; }
        }

        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchAssociative");
            m_Location.X             = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y             = be.SlotAsInteger("Top",m_Location.Y);
            m_MaxDepth               = be.SlotAsInteger("MaxDepth",m_MaxDepth);
            m_MaxSym                 = be.SlotAsInteger("MaxSym",m_MaxSym);
            m_EvaluationMethod       = be.SlotAsInteger("EvaluationMethod",m_EvaluationMethod);
            m_FileYieldsAssociations = be.SlotAsBool("FileYieldsAssociations", m_FileYieldsAssociations);
            m_FullTextSearch         = be.SlotAsBool("FullTextSearch", m_FullTextSearch);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchAssociative");
            be.SetSlotValue("Left"                  ,m_Location.X);
            be.SetSlotValue("Top"                   ,m_Location.Y);
            be.SetSlotValue("MaxDepth"              ,m_MaxDepth);
            be.SetSlotValue("MaxSym"                ,m_MaxSym);
            be.SetSlotValue("EvaluationMethod"      ,m_EvaluationMethod);
            be.SetSlotValue("FileYieldsAssociations", m_FileYieldsAssociations);
            be.SetSlotValue("FullTextSearch"        , m_FullTextSearch);
        }          
    }

    //--------------------------------------------------------------------------
    public class FormUnknownKeywordsSettingsClass : IBlospConfigurationFileClient
    {
        private Point m_Location = new Point(100, 100);

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormUnknownKeywords");
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top", m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormUnknownKeywords");
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }         
    }

    //--------------------------------------------------------------------------
    public class FormConceptSettingsClass : IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(500, 300);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }

        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormConcept");
            m_Size.Width = be.SlotAsInteger("Width", m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height", m_Size.Height);
            m_Location.X = be.SlotAsInteger("Left", m_Location.X);
            m_Location.Y = be.SlotAsInteger("Top", m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormConcept");
            be.SetSlotValue("Width", m_Size.Width);
            be.SetSlotValue("Height", m_Size.Height);
            be.SetSlotValue("Left", m_Location.X);
            be.SetSlotValue("Top", m_Location.Y);
        }
    }

    //--------------------------------------------------------------------------
    public class FormSearchFilenameSettingsClass : IBlospConfigurationFileClient
    {
        private Size m_Size = new Size(500, 300);
        private Point m_Location = new Point(100, 100);

        public Size Size
        {
            get { return m_Size; }
            set { m_Size = value; }
        }

        public Point Location
        {
            get { return m_Location; }
            set { m_Location = value; }
        }
        
        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchFilename");
            m_Size.Width  = be.SlotAsInteger("Width",m_Size.Width);
            m_Size.Height = be.SlotAsInteger("Height",m_Size.Height);
            m_Location.X  = be.SlotAsInteger("Left",m_Location.X);
            m_Location.Y  = be.SlotAsInteger("Top",m_Location.Y);
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FormSearchFilename");
            be.SetSlotValue("Width" ,m_Size.Width);
            be.SetSlotValue("Height",m_Size.Height);
            be.SetSlotValue("Left"  ,m_Location.X);
            be.SetSlotValue("Top"   ,m_Location.Y);
        }         
    }

    
    //--------------------------------------------------------------------------

    public class FileHistoryClass : IBlospConfigurationFileClient
    {
        private List<string> m_History = new List<string>();

        public List<string> History
        {
            get { return m_History; }
            set { m_History = value; }
        }


        public void Load(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FileHistory");
            
            List <string> temp = be.SlotAsMemo("History");
            m_History.Clear();
            m_History.AddRange(temp);
            
        }

        public void Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("FileHistory");
            be.SetSlotValue("History",m_History);
        }   

    }









#endregion



    //--------------------------------------------------------------------------
    /// <summary>
    /// This class represents the configuration file. It has a singleton,
    /// which is serialized into and from XML.
    /// </summary>
    //--------------------------------------------------------------------------
    public class Configuration : IBlospConfigurationFileClient
    {
        private static Configuration instance;

//      public static Encoding encoding = System.Text.Encoding.UTF8;
//      public static Encoding encoding = System.Text.Encoding.GetEncoding("ISO-8859-1");
        public static Encoding encoding = System.Text.Encoding.GetEncoding(1252);
        private BlospDatabase m_DB = new BlospDatabase();



        //----------------------------------------------------------------------
        private FormOptionenSettingsClass          m_FormOptionenSettings          = new FormOptionenSettingsClass();
        private FormMainSettingsClass              m_FormMainSettings              = new FormMainSettingsClass();
        private FormGraphSettingsClass             m_FormGraphSettings             = new FormGraphSettingsClass();
        private FormStandardEditorSettingsClass    m_FormStandardEditorSettings    = new FormStandardEditorSettingsClass();
        private FormStandardSettingsClass          m_FormStandardSettings          = new FormStandardSettingsClass();
        private FormSearchSymbolSettingsClass      m_FormSearchSymbolSettings      = new FormSearchSymbolSettingsClass();
        private FormSearchAssociativeSettingsClass m_FormSearchAssociativeSettings = new FormSearchAssociativeSettingsClass();
        private FormUnknownKeywordsSettingsClass   m_FormUnknownKeywordsSettings   = new FormUnknownKeywordsSettingsClass();
        private FormConceptSettingsClass           m_FormConceptSettings           = new FormConceptSettingsClass();
        private FormSearchFilenameSettingsClass    m_FormSearchFilenameSettings    = new FormSearchFilenameSettingsClass();
        private FileHistoryClass m_FileHistoryClass = new FileHistoryClass();
        //----------------------------------------------------------------------
       
        //--- Global settings --------------------------------------------------
        private int m_MaxFileHistory;
        private int m_MaxQueryHistory;
        private int m_MaxDirectoryHistory;
        private List<string> m_KnownClusterFiles  = new List<string>();
        private List<string> m_ActiveClusterFiles = new List<string>();
        private List<string> m_History            = new List<string>();
        private List<string> m_Directories        = new List<string>();
        private bool      m_AskBeforeDelete       = true;
        private bool      m_DisplayDistribution   = true;
        private bool      m_GotoLastRecord        = true;
        private bool      m_ClassicalDatabaseView = false;
        private bool      m_UseSpecialBrowser     = false;
        private string    m_HttpBrowserExe        = "";
        //----------------------------------------------------------------------

        public BlospDatabase DB
        {
            get { return m_DB; }
        }

        //----------------------------------------------------------------------

        public List<string> History
        {
            get { return m_History; }
            set { m_History = value; }
        }

        public List<string> KnownClusterFiles
        {
            get { return m_KnownClusterFiles; }
        }

        public List<string> ActiveClusterFiles
        {
            get { return m_ActiveClusterFiles; }
        }

        public List<string> Directories
        {
            get { return m_Directories; }
            set { m_Directories = value; }
        }
        //----------------------------------------------------------------------

        public FormOptionenSettingsClass FormOptionenSettings
        {
            get { return m_FormOptionenSettings; }
            set { m_FormOptionenSettings = value; }
        }

        public FormMainSettingsClass FormMainSettings
        {
            get { return m_FormMainSettings; }
            set { m_FormMainSettings = value; }
        }

        public FormGraphSettingsClass FormGraphSettings
        {
            get { return m_FormGraphSettings; }
            set { m_FormGraphSettings = value; }
        }

        public FormStandardEditorSettingsClass FormStandardEditorSettings
        {
            get { return m_FormStandardEditorSettings; }
            set { m_FormStandardEditorSettings = value; }
        }

        public FormStandardSettingsClass FormStandardSettings
        {
            get { return m_FormStandardSettings; }
            set { m_FormStandardSettings = value; }
        }

        public FormSearchSymbolSettingsClass FormSearchSymbolSettings
        {
            get { return m_FormSearchSymbolSettings; }
            set { m_FormSearchSymbolSettings = value; }
        }

        public FormSearchAssociativeSettingsClass FormSearchAssociativeSettings
        {
            get { return m_FormSearchAssociativeSettings; }
            set { m_FormSearchAssociativeSettings = value; }
        }

        public FormUnknownKeywordsSettingsClass FormUnknownKeywordsSettings
        {
            get { return m_FormUnknownKeywordsSettings; }
            set { m_FormUnknownKeywordsSettings = value; }
        }



        public FormConceptSettingsClass FormConceptSettings
        {
            get { return m_FormConceptSettings; }
            set { m_FormConceptSettings = value; }
        }

        public FormSearchFilenameSettingsClass FormSearchFilenameSettings
        {
            get { return m_FormSearchFilenameSettings; }
            set { m_FormSearchFilenameSettings = value; }
        }

        public FileHistoryClass FileHistorySettings
        {
            get { return m_FileHistoryClass; }
            set { m_FileHistoryClass = value; }
        }




        public int MaxFileHistory
        {
            get { return m_MaxFileHistory; }
            set { m_MaxFileHistory = value; }
        }

        public int MaxQueryHistory
        {
            get { return m_MaxQueryHistory; }
            set { m_MaxQueryHistory = value; }
        }

        public int MaxDirectoryHistory
        {
            get { return m_MaxDirectoryHistory; }
            set { m_MaxDirectoryHistory = value; }
        }

        public bool GotoLastRecord
        {
            get { return m_GotoLastRecord; }
            set { m_GotoLastRecord = value; }
        }


        public bool ClassicalDatabaseView
        {
            get { return m_ClassicalDatabaseView; }
            set { m_ClassicalDatabaseView = value; }
        }

        public bool UseSpecialBrowser
        {
            get { return m_UseSpecialBrowser; }
            set { m_UseSpecialBrowser = value; }
        }

        public string HttpBrowserExe
        {
            get { return m_HttpBrowserExe; }
            set { m_HttpBrowserExe = value; }
        }


        public bool AskBeforeDelete
        {
            get { return m_AskBeforeDelete; }
            set { m_AskBeforeDelete = value; }
        }
	
        public bool DisplayDistribution
        {
            get { return m_DisplayDistribution; }
            set { m_DisplayDistribution = value; }
        }
	



        
        private void Load()
        {
            string filename = Path.GetDirectoryName(Application.ExecutablePath) + @"\unarch.cfg";
            
            try
            {
                m_DB.LoadFromFile(filename);
                
                LoadFromBlospDatabase(m_FormOptionenSettings);
                LoadFromBlospDatabase(m_FormMainSettings);
                LoadFromBlospDatabase(m_FormGraphSettings);
                LoadFromBlospDatabase(m_FormStandardEditorSettings);
                LoadFromBlospDatabase(m_FormStandardSettings);
                LoadFromBlospDatabase(m_FormUnknownKeywordsSettings);
                LoadFromBlospDatabase(m_FormConceptSettings);
                LoadFromBlospDatabase(m_FormSearchFilenameSettings);
                LoadFromBlospDatabase(m_FormSearchAssociativeSettings);
                LoadFromBlospDatabase(m_FormSearchSymbolSettings);
                LoadFromBlospDatabase(m_FileHistoryClass);
                LoadFromBlospDatabase(this);                
            }
            catch (FileNotFoundException)
            {
            }
        }
        
        
        //----------------------------------------------------------------------
        /// <summary>
        /// Must be called once when the program is about to be closed.
        /// </summary>
        //----------------------------------------------------------------------
        public void Save()
        {
            string filename = Path.GetDirectoryName(Application.ExecutablePath) + @"\unarch.cfg";

            SaveToBlospDatabase(m_FormOptionenSettings);
            SaveToBlospDatabase(m_FormMainSettings);
            SaveToBlospDatabase(m_FormGraphSettings);
            SaveToBlospDatabase(m_FormStandardEditorSettings);
            SaveToBlospDatabase(m_FormStandardSettings);
            SaveToBlospDatabase(m_FormUnknownKeywordsSettings);
            SaveToBlospDatabase(m_FormConceptSettings);
            SaveToBlospDatabase(m_FormSearchFilenameSettings);
            SaveToBlospDatabase(m_FormSearchAssociativeSettings);
            SaveToBlospDatabase(m_FormSearchSymbolSettings);
            SaveToBlospDatabase(m_FileHistoryClass);
            SaveToBlospDatabase(this);
            
            m_DB.SaveToFile(filename);            
        }        
        
        private void SaveToBlospDatabase(IBlospConfigurationFileClient f)
        {
            f.Save(m_DB);
        }
        
        private void LoadFromBlospDatabase(IBlospConfigurationFileClient f)
        {
            f.Load(m_DB);
        }

        

        static public Configuration GetInstance()
        {
            if (instance == null)
               {
                instance = new Configuration();              
                instance.Load();
               }
            
            return instance;
        }


        //--------------------------------------------------------------------------

        public void RestrictArrayListLength(List<string> lis, int length)
        {
            while (lis.Count > length)
                lis.RemoveAt(lis.Count - 1);
        }


        void IBlospConfigurationFileClient.Load(BlospDatabase db)
        {
            List<string> temp;
        
            BlospExpression be = db.FindOrCreate("Global");
            m_MaxFileHistory        = be.SlotAsInteger("MaxFileHistory",m_MaxFileHistory);
            m_MaxQueryHistory       = be.SlotAsInteger("MaxQueryHistory", m_MaxQueryHistory);
            m_MaxDirectoryHistory   = be.SlotAsInteger("MaxDirectoryHistory", m_MaxDirectoryHistory);
            m_AskBeforeDelete       = be.SlotAsBool("AskBeforeDelete", m_AskBeforeDelete);
            m_DisplayDistribution   = be.SlotAsBool("DisplayDistribution",m_DisplayDistribution);
            m_GotoLastRecord        = be.SlotAsBool("GotoLastRecord",m_GotoLastRecord);
            m_ClassicalDatabaseView = be.SlotAsBool("ClassicalDatabaseView",m_ClassicalDatabaseView);
            m_UseSpecialBrowser     = be.SlotAsBool("UseSpecialBrowser",m_UseSpecialBrowser);
            m_HttpBrowserExe        = be.SlotAsString("HttpBrowserExe",m_HttpBrowserExe);
            
            temp = be.SlotAsMemo("KnownClusterFiles");
            m_KnownClusterFiles.Clear();
            m_KnownClusterFiles.AddRange(temp);
            
            temp = be.SlotAsMemo("ActiveClusterFiles");
            m_ActiveClusterFiles.Clear();
            m_ActiveClusterFiles.AddRange(temp);

            temp = be.SlotAsMemo("History");
            m_History.Clear();
            m_History.AddRange(temp);

            temp = be.SlotAsMemo("Directories");
            m_Directories.Clear();
            m_Directories.AddRange(temp);
        }

        void IBlospConfigurationFileClient.Save(BlospDatabase db)
        {
            BlospExpression be = db.FindOrCreate("Global");
            be.SetSlotValue("MaxFileHistory"       , m_MaxFileHistory);
            be.SetSlotValue("MaxQueryHistory"      , m_MaxQueryHistory);
            be.SetSlotValue("MaxDirectoryHistory"  , m_MaxDirectoryHistory);
            be.SetSlotValue("AskBeforeDelete"      , m_AskBeforeDelete);
            be.SetSlotValue("DisplayDistribution"  , m_DisplayDistribution);
            be.SetSlotValue("GotoLastRecord"       , m_GotoLastRecord);
            be.SetSlotValue("ClassicalDatabaseView", m_ClassicalDatabaseView);
            be.SetSlotValue("UseSpecialBrowser"    , m_UseSpecialBrowser);
            be.SetSlotValue("HttpBrowserExe"       , m_HttpBrowserExe);
            be.SetSlotValue("KnownClusterFiles"    , m_KnownClusterFiles);            
            be.SetSlotValue("ActiveClusterFiles"   , m_ActiveClusterFiles);
            be.SetSlotValue("History"              , m_History);
            be.SetSlotValue("Directories"          , m_Directories);
        }           
    }
}
