// 6.12.2009
// Joachim Pimiskern
using System;
using System.Collections;
using System.Text;
using System.Windows.Forms;
using System.Collections.Generic;

namespace Unarch5
{
    public interface IFileHistoryReceiver
    {
        void AddFileToHistory(string filename);
    }

    class FileHistory
    {
        Queue q = new Queue();
        ToolStripMenuItem m_Parent;
        ToolStripItem     m_Start;
        ToolStripItem     m_End;
        EventHandler      m_OnClick;
        int               m_MaxCount = 3;

        public FileHistory(
                           ToolStripMenuItem parent, 
                           ToolStripItem     start, 
                           ToolStripItem     end,
                           EventHandler      onClick
                          )
        {
            m_Parent     = parent;
            m_Start      = start;
            m_End        = end;
            m_OnClick    = onClick;
        }


        public int MaxCount
        {
            get { return m_MaxCount; }
            set { m_MaxCount = value;}
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Remove menu items between m_Start and m_End.
        /// </summary>
        ///---------------------------------------------------------------------
        public void ClearMenu()
        {
            int m_StartIndex = m_Parent.DropDownItems.IndexOf(m_Start);
            int m_EndIndex   = m_Parent.DropDownItems.IndexOf(m_End);

            for (int i = m_EndIndex - 1; i > m_StartIndex; i--)
                 m_Parent.DropDownItems.RemoveAt(i);
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Put a filename into the internal datastructure q.
        /// </summary>
        /// <param name="filename">A filename</param>
        ///---------------------------------------------------------------------
        public void Add(string filename)
        {
            if (!q.Contains(filename))
                q.Enqueue(filename);

            while (q.Count > m_MaxCount)
               q.Dequeue();

            ToMenu();
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Add a list of filenames to the internal data structure q.
        /// </summary>
        /// <param name="filenames">List of filename strings</param>
        ///---------------------------------------------------------------------
        public void Add(List<string> filenames)
        {
            q.Clear();

            foreach (string s in filenames)
                if (!q.Contains(s))
                    q.Enqueue(s);

            while (q.Count > m_MaxCount)
                q.Dequeue();

            ToMenu();
        }


        public List<string> ToStringList()
        {
            object[] list = q.ToArray();
            
            List<string> result = new List<string>();
           
            foreach (string s in list)
                result.Add(s);
            
            return result;
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Transfer the file history from the internal
        /// data structure q into the menu.
        /// </summary>
        ///---------------------------------------------------------------------
        public void ToMenu()
        {
            List<string> list = new List<string>();

            ClearMenu();

            foreach (string s in q)
                list.Add(s);


            for (int i = 0; i < list.Count; i++)
            {
                string s = list[list.Count - i - 1];
                ToolStripMenuItem newEntry = new ToolStripMenuItem("&" + i.ToString() + " " + s, null, m_OnClick);
                newEntry.Tag = s;
                m_Parent.DropDownItems.Insert(m_Parent.DropDownItems.IndexOf(m_End), newEntry);
            }
        }
    }
}
