//------------------------------------------------------------------------------
// Purpose: List of filenames as documents.
// Date   : 05.Dec.2009
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Fusym;
using System.Data;


namespace Unarch5
{
    public class FilenameDocuments : IDocumentList
    {
        private string m_Path;
        private List<string> m_Filenames = new List<string>();
        private int m_Idx;

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public FilenameDocuments(string path)
        {
            m_Path = path;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Reads a directory recursively. Triggered by the access
        /// to the first document.
        /// </summary>
        ///---------------------------------------------------------------------
        private List<string> ReadDirectoryRecursive(string startdir)
        {
            List<string> result = new List<string>();
            ReadDirectoryRecursive_Helper(result, startdir);
            return result;
        }


        private void ReadDirectoryRecursive_Helper(List<string> list, string startdir)
        {
            DirectoryInfo di;

            try
            {
                di = new DirectoryInfo(startdir);
            }
            catch(ArgumentException)
            {
                return;
            }

            try
            {

                foreach (FileInfo fi in di.GetFiles())
                {
                    list.Add(fi.FullName);
                }
            }
            catch
            {
            }

            try
            {
                foreach (DirectoryInfo subDir in di.GetDirectories())
                {
                    list.Add(subDir.FullName);
                    ReadDirectoryRecursive_Helper(list, subDir.FullName);
                }
            }
            catch
            {
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Interface function.
        /// </summary>
        ///---------------------------------------------------------------------
        public IDocument GetFirstDocument()
        {
            // Preparation
            m_Filenames = ReadDirectoryRecursive(m_Path);

            m_Idx = 0;
            return GetNextDocument();
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Interface function.
        /// </summary>
        ///---------------------------------------------------------------------
        public IDocument GetNextDocument()
        {
            if (m_Idx < m_Filenames.Count)
               {
                    StructuredString s = new StructuredString(m_Filenames[m_Idx++], null);
                    return s;
               }
            else
                return null;
        }


    }
}
