//------------------------------------------------------------------------------
// Purpose: Organizes the search for filenames.
// Date   : 15.Sep.2010
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using Fusym;
using System.Text.RegularExpressions;

namespace Unarch5
{
    public class FilenameSearchCustomization : ISearchProgressReceiver
    {
        IDocumentList m_DocumentList;
        ValuedDocList m_ValuedDocList;
        string        m_Query;


        public FilenameSearchCustomization(
                                           IDocumentList documentlist, 
                                           ValuedDocList valuedDocList,
                                           string query
                                          )
        {
            m_DocumentList  = documentlist;
            m_ValuedDocList = valuedDocList;
            m_Query         = query.Trim().ToUpper();
        }





        public void Search()
        {
            int maxSym   = 10000;
            int maxDepth = 5;


            //--- Semantic network -----------------------------------------
            SemanticNetwork semanticNetwork = new SemanticNetwork(Configuration.encoding);


            LoadClusterFiles(semanticNetwork);
            //--- DocumentEvaluator ----------------------------------------


            List<string> inputSymbols = GetQuerySymbols(m_Query);

            EvaluationMethod evm = EvaluationMethod.CloudMatching;



            IDocumentEvaluator documentEvaluator = semanticNetwork.NewDocumentEvaluator(
                                                                          inputSymbols,
                                                                          maxDepth,
                                                                          maxSym,
                                                                          evm,
                                                                          true
                                                                         );




            //--------------------------------------------------------------


            SearchManager sm = new SearchManager(
                                                 this,
                                                 m_DocumentList,
                                                 m_ValuedDocList,
                                                 documentEvaluator,
                                                 semanticNetwork
                                                );


            sm.Run();


        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load all cluster files that are mentioned in the configuration 
        /// settings as active cluster files.
        /// </summary>
        ///---------------------------------------------------------------------
        private void LoadClusterFiles(SemanticNetwork sn)
        {
            Configuration c = Configuration.GetInstance();
            List<string> activeFiles = c.ActiveClusterFiles;

            foreach (string filename in activeFiles)
            {
                try
                {
                    sn.LoadFromFile(filename);
                }
                catch
                {
                }
            }
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Get the words of the query.
        /// </summary>
        ///---------------------------------------------------------------------
        private List<string> GetQuerySymbols(string s)
        {
            List<string> result = new List<string>();

            s = s.ToUpper();

            Regex r = new Regex(@"[^\s]+");
            MatchCollection m = r.Matches(s);
            foreach (Match ma in m)
            {
                string sym = ma.ToString();
                result.Add(sym);
            }

            return result;
        }


        public void SetCountOfDocuments(int count)
        {
        }

        public void SetIndexOfDocument(int idx)
        {
        }
    }
}
