using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using Fusym;

namespace Unarch5
{
    public partial class FormClustersOfSymbols : Form
    {
        private SemanticNetwork m_SemanticNetwork;

        public FormClustersOfSymbols()
        {
            InitializeComponent();

            m_SemanticNetwork = GlobalClusterManager.GetInstance().SemanticNetwork;
            textBoxMaxCount.Text = m_SemanticNetwork.NodeList.Count.ToString();
            textBoxMaxDepth.Text = "20";
            textBoxClusterCount.Text = "2";
        }

        private void buttonSearch_Click(object sender, EventArgs e)
        {
            Run();
        }

        private List<string> GetQuerySymbols(string s)
        {
            List<string> result = new List<string>();

            s = s.ToUpper();

            Regex r = new Regex(@"[^\s]+");
            MatchCollection m = r.Matches(s);
            foreach (Match ma in m)
            {
                string sym = ma.ToString();
                if (! result.Contains(sym))
                   result.Add(sym);
            }

            return result;
        }



        private void Run()
        {
            RandomWalk rw = new RandomWalk(m_SemanticNetwork);

            List<string> symbols = GetQuerySymbols(textBoxInput.Text);
            int maxCount;
            try
            {
                maxCount = int.Parse(textBoxMaxCount.Text);
            }
            catch
            {
                maxCount = m_SemanticNetwork.NodeList.Count;
                textBoxMaxCount.Text = maxCount.ToString();
            }

            int maxDepth;
            try
            {
                maxDepth = int.Parse(textBoxMaxDepth.Text);
            }
            catch
            {
                maxDepth = 20;
                textBoxMaxDepth.Text = maxDepth.ToString();
            }

            int maxClusters;
            try
            {
                maxClusters = int.Parse(textBoxClusterCount.Text);
            }
            catch
            {
                maxClusters = 2;
                textBoxClusterCount.Text = maxClusters.ToString();
            }







            List<List<Node>> lists = rw.Clustering(symbols, maxCount, maxDepth, maxClusters);

            textBoxClusters.Text = "";
            foreach (List<Node> nodeList in lists)
            {
                string s = "";
                foreach (Node n in nodeList)
                    s += n.ToString() + " ";
                textBoxClusters.Text += s + Environment.NewLine;
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormClustersOfSymbols_Load(object sender, EventArgs e)
        {

        }
    }
}