//------------------------------------------------------------------------------
// Purpose: Input form for concepts in BLOSP 
// Date   : 28.Jul.2007
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;
using Fusym;
using Symps;
using System.IO;

namespace Unarch5
{
    public partial class FormConcept : Form
    {
        // Koennte man mal zentralisieren.
        public static Encoding encoding = System.Text.Encoding.GetEncoding(1252);


        BlospDatabase knowledgebase = new BlospDatabase();
        ValuedDocList m_ValuedDocList = new ValuedDocList();
        DataSetState m_State;
        int current = 0;
        private bool m_Modified;
        List<string> m_Record = new List<string>();
        private string m_Query;



        private enum DataSetState
        {
            dsBrowsing,
            dsEditing,
            dsInsert
        }


        public string Query
        {
            get { return m_Query; ; }
            set { m_Query = value ; }
        }


        public FormConcept()
        {
            InitializeComponent();

        }

        public bool Modified
        {
            get { return m_Modified; }
            set
            {
                if (value == true)
                {
                    buttonStoreFile.Enabled = true;
                }
                else
                {
                    buttonStoreFile.Enabled = false;
                }

                m_Modified = value;
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Actions when state changes.
        /// </summary>
        /// <param name="state"></param>
        ///---------------------------------------------------------------------
        private void SetState(DataSetState state)
        {
            m_State = state;
            if (m_State == DataSetState.dsBrowsing)
            {
                buttonFirst.Enabled  = true;
                buttonPrev.Enabled   = true;
                buttonNext.Enabled   = true;
                buttonLast.Enabled   = true;
                buttonEdit.Enabled   = true;
                buttonInsert.Enabled = true;
                buttonDelete.Enabled = true;
                buttonStore.Enabled  = false;
                buttonCancel.Enabled = false;
                checkNavigationButtons();

                /*
                textBoxInfo.ReadOnly = true;
                textBoxInfo.BackColor = Color.White;
                 */
            }
            else
                if ((m_State == DataSetState.dsEditing) || (m_State == DataSetState.dsInsert))
                {
                    buttonFirst.Enabled = false;
                    buttonPrev.Enabled = false;
                    buttonNext.Enabled = false;
                    buttonLast.Enabled = false;
                    buttonEdit.Enabled = false;
                    buttonInsert.Enabled = false;
                    buttonDelete.Enabled = false;
                    buttonStore.Enabled = true;
                    buttonCancel.Enabled = true;
                    //                    textBoxInfo.ReadOnly = false;
                }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load the record that corresponds to the current index
        /// </summary>
        ///---------------------------------------------------------------------
        private void ApplyCurrent()
        {
            if ((current >= 0) && (current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[current];
                textBoxInput.Text = ((BlospExpression)vdoc.Object).ToString();
            }
            else
                textBoxInput.Clear();

            checkNavigationButtons();
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// This function must be called in browsing state only.
        /// </summary>
        ///---------------------------------------------------------------------
        private void checkNavigationButtons()
        {
            if (current <= 0)
            {
                buttonFirst.Enabled = false;
                buttonPrev.Enabled = false;
            }
            else
            {
                buttonFirst.Enabled = true;
                buttonPrev.Enabled = true;
            }

            if (current >= m_ValuedDocList.Count - 1)
            {
                buttonNext.Enabled = false;
                buttonLast.Enabled = false;
            }
            else
            {
                buttonNext.Enabled = true;
                buttonLast.Enabled = true;
            }
        }















        ///---------------------------------------------------------------------
        /// <summary>
        /// Make sure that all records of lrf are in m_ValuedDocList. Value = index
        /// </summary>
        ///---------------------------------------------------------------------
        private void SelectAll()
        {

            m_ValuedDocList.Clear();

            for (int i = 0; i < knowledgebase.Count; i++)
            {
                double value = i;
                m_ValuedDocList.Add(knowledgebase[i], value);
            }

            current = 0;
            ApplyCurrent();
        }


        private void buttonFirst_Click(object sender, EventArgs e)
        {
            current = 0;
            ApplyCurrent();
        }

        private void buttonPrev_Click(object sender, EventArgs e)
        {
            current--;
            if (current < 0)
                current = 0;
            ApplyCurrent();
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            current++;
            if (current >= m_ValuedDocList.Count)
                current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }

        private void buttonLast_Click(object sender, EventArgs e)
        {
            current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }


        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }







        ///---------------------------------------------------------------------
        /// <summary>
        /// Parse the content of the textBoxInput into a data structure
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        ///---------------------------------------------------------------------
        private void buttonParse_Click(object sender, EventArgs e)
        {
            string[] theArray = Regex.Split(textBoxInput.Text, System.Environment.NewLine);
            List<string> sl = new List<string>(theArray);

            BlospParser icp = new BlospParser(sl);
            BlospExpression root = icp.Parse();



            foreach (BlospExpression element in root[0].Children)
            {
                MessageBox.Show(element.ToString());
                //MessageBox.Show(element.SlotAsString("m2"     ,"Default"));
                //MessageBox.Show(element.SlotAsString("epsilon","Default"));
            }


            /*
            foreach (ConceptSystemElement element in items)
            {
                foreach (ConceptSystemElement f in items)
                {
                    if ((element is ConceptList) && (f is Concept))
                    {
                        ConceptInferenceSystem cis = new ConceptInferenceSystem();
                        Concept c = cis.Transform(element as ConceptList, f as Concept);
                        MessageBox.Show(c.ToString());
                    }
                }
            }
             */
            

        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Read a stringlist.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        ///---------------------------------------------------------------------
        private void buttonLoad_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\";
            openFileDialog.FileName = "concepts.txt";
            openFileDialog.Filter = "BLOSP-Dateien (*.bsp)|*.bsp|Text-Dateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                knowledgebase.LoadFromFile(openFileDialog.FileName);
                SelectAll();
            }

        }


        private void buttonStoreFile_Click(object sender, EventArgs e)
        {
            string filename;

            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.InitialDirectory = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\";
                saveFileDialog.FileName = "concepts.txt";


                saveFileDialog.Filter = "BLOSP-Dateien (*.bsp)|*.bsp|Text-Dateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";

                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                    filename = saveFileDialog.FileName;
                else
                    return;
            }

            knowledgebase.SaveToFile(filename);
        }









        private void buttonInsert_Click(object sender, EventArgs e)
        {
            m_Record.Clear();
            textBoxInput.Clear();
            ActiveControl = textBoxInput;
            SetState(DataSetState.dsInsert);
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            if ((current >= 0) && (current < m_ValuedDocList.Count))
            {
                SetState(DataSetState.dsEditing);
                ActiveControl = textBoxInput;
            }

        }

        private void buttonFirst_Click_1(object sender, EventArgs e)
        {
            current = 0;
            ApplyCurrent();

        }

        private void buttonPrev_Click_1(object sender, EventArgs e)
        {
            current--;
            if (current < 0)
                current = 0;
            ApplyCurrent();
        }

        private void buttonNext_Click_1(object sender, EventArgs e)
        {
            current++;
            if (current >= m_ValuedDocList.Count)
                current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }

        private void buttonLast_Click_1(object sender, EventArgs e)
        {
            current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            //            Configuration c = Configuration.GetInstance();

            //            if (c.AskBeforeDelete)
            if (true)
            {
                DialogResult dialogResult = MessageBox.Show("Datensatz lschen", "Frage",
                                MessageBoxButtons.YesNo,
                                MessageBoxIcon.Question,
                                MessageBoxDefaultButton.Button1
                               );
                if (dialogResult == System.Windows.Forms.DialogResult.No)
                {
                    return;
                }
            }

            if ((current >= 0) && (current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[current];
                BlospExpression element = (BlospExpression)vdoc.Object;

                element.Delete();
                textBoxInput.Clear();

                Modified = true;
            }

        }

        private BlospExpression AddToKnowledgebase()
        {
            BlospParser icp = new BlospParser(m_Record);
            BlospExpression root = icp.Parse();
            BlospExpression result = null;

            foreach (BlospExpression be in root[0].Children)
            {
                knowledgebase.Add(be);
                result = be;
            }

            return result;
        }

        private void buttonStore_Click(object sender, EventArgs e)
        {
            ScreenToRecord();
            if (m_State == DataSetState.dsEditing)
            {
                ValuedDocument vdoc = m_ValuedDocList[current];
                if (vdoc != null)
                {
                    ((BlospExpression)vdoc.Object).Delete();
                    vdoc.Object = AddToKnowledgebase();
                }

                AddToKnowledgebase();
            }
            else
                if (m_State == DataSetState.dsInsert)
                {
                    AddToKnowledgebase();
                    SelectAll();
                }

            Modified = true;
            SetState(DataSetState.dsBrowsing);
        }




        private void ScreenToRecord()
        {
            string[] theArray = Regex.Split(textBoxInput.Text, System.Environment.NewLine);

            m_Record.Clear();
            foreach (string s in theArray)
                m_Record.Add(s);
        }




        private void buttonCancel_Click(object sender, EventArgs e)
        {
            SetState(DataSetState.dsBrowsing);
            //--- Restore old content ---
            ApplyCurrent();
        }



        private void button1_Click(object sender, EventArgs e)
        {
            BlospExpression.SearchConceptNames = checkBoxConceptNames.Checked;
            BlospExpression.SearchSlotNames    = checkBoxSlotNames.Checked;
            BlospExpression.SearchSlotValues   = checkBoxSlotValues.Checked;        
            
            using (FormSearchAssociative f = new FormSearchAssociative("AnyFile", knowledgebase, null, m_ValuedDocList))
            {
                f.ShowDialog();
                Query = f.Query;
                current = 0;
                ApplyCurrent();
            }
        }

        private void FormConcept_Load(object sender, EventArgs e)
        {
            LoadSettings();
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Load settings from the config file
        /// </summary>
        ///---------------------------------------------------------------------
        private void LoadSettings()
        {
            Configuration c = Configuration.GetInstance();
            this.Location   = c.FormConceptSettings.Location;
            this.Size       = c.FormConceptSettings.Size;
//            this.m_FileHistory.MaxCount = c.MaxFileHistory;
            //this.m_FileHistory.Add(c.FileHistorySettings.History);
//            this.SetClassicalDatabaseView(c.ClassicalDatabaseView);
        }


        private void StoreSettings()
        {
            Configuration c = Configuration.GetInstance();
            c.FormConceptSettings.Location = this.Location;
            c.FormConceptSettings.Size = this.Size;
        }


        private void FormConcept_FormClosed(object sender, FormClosedEventArgs e)
        {
            StoreSettings();
        }



        private void buttonPathToHere_Click(object sender, EventArgs e)
        {
            if ((current >= 0) && (current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[current];
                IDocument lrec = (IDocument)vdoc.Object;

                using (FormPathToHere f = new FormPathToHere())
                {
                    f.lrec = lrec;
                    f.Query = m_Query;
                    f.ShowDialog();
                }
            }
        }

        private void buttonSelectAll_Click(object sender, EventArgs e)
        {
            SelectAll();
            Query = "";
        }

        private void button1_Click_1(object sender, EventArgs e)
        {
            string s;

            BlospExpression b1 = knowledgebase.Add();
            b1.AddName("111111");

            BlospExpression b2 = new BlospExpression();
            b2.AddName("22222");

            BlospExpression b3 = new BlospExpression();
            b3.AddName("33333");

            BlospExpression b4 = new BlospExpression();
            b4.AddName("44444");

            BlospExpression b5 = new BlospExpression();
            b5.AddName("55555");





            b3.SetSlotValue("22222", b4);
            b3.SetSlotValue("22222", b5);
            b1.SetSlotValue("22222", b3);

            s = b1.ToString();
            MessageBox.Show(s);


            BlospExpression x = b1.FindSlotLine("22222");
            if (x != null)
                MessageBox.Show(x.ToString());
        }
    }
}