//------------------------------------------------------------------------------
// Purpose: 
// Date   : 16.Dec.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Fusym;

namespace Unarch5
{
    public partial class FormExport : Form
    {
        public string Query;
        private HtmlOptions m_OptionsForHtml = new HtmlOptions();
        public ValuedDocList DocList;

        public FormExport()
        {
            InitializeComponent();
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Initialize HTML options with reasonable default values.
        /// </summary>
        ///---------------------------------------------------------------------
        private void SetHtmlOptions()
        {
            m_OptionsForHtml.UseTitle           = true;
            m_OptionsForHtml.UseMetaDescription = true;
            m_OptionsForHtml.UseMetaKeywords    = true;
            m_OptionsForHtml.UseMetaLanguage    = true;
            m_OptionsForHtml.UseMetaRobots      = true;
            m_OptionsForHtml.UseHeadline        = true;

            m_OptionsForHtml.Title              = Query;
            m_OptionsForHtml.MetaDescription    = Query;
            m_OptionsForHtml.MetaKeywords       = Query;
            m_OptionsForHtml.MetaLanguage       = "de,en";
            m_OptionsForHtml.MetaRobots         = "index,follow";
            m_OptionsForHtml.Headline           = Query;
        }



        private int GetCountOfMarkedRecords()
        {
            int result = 0;

            for (int i = 0; i < DocList.Count; i++)
            {
                ValuedDocument vdoc = DocList[i];
                if (vdoc.Mark)
                   result++;
            }

            return result;
        }

        private void FormExport_Load(object sender, EventArgs e)
        {
            int totalCount  = DocList.Count;
            int markedCount = GetCountOfMarkedRecords();

            rbAktuell.Text = "Aktuelle Auswahl (" + totalCount.ToString() + ")";
            rbMarkiert.Text = "Markierte (" + markedCount.ToString() + ")";

            if (markedCount == 0)
                rbAktuell.Checked = true;
            else
                rbMarkiert.Checked = true;
        }


        private void buttonExport_Click(object sender, EventArgs e)
        {
            if (rbUnarch.Checked)
            {
                ExportUnarch();
                Close();
            }

            if (rbHTML.Checked)
            {
                ExportHTML();
                Close();
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Export in Unarch format
        /// </summary>
        ///---------------------------------------------------------------------
        private void ExportUnarch()
        {
            string filename;

            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                saveFileDialog.FileName = "default.una";
                saveFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";

                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                    filename = saveFileDialog.FileName;
                else
                    return;
            }



            // Construct a big string consisting of all records of the current selection.
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < DocList.Count; i++)
            {
                ValuedDocument vdoc = DocList[i];
                LineRecord lrec = (LineRecord)vdoc.Object;

                if (rbMarkiert.Checked && !vdoc.Mark)
                   continue;

                sb.AppendLine(lrec.Text);
                sb.AppendLine();
            }





            try
            {
                FileStream IniStream = new FileStream(filename, FileMode.Create);
                if (!IniStream.CanWrite)
                {
                    IniStream.Close();
                    return;
                }
                StreamWriter writer = new StreamWriter(IniStream, Configuration.encoding);
                writer.Write(sb.ToString());
                writer.Flush();
                writer.Close();
                IniStream.Close();
                return;
            }
            catch
            {
                return;
            }
        }









        private string LineRecordToHtml(LineRecord lrec)
        {
            if (lrec is LineRecordNotiz)
            {
                LineRecordNotiz l = lrec as LineRecordNotiz;

                return "<hr>" + System.Environment.NewLine +
                       "<font size=\"-2\">" +
                       l.Tag.ToString() + "." + l.Monat.ToString() + "." + l.Jahr.ToString() + "<br><br>" + System.Environment.NewLine +
                       "</font>" +
                       "<pre>" + System.Environment.NewLine +
                           EncodeForHtml(l.Info) +
                       "</pre>" + System.Environment.NewLine + "<br>" +
                       "<font size=\"-1\">" +
                           EncodeForHtml(l.Stichworte) +
                       "</font>" + System.Environment.NewLine;

            }
            else
                if (lrec is LineRecordUrl)
                {
                    LineRecordUrl l = lrec as LineRecordUrl;

                    return "<hr>" + System.Environment.NewLine +
                           "<font size=\"-2\">" +
                           l.Tag.ToString() + "." + l.Monat.ToString() + "." + l.Jahr.ToString() + "<br><br>" + System.Environment.NewLine +
                           "</font>" + System.Environment.NewLine +


                           "<a href=\"" + l.URL + "\" target=\"_blank\">" +
                               l.URL + "</a><br>" + System.Environment.NewLine +
                               EncodeForHtml(l.Titel) + "<br>" + System.Environment.NewLine +

                           "<font size=\"-1\">" + System.Environment.NewLine +
                               EncodeForHtml(l.Stichworte) + System.Environment.NewLine +
                           "</font>" + System.Environment.NewLine;
                }
                else
                    return "<hr>" + System.Environment.NewLine +
                           "<pre>" + System.Environment.NewLine +
                           EncodeForHtml(lrec.Text) + System.Environment.NewLine +
                           "</pre>" + System.Environment.NewLine;
        }






        ///---------------------------------------------------------------------
        /// <summary>
        /// Export in HTML format
        /// </summary>
        ///---------------------------------------------------------------------
        private void ExportHTML()
        {
            string filename;

            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                saveFileDialog.FileName = "default.html";
                saveFileDialog.Filter = "HTML-Dateien (*.html)|*.html|Alle Dateien (*.*)|*.*";

                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                    filename = saveFileDialog.FileName;
                else
                    return;
            }






            // Construct a big string consisting of all records of the current selection.
            StringBuilder sb = new StringBuilder();


            sb.AppendLine("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
            sb.AppendLine("<html>");
            sb.AppendLine("<head>");


            if (m_OptionsForHtml.UseTitle)
               sb.AppendLine("  <title>" + EncodeForHtml(m_OptionsForHtml.Title) + "</title>");

            if (m_OptionsForHtml.UseMetaDescription)
               sb.AppendLine("  <meta name=\"description\" content=\"" + EncodeForHtml(m_OptionsForHtml.MetaDescription) + "\">");

           if (m_OptionsForHtml.UseMetaKeywords)
               sb.AppendLine("  <meta name=\"keywords\" content=\"" + EncodeForHtml(m_OptionsForHtml.MetaKeywords) + "\">");

           if (m_OptionsForHtml.UseMetaLanguage)
               sb.AppendLine("  <meta name=\"language\" content=\"" + EncodeForHtml(m_OptionsForHtml.MetaLanguage) + "\">");

            if (m_OptionsForHtml.UseMetaRobots)
               sb.AppendLine("  <meta name=\"robots\" content=\"" + EncodeForHtml(m_OptionsForHtml.MetaRobots) + "\">");


            sb.AppendLine("  <style type=\"text/css\"> body{ font-family: Arial} </style>");
            sb.AppendLine("</head>");
            sb.AppendLine("<body>");


            if (m_OptionsForHtml.UseHeadline)
            {
                sb.AppendLine("<h1>" + EncodeForHtml(m_OptionsForHtml.Headline) + "</h1>");
                sb.AppendLine("");
            }


            for (int i = 0; i < DocList.Count; i++)
            {
                ValuedDocument vdoc = DocList[i];
                LineRecord lrec = (LineRecord)vdoc.Object;

                if (rbMarkiert.Checked && !vdoc.Mark)
                    continue;

                sb.AppendLine(LineRecordToHtml(lrec));
                sb.AppendLine();
            }

            sb.AppendLine("</body>");
            sb.AppendLine("</html>");




            try
            {
                FileStream IniStream = new FileStream(filename, FileMode.Create);
                if (!IniStream.CanWrite)
                {
                    IniStream.Close();
                    return;
                }
                StreamWriter writer = new StreamWriter(IniStream, Configuration.encoding);
                writer.Write(sb.ToString());
                writer.Flush();
                writer.Close();
                IniStream.Close();
                return;
            }
            catch
            {
                return;
            }
        }





        private string EncodeForHtml(string s)
        {
            string result = "";

            for (int i = 0; i < s.Length; i++)
            {
                char c = s[i];

                if (c == '<')
                    result += "&lt;";
                else
                if (c == '>')
                   result += "&gt;";
                else
                if (c == '')
                   result += "&auml;";
                else
                if (c == '')
                   result += "&ouml;";
                else
                if (c == '')
                   result += "&uuml;";
                else
                if (c == '')
                   result += "&Auml;";
                else
                if (c == '')
                   result += "&Ouml;";
                else
                if (c == '')
                   result += "&Uuml;";
                else
                if (c == '')
                   result += "&szlig;";
                else
                   result += c;
            }

            return result;
        }

        private void buttonHtmlOptions_Click(object sender, EventArgs e)
        {
            using (FormHtmlOptions f = new FormHtmlOptions(m_OptionsForHtml))
            {
                DialogResult dr = f.ShowDialog();
            }
        }

        private void rbHTML_CheckedChanged(object sender, EventArgs e)
        {
            if (rbHTML.Checked)
               buttonHtmlOptions.Enabled = true;
        }

        private void rbUnarch_CheckedChanged(object sender, EventArgs e)
        {
            if (rbUnarch.Checked)
               buttonHtmlOptions.Enabled = false;
        }

        private void FormExport_Shown(object sender, EventArgs e)
        {
            SetHtmlOptions();
        }

    }
}