// 7.5.2007
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fusym;

namespace Unarch5
{
    public partial class FormGrid : Form
    {
        private ValuedDocList m_ValuedDocList;
        private FormStandard m_FormStandard;
        private DataTable dt;

        public FormGrid(FormStandard formStandard)
        {
            InitializeComponent();

            this.MdiParent = formStandard.MdiParent;
            m_FormStandard = formStandard;
        }

        public void SetValuedDocList(ValuedDocList doclist)
        {
            m_ValuedDocList = doclist;
            CreateTable();
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Loop through m_ValuedDocList. Copy some data to the table dt.
        /// </summary>
        ///---------------------------------------------------------------------
        private void CreateTable()
        {
            dt = null;
            dt = new DataTable();

            dt.Clear();
            dt.Columns.Clear();

            dt.Columns.Add(new DataColumn("Id", typeof(int)));
            dt.Columns.Add(new DataColumn("Keywords"));
            dt.Columns.Add(new DataColumn("Value", typeof(double)));
            dt.Columns.Add(new DataColumn("Date",typeof(DateTime)));
            dt.Columns.Add(new DataColumn("Mark", typeof(bool)));

            for (int i = 0; i < m_ValuedDocList.Count; i++)
            {
                ValuedDocument vdoc = m_ValuedDocList[i];
                object o = vdoc.Object;
                DateTime d;

                if (o is IRecordWithDate)
                    d = (o as IRecordWithDate).Date();
                else
                    d = DateTime.MinValue;

                AddNewRow(dt, 
                          i,
                          vdoc.theString, 
                          vdoc.Value,
                          d,
                          vdoc.Mark
                         );
            }



            dataGridView.DataSource = dt;
            dataGridView.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;

            dataGridView.Columns["Id"   ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Date" ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Mark" ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Value"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Value"].DefaultCellStyle.Format = "N3";

            //            BreiteAnSpaltenAnpassen(dataGridView);
            //            dataGridView.Columns[3].DefaultCellStyle.Format = "dd.MMMM yyyy";
          //  dataGridView.Columns[0].Visible = false;
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Add a new row.
        /// </summary>
        ///---------------------------------------------------------------------
        private void AddNewRow(
                               DataTable dt, 
                               int       id, 
                               string    keywords, 
                               double    value,
                               DateTime  d,
                               bool      mark
                              )
        {
            DataRow dr = dt.NewRow();


            dr[0] = id;
            dr[1] = keywords;
            dr[2] = value;
            dr[3] = d;
            dr[4] = mark;

            try
            {
                dt.Rows.Add(dr);
            }
            catch (Exception e)
            {
                MessageBox.Show("Main: " + e.Message);
            }
        }




        private void buttonClose_Click(object sender, EventArgs e)
        {
            Hide();
        }

        private void dataGridView_RowEnter(object sender, DataGridViewCellEventArgs e)
        {
            int rowNumber = e.RowIndex;
            if ((rowNumber < 0) || (rowNumber >= dt.Rows.Count))
                return;

            int idx = (int)dataGridView.Rows[rowNumber].Cells[0].Value;
            m_FormStandard.Current = idx;
        }


        private void dataGridView_CellValueChanged(object sender, DataGridViewCellEventArgs e)
        {
            int rowNumber = e.RowIndex;
            if ((rowNumber < 0) || (rowNumber >= dt.Rows.Count))
                return;

            int idx = (int)dataGridView.Rows[rowNumber].Cells[0].Value;

            ValuedDocument vdoc = m_ValuedDocList[idx];
            bool checkState = (bool)dataGridView.Rows[rowNumber].Cells[4].Value;
            vdoc.Mark = checkState;
        }

    }

}