//------------------------------------------------------------------------------
// Purpose: Lisp-like expressions, solely based on symbols and parentheses,
//          in order to support AI applications
//
// Date   : 28. Oct. 2010
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fusym;
using System.IO;

namespace Unarch5
{
    public partial class FormLispExpressions : Form
    {
        SemanticNetwork m_SemanticNetwork;

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public FormLispExpressions()
        {
            InitializeComponent();

            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            m_SemanticNetwork = gcm.SemanticNetwork;
        }


        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }



        private void buttonAnalyse_Click(object sender, EventArgs e)
        {
            string crlf = Environment.NewLine;

            Parser p = new Parser(textBoxInput.Text);

            try
            {
                Expression ex = p.Parse();

                textBoxOutput.Text = "";

                List<string> sl = Expression.PrettyPrintList(ex);

                // Mit StringBuilder zu-tun
                foreach (string s in sl)
                    textBoxOutput.Text += s + Environment.NewLine;
            }
            catch (Exception exc)
            {
                textBoxOutput.Text = exc.Message;
            }

        }

        private void Dis2play(Expression ex)
        {

        }



        private void Search()
        {
            Parser p = new Parser(textBoxInput.Text);
            Expression ex = p.Parse();

            GlobalClusterManager g = GlobalClusterManager.GetInstance();

            ValuedDocList docList = new ValuedDocList();

            SympsSearchCustomization f = new SympsSearchCustomization(
                                                                        g.SemanticNetwork,
                                                                        ex,
                                                                        docList,
                                                                        textBoxQuery.Text
                                                                       );

            f.Search();

            textBoxOutput.Text = "";
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < docList.Count; i++)
            {
                Expression e = (Expression)docList[i].Object;

                List<string> sl = Expression.PrettyPrint(e);
                foreach (string s in sl)
                    sb.AppendLine(s);
                sb.AppendLine();
            }

            textBoxOutput.Text = sb.ToString();
        }



        private void buttonSearch_Click(object sender, EventArgs e)
        {
            Search();
        }


        Concepts.InferenceRule ParseRule(ExpressionList e) 
        {
            if (e == null)
                return null;

            //--- Rule type ----------------------------------------------------

            ExpressionSymbol ruleType = e.Car as ExpressionSymbol;

            if (ruleType == null)
                return null;

            if (
                (ruleType.Symbol != "IMPLIES") &&
                (ruleType.Symbol != "EQUIVALENT") &&
                (ruleType.Symbol != "=>") &&
                (ruleType.Symbol != "<=>")
               )
                return null;


            //--- Left part ----------------------------------------------------

            if (!(e.Cdr is ExpressionList) && (e.Cdr != null))
                return null;

            ExpressionList cdr = e.Cdr as ExpressionList;
            if (cdr == null)
                return null;

            Concepts.Concept leftPart = ParseConcept(cdr.Car as ExpressionList);

            if (leftPart == null)
                return null;

            //--- Right part ---------------------------------------------------

            cdr = cdr.Cdr as ExpressionList;
            if (cdr == null)
                return null;

            Concepts.Concept rightPart = ParseConcept(cdr.Car as ExpressionList);

            if (rightPart == null)
                return null;



            


            Concepts.InferenceRule result = new Concepts.InferenceRule();
            result.RuleType  = m_SemanticNetwork.AddSymbol(ruleType.Symbol);
            result.LeftSide  = leftPart;
            result.RightSide = rightPart;

            return result;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Parse a Lisp expression and try to create a concept out of it.
        /// </summary>
        ///---------------------------------------------------------------------
        Concepts.Concept ParseConcept(Expression exp)
        {
            if (exp is ExpressionSymbol)
            {
                Concepts.Concept r = new Concepts.Concept();
                r.ConceptName = m_SemanticNetwork.AddSymbol((exp as ExpressionSymbol).Symbol);
                return r;
            }

            ExpressionList e = exp as ExpressionList;

            if (e == null)
                return null;

            //--- Concept name ---

            ExpressionSymbol conceptName = e.Car as ExpressionSymbol;

            if (conceptName == null)
                return null;

            if (!(e.Cdr is ExpressionList) && (e.Cdr != null))
                return null;





            //--- Slots ---

            List<Concepts.SlotValuePair> slotValuePairs = new List<Concepts.SlotValuePair>();




            ExpressionList cdr = e.Cdr as ExpressionList;
            while (cdr != null)
            {
                ExpressionSymbol slotName;
                Concepts.Concept slotValue;

                if (cdr.Car is ExpressionSymbol)
                {
                    slotName  = cdr.Car as ExpressionSymbol;
                    slotValue = null;
                }
                else
                {
                    ExpressionList slotvaluePair = cdr.Car as ExpressionList;

                    if (slotvaluePair == null)
                        return null;

                    slotName = slotvaluePair.Car as ExpressionSymbol;
                    if (slotName == null)
                        return null;

                    slotValue = ParseConcept(slotvaluePair.Cdr as ExpressionList);
                }





                Concepts.SlotValuePair p = new Concepts.SlotValuePair();
                p.SlotName = m_SemanticNetwork.AddSymbol(slotName.Symbol);
                p.SlotValue = slotValue; 
                slotValuePairs.Add(p);



                cdr = cdr.Cdr as ExpressionList;
            }






            //--- Synthesis ---

            Concepts.Concept result = new Concepts.Concept();
            result.ConceptName = m_SemanticNetwork.AddSymbol(conceptName.Symbol);
            result.SlotValuePairs.AddRange(slotValuePairs); // Zuweisen ist schneller

            return result;
        }


        private void buttonLoad_Click(object sender, EventArgs e)
        {
            string fn = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\global.lsp";


            try
            {
                Expression ex = new Expression();

                string s = ex.LoadFromFile(fn);
                Parser p = new Parser(s);
                ex = p.Parse();

                foreach (Expression e1 in Expression.EnumerateListElements(ex))
                {
                    Concepts.Concept c = ParseConcept(e1);
                    Concepts.InferenceRule rule = ParseRule(e1 as ExpressionList);

                    if (rule != null)
                        Concepts.InferenceEngine.RuleBase.Add(rule);
                }


                List<Concepts.InferenceRule> rules = Concepts.InferenceEngine.RuleBase;




                textBoxInput.Text = "";
                List<string> sl = Expression.PrettyPrintList(ex);
                StringBuilder sb = new StringBuilder();
                foreach (string st in sl)
                    sb.AppendLine(st);
                textBoxInput.Text = sb.ToString();
            }
            catch (Exception exc)
            {
                textBoxInput.Text = exc.Message;
            }
        
        
        
        
        
        }

        private void buttonInference_Click(object sender, EventArgs e)
        {
            Concepts.InferenceEngine inf = new Concepts.InferenceEngine(m_SemanticNetwork);
            List<Concepts.InferenceRule> rulebase = Concepts.InferenceEngine.RuleBase;

            





            Parser p = new Parser(textBoxInput.Text);
            textBoxOutput.Text = "";

            try
            {
                Expression ex = p.Parse();
                if (ex is ExpressionList)
                    ex = (ex as ExpressionList).Car;

                Concepts.Concept instance = ParseConcept(ex);

                if (instance != null)
                {
                    // Apply each of the rules in the rulebase.
                    foreach (Concepts.InferenceRule rule in rulebase)
                    {
                        List<Concepts.Binding> bindings = inf.Match(rule.LeftSide, instance);

                        if (bindings != null)
                        {
                            Concepts.Concept c2 = inf.Apply(rule.LeftSide, rule.RightSide, instance, bindings);
                            textBoxOutput.Text += c2.ToString() + Environment.NewLine;
                        }
                    }
                }


                /*
                List<string> sl = Expression.PrettyPrintList(ex);

                // Mit StringBuilder zu-tun
                foreach (string s in sl)
                    textBoxOutput.Text += s + Environment.NewLine;*/
            }
            catch (Exception exc)
            {
                textBoxOutput.Text = exc.Message;
            }
        }

        private void buttonClear_Click(object sender, EventArgs e)
        {
            textBoxInput.Text  = "";
            textBoxOutput.Text = "";
        }
    }
}