//------------------------------------------------------------------------------
// Purpose: Main form of Unarch
//
// Date   : 17.Jun.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using UnarchControls;
using Fusym;
using Symps;
using System.Xml;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace Unarch5
{
    public partial class FormMain: Form, IFileHistoryReceiver
    {
        private int    childFormNumber = 0;
        string[]       m_Parameters;
        FileHistory    m_FileHistory;

        public FormMain(string[] parameters)
        {
            InitializeComponent();
            m_Parameters = parameters;
        }

        private void ShowNewForm(object sender, EventArgs e)
        {
            // Create a new instance of the child form.
            FormStandard childForm = new FormStandard("");
            // Make it a child of this MDI form before showing it.
            childForm.MdiParent = this;
            childForm.FileHistoryReceiver = this;
            childForm.Text = "Window " + childFormNumber++;
            childForm.Show();
        }

        private void OpenFile(object sender, EventArgs e)
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                openFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";
                if (openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    FormStandard f = new FormStandard(openFileDialog.FileName);
                    f.MdiParent = this;
                    f.FileHistoryReceiver = this;
                    f.Show();
                }
            }
        }

        private void SaveAsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                saveFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    string FileName = saveFileDialog.FileName;
                    // TODO: Add code here to save the current contents of the form to a file.
                }
            }
        }

        private void ExitToolsStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void CutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // TODO: Use System.Windows.Forms.Clipboard to insert the selected text or images into the clipboard
        }

        private void CopyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // TODO: Use System.Windows.Forms.Clipboard to insert the selected text or images into the clipboard
        }

        private void ToolBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            toolStrip.Visible = toolBarToolStripMenuItem.Checked;
        }

        private void StatusBarToolStripMenuItem_Click(object sender, EventArgs e)
        {
            statusStrip.Visible = statusBarToolStripMenuItem.Checked;
        }

        private void CascadeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.Cascade);
        }

        private void TileVerticleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileVertical);
        }

        private void TileHorizontalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.TileHorizontal);
        }

        private void ArrangeIconsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            LayoutMdi(MdiLayout.ArrangeIcons);
        }

        private void CloseAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (Form childForm in MdiChildren)
            {
                childForm.Close();
            }
        }

        private void toolStripMenuItemCluster_Click(object sender, EventArgs e)
        {
            string filename = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\cluster.txt";
            FormStandard f = new FormStandard(filename,EditorType.Cluster);
            f.MdiParent = this;
            f.FileHistoryReceiver = this;
            f.Show();
        }

        private void symboleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormSearchSymbol f = new FormSearchSymbol();
            f.MdiParent = this;
            f.Show();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (FormAboutBox f = new FormAboutBox())
            {
                f.ShowDialog();
            }
        }




        private void FormMain_Load(object sender, EventArgs e)
        {
            m_FileHistory = new FileHistory(
                                            fileMenu,
                                            toolStripSeparatorFileHistoryBegin,
                                            toolStripSeparatorFileHistoryEnd,
                                            FileHistoryClick
                                           );

            LoadSettings();


            // Fill the global cluster manager with data
            // from the semantic network files.
            bgwGlobalClusterManager.RunWorkerAsync();


            // Load all .una files given in command line parameters
            foreach (string s in m_Parameters)
            {
                FormStandard f = new FormStandard(s);
                f.MdiParent = this;
                f.FileHistoryReceiver = this;
                f.Show();
            }
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            StoreSettings();
            Configuration c = Configuration.GetInstance();
            c.Save();
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Load settings from the config file
        /// </summary>
        ///---------------------------------------------------------------------
        private void LoadSettings()
        {
            Configuration c = Configuration.GetInstance();
            
            this.Location = c.FormMainSettings.Location;
            this.Size     = c.FormMainSettings.Size;
            this.m_FileHistory.MaxCount = c.MaxFileHistory;
            this.m_FileHistory.Add(c.FileHistorySettings.History);
            this.SetClassicalDatabaseView(c.ClassicalDatabaseView);
            this.cbSearchFile.Items.Clear();
            this.cbSearchFile.Items.AddRange(c.History.ToArray());
            this.cbDirectoryRoot.Items.Clear();
            this.cbDirectoryRoot.Items.AddRange(c.Directories.ToArray());
        }


        private void StoreSettings()
        {
            Configuration c = Configuration.GetInstance();
            c.FormMainSettings.Location = this.Location;
            c.FormMainSettings.Size     = this.Size;
            
            c.FileHistorySettings.History.Clear();
            c.FileHistorySettings.History.AddRange(m_FileHistory.ToStringList());

            /*
            c.History.Clear();
            AddToStringList(c.History, this.cbSearchFile.Items);
            c.RestrictArrayListLength(c.History , c.MaxQueryHistory);
            */

            c.Directories.Clear();
            AddToStringList(c.Directories, this.cbDirectoryRoot.Items);
            c.RestrictArrayListLength(c.Directories, c.MaxDirectoryHistory);
          }



        private void AddToStringList(List<string> list, ComboBox.ObjectCollection objects)
        {
            foreach (string s in objects)
                if (!list.Contains(s))
                    list.Insert(0, s);
        }


        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormUniversalEditor f = new FormUniversalEditor(); // Was ist das fr ein Formular? zu-tun
            f.MdiParent = this;
            f.Show();
        }

        private void toolStripMenuItemNotizen_Click(object sender, EventArgs e)
        {
            string filename = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\memory.txt";
            FormStandard f = new FormStandard(filename,EditorType.Notizen);
            f.MdiParent = this;
            f.FileHistoryReceiver = this;
            f.Show();
        }

        private void toolStripMenuItemUrls_Click(object sender, EventArgs e)
        {
            string filename = Path.GetDirectoryName(Application.ExecutablePath) + @"\database\url.txt";
            FormStandard f = new FormStandard(filename,EditorType.URLs);
            f.MdiParent = this;
            f.FileHistoryReceiver = this;
            f.Show();
        }



        public void AddFileToHistory(string filename)
        {
            m_FileHistory.Add(filename);
        }
         


        private void FileHistoryClick(object sender, EventArgs e)
        {
            ToolStripMenuItem MenuItem = (ToolStripMenuItem)sender;
            string filename = MenuItem.Tag.ToString();

            FormStandard f = new FormStandard(filename);
            f.MdiParent = this;
            f.FileHistoryReceiver = this;
            f.Show();
        }

        private void optionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (FormOptionen f = new FormOptionen())
            {
                f.ShowDialog();

                // The settings may have been modified.
                LoadSettings();
            }
        }

        private void saveActiveMdiChild()
        {
            Form activeChild = this.ActiveMdiChild;

            if (activeChild is FormStandard)
                (activeChild as FormStandard).TryToSave(false);
        }


        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            saveActiveMdiChild();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveActiveMdiChild();
        }


        private void SetClassicalDatabaseView(bool value)
        {
            editMenu.Visible = value;
            toolStripSeparator1.Visible = false;
            toolStripSeparator2.Visible = value;
            toolStripButtonNotizen.Visible = value;
            toolStripButtonUrls.Visible = value;
            toolStripButtonCluster.Visible = value;
            toolStripButtonSymbol.Visible = value;
            toolStripButtonClose.Visible = value;
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load semantic networks.
        /// </summary>
        ///---------------------------------------------------------------------
        private void LoadGlobalClusterFiles()
        {
            Configuration c = Configuration.GetInstance();
            List<string> activeFiles = c.ActiveClusterFiles;
            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            SemanticNetwork cm = gcm.SemanticNetwork;

            foreach (string filename in activeFiles)
            {
                bgwGlobalClusterManager.ReportProgress(0, filename);
                Application.DoEvents();
                try
                {
                    cm.LoadFromFile(filename);
                }
                catch
                {
                }
            }

            bgwGlobalClusterManager.ReportProgress(0, "");
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load cluster files for the global cluster manager
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        ///---------------------------------------------------------------------
        private void bgwGlobalClusterManager_DoWork(object sender, DoWorkEventArgs e)
        {
            LoadGlobalClusterFiles();
            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            gcm.Loaded = true;
        }

        private void bgwGlobalClusterManager_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            toolStripStatusLabel.Text = (string)e.UserState;
        }

        private void testToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            FormGraph f = new FormGraph();
            f.MdiParent = this;
            f.Show();
        }





        private void toolStripButtonConcepts_Click(object sender, EventArgs e)
        {
            FormConcept f = new FormConcept();
            f.MdiParent = this;
            ConceptInferenceSystem conceptInferenceSystem = new ConceptInferenceSystem();
            f.Show();
        }

        private void buttonExperiment_Click(object sender, EventArgs e)
        {
            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            SemanticNetwork cm = gcm.SemanticNetwork;
            
            Betweenness b = new Betweenness();

            string s = b.Test(cm, 10);


//            string s = b.RunTestCycles(cm,100);
            MessageBox.Show(s);
        }

        private void zusammenhangskomponentenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormConnectedComponents f = new FormConnectedComponents();
            f.Show();
        }


        private void assoziationsdateiOptimierenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormOptimizeAssociationFile f = new FormOptimizeAssociationFile();
            f.MdiParent = this;
            f.Show();
        }

        private void bitvektorenFrSymboleToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormSymbolBits f = new FormSymbolBits();
            f.MdiParent = this;
            f.Show();

        }

        private void cbSearchFile_DropDown(object sender, EventArgs e)
        {
            cbSearchFile.DropDownStyle = ComboBoxStyle.Simple;
            MessageBox.Show("Hallo");
        }

        private void cbSearchFile_DropDownClosed(object sender, EventArgs e)
        {
            cbSearchFile.Enabled = true;
        }

        private void cbSearchFile_Validated(object sender, EventArgs e)
        {
            cbSearchFile.DropDownStyle = ComboBoxStyle.DropDownList;
        }

        private void toolStripButtonSearchFileNames_Click(object sender, EventArgs e)
        {
            FormSearchFilename f = new FormSearchFilename(cbSearchFile.Text, cbDirectoryRoot.Text);
            f.MdiParent = this;
            f.Show();

            // Query history
            cbSearchFile.Text = cbSearchFile.Text.ToUpper();
            if ((cbSearchFile.Items.IndexOf(cbSearchFile.Text) < 0) && (cbSearchFile.Text.Length > 0))
                cbSearchFile.Items.Insert(0, cbSearchFile.Text);

            // Add chosen root directory to the history
            if ((cbDirectoryRoot.Items.IndexOf(cbDirectoryRoot.Text) < 0) && (cbDirectoryRoot.Text.Length > 0))
                cbDirectoryRoot.Items.Insert(0, cbDirectoryRoot.Text); 
        }

        private void cbSearchFile_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                // Perform a PostMessage. In C#, Control.BeginInvoke does this.
                // The underlying Control is 'this' form, but presumably any
                // other control would serve the same purpose. An event of another
                // control is called, with, in this case, two parameter which are null.
                this.BeginInvoke(new System.EventHandler(toolStripButtonSearchFileNames_Click), new object[] { null, null });

                // Avoid the beep that would come upon the Enter key.
                e.SuppressKeyPress = true;
            }
        }


        private void cbSearchFile_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Char.IsLetter(e.KeyChar))
                e.KeyChar = Char.ToUpper(e.KeyChar);
        }



        private void toolStripButtonBrowseForFolder_Click(object sender, EventArgs e)
        {
            if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
            {
                cbDirectoryRoot.Text = folderBrowserDialog.SelectedPath;
            }

        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// See http://www.ms-news.net/f824/text-eingaben-der-toolstrip-9230859.html
        /// Workaround to intercept backslash.
        /// </summary>
        ///---------------------------------------------------------------------
        class MyToolStripComboBox : ToolStripComboBox
        {
            const int WM_KEYDOWN = 0x100;
            const Keys Backspace = Keys.OemOpenBrackets | Keys.Control | Keys.Alt;

            protected override bool ProcessCmdKey(ref Message m, Keys keyData)
            {
                if (m.Msg == WM_KEYDOWN && keyData == Backspace)
                {
                    this.SelectedText = "\\"; 
                    return true;
                }



                return base.ProcessCmdKey(ref m, keyData);
            }
        }

        private void FormMain_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyData == Keys.Enter)
            {
                if (cbSearchFile.Focused)
                {
                    e.SuppressKeyPress = true;
                }
            }
        }

        private void cbDirectoryRoot_KeyDown(object sender, KeyEventArgs e)
        {
            cbSearchFile_KeyDown(sender, e);
        }

        private void sympsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormLispExpressions f = new FormLispExpressions();
            f.ShowDialog();
        }

        private void verteilungDerGradeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormStatistics f = new FormStatistics();
            f.ShowDialog();
        }

        private void konzepttransformationenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormConceptTransformations f = new FormConceptTransformations();
            f.ShowDialog();
        }

        private void clusteringToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormClustersOfSymbols f = new FormClustersOfSymbols();
            f.ShowDialog();
        }

        private void violettToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormViolet f = new FormViolet();
            f.Show();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            /*
            GraphClustering g = new GraphClustering();
            g.Test1();
             */

            FormScrollBars f = new FormScrollBars();
            f.ShowDialog();
        }



    }
}
