// 6.Jun.2010
// Joachim Pimiskern
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Fusym;
using UnarchControls;

namespace Unarch5
{
    public partial class FormOptimizeAssociationFile : Form
    {
        private bool bEnd = true;
        private Random m_Rnd = new Random(22);
        private ClusterList m_Clusters;
        private SemanticNetwork m_SemanticNetwork;
        private SymbolAsBitsManager m_SymbolAsBitsManager;

        public FormOptimizeAssociationFile()
        {
            InitializeComponent();

            m_SemanticNetwork = GlobalClusterManager.GetInstance().SemanticNetwork;
            m_Clusters = new ClusterList();
            m_SymbolAsBitsManager = new SymbolAsBitsManager(m_SemanticNetwork, 32);
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonStart_Click(object sender, EventArgs e)
        {
         //   try
            {
                FileInfo fi = new FileInfo(textBoxFilename.Text);
                if (fi.Exists)
                    DoIt(fi.FullName);
            }
            /*
            catch(Exception ex)
            {
                MessageBox.Show("Fehler: " + ex.Message);
            }*/
        }




        ///---------------------------------------------------------------------
        /// <summary>
        /// Main loop.
        /// </summary>
        ///---------------------------------------------------------------------
        private void DoIt(string filename)
        {
            if (bEnd == false)
            {
                bEnd = true;
                return;
            }

            buttonStart.Text = "&Stop";
            buttonStart.Update();

            m_Clusters.Clear();
            m_Clusters.LoadFromFile(filename);

            if (cbScramble.Checked)
                m_Clusters.Scramble();



            int steps = 0;
            bEnd = false;
            while (!bEnd)
            {
                OneStep1();
                OneStep2();
                //OneStep3();

                steps++;

                //--- Display statistics ---
                if (steps % 100000 == 0)
                {
                    RemoveEmptyClusters();
                    textBoxSteps.Text = steps.ToString();
                    textBoxNeighbourhood . Text = SumNeighbourValues().ToString();
                    textBoxLevenshteinDistanceSum.Text = SumLevenShtein().ToString();
                    textBoxCountOfRuns   . Text = m_Clusters.CountOfRuns().ToString();
                    textBoxLongestRun    . Text = m_Clusters.LongestRun().ToString();

                }

                Application.DoEvents();
            }

            buttonStart.Text = "&Start";
        }



        private void RemoveEmptyClusters()
        {
            int count = m_Clusters.Items.Count;
            ClusterList result = new ClusterList(count);
            for (int i = 0; i < count; i++)
            {
                Cluster cl = m_Clusters.Items[i];
                if (cl.Items.Count > 0)
                    result.Items.Add(cl);
            }

            m_Clusters = result;
        }


        private int SumNeighbourValues()
        {
            int result = 0;
            int count = m_Clusters.Items.Count;
            Cluster old = m_Clusters.Items[0];

            for (int i = 1; i < count; i++)
            {
                Cluster current = m_Clusters.Items[i];
                result += current.CountOfCommonSymbols(old);
                old = current;
            }

            return result;
        }






        private object SumLevenShtein()
        {
            int result = 0;
            int count = m_Clusters.Items.Count;
            Cluster old = m_Clusters.Items[0];

            for (int i = 1; i < count; i++)
            {
                Cluster current = m_Clusters.Items[i];
                result += MinimalEditDistance(old, current);
                old = current;
            }

            return result;
        }







        ///---------------------------------------------------------------------
        /// <summary>
        /// One optimization try.
        /// </summary>
        ///---------------------------------------------------------------------
        private void OneStep1()
        {
            int valueOld, valueNew;

            // Choose two different clusters
            int idx1 = m_Rnd.Next(m_Clusters.Items.Count);
            int idx2 = m_Rnd.Next(m_Clusters.Items.Count);

            if (idx1 == idx2)
                return;

            // Swallow, if first contains the second one.
            if (
                (m_Clusters.Items[idx2].Items.Count > 0) &&
                (m_Clusters.Items[idx1].Contains(m_Clusters.Items[idx2]))
               )
            {
                textBoxActions.Text += "Deleted:" + Environment.NewLine + m_Clusters.Items[idx2].ToString() + Environment.NewLine;
                ScrollToEnd();
                m_Clusters.Items[idx2].Clear();
                return;
            }



            // Swallow, if the second contains the first one.
            if (
                (m_Clusters.Items[idx1].Items.Count > 0) &&
                (m_Clusters.Items[idx2].Contains(m_Clusters.Items[idx1]))
               )
            {
                textBoxActions.Text += "Deleted:" + Environment.NewLine + m_Clusters.Items[idx1].ToString() + Environment.NewLine;
                ScrollToEnd();
                m_Clusters.Items[idx1].Clear();
                return;
            }


            // Make sure there is a successor
            if (idx1 + 1 >= m_Clusters.Items.Count)
                return;

            // Make sure there is a successor's successor
            if (idx2 + 2 >= m_Clusters.Items.Count)
                return;

            // The idea is to move a cluster d into a new context.
            //
            // Old:   a b     c d e
            //        -       -
            // New:   a d b   c e
            //

            Cluster a, b, c, d, e;
            a = m_Clusters.Items[idx1 + 0];
            b = m_Clusters.Items[idx1 + 1];
            c = m_Clusters.Items[idx2 + 0];
            d = m_Clusters.Items[idx2 + 1];
            e = m_Clusters.Items[idx2 + 2];


            /*
            if (m_Rnd.NextDouble() > 0.5) // Apply different methods to determine the relationships.
            {
                valueOld = a.HasCommonSymbol(b) + c.HasCommonSymbol(d) + d.HasCommonSymbol(e);
                valueNew = a.HasCommonSymbol(d) + d.HasCommonSymbol(b) + c.HasCommonSymbol(e);
            }
            else
            {
                valueOld = a.CountOfCommonSymbols(b) + c.CountOfCommonSymbols(d) + d.CountOfCommonSymbols(e);
                valueNew = a.CountOfCommonSymbols(d) + d.CountOfCommonSymbols(b) + c.CountOfCommonSymbols(e);
            }

            if (valueNew == valueOld)
            {
                valueOld = -(m_Clusters.MinimalSemanticDistance(a,b) + m_Clusters.MinimalSemanticDistance(c,d) + m_Clusters.MinimalSemanticDistance(d,e));
                valueNew = -(m_Clusters.MinimalSemanticDistance(a,d) + m_Clusters.MinimalSemanticDistance(d,b) + m_Clusters.MinimalSemanticDistance(c,e));
            }



            if (valueNew == valueOld)*/
            {
                valueOld = -(MinimalEditDistance(a, b) + MinimalEditDistance(c, d) + MinimalEditDistance(d, e));
                valueNew = -(MinimalEditDistance(a, d) + MinimalEditDistance(d, b) + MinimalEditDistance(c, e));
            }


            /*
            if ((valueNew == valueOld) && (m_Rnd.NextDouble() > 0.5))
            {
                valueOld = -(MaximalEditDistance(a, b) + MaximalEditDistance(c, d) + MaximalEditDistance(d, e));
                valueNew = -(MaximalEditDistance(a, d) + MaximalEditDistance(d, b) + MaximalEditDistance(c, e));
            }*/





            if (valueNew == valueOld)
            {
                if (m_Rnd.NextDouble() > 0.5)
                {
                    valueNew = 0;
                    valueOld = 1;
                }
                else
                {
                    valueNew = 1;
                    valueOld = 0;
                }
            }







            /*
            if (valueNew == valueOld)
            {
                double d1 = Math.Abs(a.LD - b.LD) + Math.Abs(c.LD - d.LD) + Math.Abs(d.LD - e.LD);
                double d2 = Math.Abs(a.LD - d.LD) + Math.Abs(d.LD - b.LD) + Math.Abs(c.LD - e.LD);

                if (d1 > d2)
                {
                    valueNew = 0;
                    valueOld = 1;
                }
                else
                if (d2 > d1)
                {
                    valueNew = 1;
                    valueOld = 0;
                }
            }

            */



            // Improvement? Then exchange.
            if (valueNew >= valueOld)
            {
                if (idx1 < idx2)
                {
                    m_Clusters.Items.RemoveAt(idx2 + 1);
                    m_Clusters.Items.Insert(idx1 + 1, d); // Insert ist eine aufwendige Operation
                }
                else // idx1 != idx2, idx2 < idx1
                {
                    m_Clusters.Items.Insert(idx1 + 1, d); // Insert ist eine aufwendige Operation
                    m_Clusters.Items.RemoveAt(idx2 + 1);
                }
            }
        }

        private int MyCommonSymbols2(Cluster a, Cluster b)
        {
            int count1 = a.Count;
            int count2 = b.Count;
            int result = 0;

            for (int i = 0; i < count1; i++)
            {
                string s1 = a.Items[i].ToString();
                for (int j = 0; j < count2; j++)
                {
                    string s2 = b.Items[j].ToString();
                    result += (m_SymbolAsBitsManager.BitsPerSymbol - m_SymbolAsBitsManager.Distance(s1,s2));
                }
            }

            return result;
        }

        private int MyCommonSymbols3(Cluster a, Cluster b)
        {
            int count1 = a.Count;
            int count2 = b.Count;
            int result = 0;

            for (int i = 0; i < count1; i++)
            {
                string s1 = a.Items[i].ToString();
                for (int j = 0; j < count2; j++)
                {
                    string s2 = b.Items[j].ToString();
                    result -= LevenshteinDistance.Compute(s1,s2);
                }
            }

            return result;
        }



        private int MinimalEditDistance(Cluster a, Cluster b)
        {
            List<Node> itemsA = a.Items;
            List<Node> itemsB = b.Items;

            int count1 = a.Count;
            int count2 = b.Count;
            int min = int.MaxValue;

            for (int i = 0; i < count1; i++)
            {
                string s1 = itemsA[i].ToString();
                for (int j = 0; j < count2; j++)
                {
                    string s2 = itemsB[j].ToString();

                    int dist = LevenshteinDistance.Compute(s1, s2);

                    if (dist < min)
                        min = dist;
                }
            }


            return min;
        }






        private int MaximalEditDistance(Cluster a, Cluster b)
        {
            List<Node> itemsA = a.Items;
            List<Node> itemsB = b.Items;

            int count1 = a.Items.Count;
            int count2 = b.Items.Count;
            int max = int.MinValue;

            for (int i = 0; i < count1; i++)
            {
                string s1 = itemsA[i].ToString();
                for (int j = 0; j < count2; j++)
                {
                    string s2 = itemsB[j].ToString();

                    int dist = LevenshteinDistance.Compute(s1, s2);

                    if (dist > max)
                        max = dist;
                }
            }


            return max;
        }












        ///---------------------------------------------------------------------
        /// <summary>
        /// If three adjacent clusters generate all combinations,
        /// reduce them to a single cluster.
        /// </summary>
        ///---------------------------------------------------------------------
        private void OneStep2()
        {
            int count0 = m_Clusters.Items.Count;

            if (count0 < 3)
                return;

            int idx1 = m_Rnd.Next(count0 - 2);
            int idx2 = idx1 + 1;
            int idx3 = idx1 + 2;


            if (idx3 >= m_Clusters.Items.Count)
                return;

            if ((idx1 == idx2) || (idx1 == idx3) || (idx2 == idx3))
                return;

            List<Node> allNodes = new List<Node>();
            Dictionary<string, int> dict = new Dictionary<string, int>();

            Cluster cl1 = m_Clusters.Items[idx1];
            Cluster cl2 = m_Clusters.Items[idx2];
            Cluster cl3 = m_Clusters.Items[idx3];

            // Handle empty clusters (that have not yet been removed)
            if ((cl1.Items.Count == 0) || (cl2.Items.Count == 0) || (cl3.Items.Count == 0))
                return;

            
            for (int i = 0; i < cl1.Items.Count; i++)
            {
                Node n = cl1.Items[i];
                if (!allNodes.Contains(n))
                    allNodes.Add(n);

                for (int j = 0; j < cl1.Items.Count; j++)
                {
                    Node m = cl1.Items[j];
                    string s = n.ToString() + ":::" + m.ToString();
                    if (!dict.ContainsKey(s))
                        dict.Add(s, 1);
                }
            }

            for (int i = 0; i < cl2.Items.Count; i++)
            {
                Node n = cl2.Items[i];
                if (!allNodes.Contains(n))
                    allNodes.Add(n);

                for (int j = 0; j < cl2.Items.Count; j++)
                {
                    Node m = cl2.Items[j];
                    string s = n.ToString() + ":::" + m.ToString();
                    if (!dict.ContainsKey(s))
                        dict.Add(s, 1);
                }
            }

            for (int i = 0; i < cl3.Items.Count; i++)
            {
                Node n = cl3.Items[i];
                if (!allNodes.Contains(n))
                    allNodes.Add(n);

                for (int j = 0; j < cl3.Items.Count; j++)
                {
                    Node m = cl3.Items[j];
                    string s = n.ToString() + ":::" + m.ToString();
                    if (!dict.ContainsKey(s))
                        dict.Add(s, 1);
                }
            }

            // Were there all combinations mentioned?
            for (int i = 0; i < allNodes.Count; i++)
            {
                Node n = allNodes[i];
                for (int j = 0; j < allNodes.Count; j++)
                {
                    Node m = allNodes[j];

                    string s = n.ToString() + ":::" + m.ToString();
                    if (!dict.ContainsKey(s))
                        return;
                }
            }

            

            // cl1 := cl1 + cl2
            for (int i = 0; i < cl2.Items.Count; i++)
                if (!cl1.Items.Contains(cl2.Items[i]))
                    cl1.Items.Add(cl2.Items[i]);

            // cl1 := cl1 + cl3
            for (int i = 0; i < cl3.Items.Count; i++)
                if (!cl1.Items.Contains(cl3.Items[i]))
                    cl1.Items.Add(cl3.Items[i]);

            cl2.Clear();
            cl3.Clear();

            textBoxActions.Text += "Joined:" + Environment.NewLine + cl1.ToString() + Environment.NewLine;
            ScrollToEnd();
        }







        private void OneStep3()
        {   // Choose two different clusters
            int idx1 = m_Rnd.Next(m_Clusters.Items.Count);
            int idx2 = m_Rnd.Next(m_Clusters.Items.Count);

            // If the first one is less interesting, move it toward the end.
            Cluster c1 = m_Clusters.Items[idx1];
            Cluster c2 = m_Clusters.Items[idx2];

            if (c1.LD < c2.LD)
            {
                m_Clusters.Items[idx1] = c2;
                m_Clusters.Items[idx2] = c1;
            }
        }









        private void buttonBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            openFileDialog.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
               textBoxFilename.Text = openFileDialog.FileName;
        }


        private void buttonStore_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog = new SaveFileDialog();
            saveFileDialog.OverwritePrompt = true;
            saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            saveFileDialog.Filter = "Textdateien (*.txt)|*.txt|Alle Dateien (*.*)|*.*";
            if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                m_Clusters.SaveToFile(saveFileDialog.FileName);
            }
        }

        private void ScrollToEnd()
        {
            textBoxActions.SelectionStart = 0;             
            textBoxActions.Select(textBoxActions.TextLength, 0);  
            textBoxActions.ScrollToCaret();               
        }


    }
}