//------------------------------------------------------------------------------
// Purpose: Settings
// Date   : 16.9.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Collections.Generic;

namespace Unarch5
{
    public partial class FormOptionen : Form
    {
        public FormOptionen()
        {
            InitializeComponent();
        }





        private void LoadSettings()
        {
            Configuration c = Configuration.GetInstance();
            this.Location = c.FormOptionenSettings.Location;
            this.Size     = c.FormOptionenSettings.Size;

            numericUpDownFileHistory.Value      = c.MaxFileHistory;
            numericUpDownQueryHistory.Value     = c.MaxQueryHistory;
            numericUpDownDirectoryHistory.Value = c.MaxDirectoryHistory;

            checkedListBoxClusterFiles.Items.Clear();
            List<string> lis = c.KnownClusterFiles;
            for (int i = 0; i < lis.Count; i++)
                AddToClusterFilesList((string)lis[i]);

            lis = c.ActiveClusterFiles;
            for (int i = 0; i < lis.Count; i++)
            {
                int idx = checkedListBoxClusterFiles.Items.IndexOf(lis[i]);
                if (idx >= 0)
                   checkedListBoxClusterFiles.SetItemCheckState(idx, CheckState.Checked);
            }


            AddToClusterFilesList(Path.GetDirectoryName(Application.ExecutablePath) + "\\database\\cluster.txt");
            AddToClusterFilesList(Path.GetDirectoryName(Application.ExecutablePath) + "\\database\\cluster_public.txt");

            this.checkBoxAskBeforeDelete.Checked     = c.AskBeforeDelete;
            this.checkBoxDisplayDistribution.Checked = c.DisplayDistribution;
            this.checkBoxLastRecord.Checked          = c.GotoLastRecord;
            this.checkBoxClassicalMenu.Checked       = c.ClassicalDatabaseView;

            this.cbUseSpecialBrowser.Checked         = c.UseSpecialBrowser;
            this.textBoxBrowser.Text                 = c.HttpBrowserExe;
            cbUseSpecialBrowser_CheckedChanged(null, null);
        }


        private void StoreSettings()
        {
            Configuration c = Configuration.GetInstance();

            c.FormOptionenSettings.Location = this.Location;
            c.FormOptionenSettings.Size = this.Size;


            c.MaxFileHistory      = (int)Math.Round(numericUpDownFileHistory.Value);
            c.MaxQueryHistory     = (int)Math.Round(numericUpDownQueryHistory.Value);
            c.MaxDirectoryHistory = (int)Math.Round(numericUpDownDirectoryHistory.Value);

            c.ActiveClusterFiles.Clear();
            c.KnownClusterFiles.Clear();
            for (int i = 0; i < checkedListBoxClusterFiles.Items.Count; i++)
            {
                string filename = (string)checkedListBoxClusterFiles.Items[i];
                c.KnownClusterFiles.Add(filename);                   
                if (checkedListBoxClusterFiles.GetItemChecked(i))
                   c.ActiveClusterFiles.Add(filename);                   
            }

            c.AskBeforeDelete       = this.checkBoxAskBeforeDelete.Checked;
            c.DisplayDistribution   = this.checkBoxDisplayDistribution.Checked;
            c.GotoLastRecord        = this.checkBoxLastRecord.Checked;
            c.ClassicalDatabaseView = this.checkBoxClassicalMenu.Checked;

            c.UseSpecialBrowser     = cbUseSpecialBrowser.Checked;
            c.HttpBrowserExe        = this.textBoxBrowser.Text;
        }


        private void buttonCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            StoreSettings();
            Close();
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Do a caseinsensitive search in checkedListBoxClusterFiles.Items.
        /// </summary>
        /// <param name="filename">The entry to search for.</param>
        /// <returns>The index of the found entry or -1</returns>
        ///---------------------------------------------------------------------
        private int FindFilenameInCheckedListBox(string filename)
        {
            filename = filename.ToUpper();

            for (int i = 0; i < checkedListBoxClusterFiles.Items.Count; i++)
            {
                if (((string)checkedListBoxClusterFiles.Items[i]).ToUpper() == filename)
                    return i;
            }

            return -1;
        }



        private void AddToClusterFilesList(string filename)
        {
            int idx = FindFilenameInCheckedListBox(filename);

            if (idx < 0)
            {
                checkedListBoxClusterFiles.Items.Add(filename);
                idx = FindFilenameInCheckedListBox(filename);

                checkedListBoxClusterFiles.SetItemCheckState(idx, CheckState.Checked);
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// If a file in checkedListBoxClusterFiles doesn't exist, uncheck it.
        /// </summary>
        ///---------------------------------------------------------------------
        private void SetCheckedStateForExistingFiles()
        {
            for (int i = 0; i < checkedListBoxClusterFiles.Items.Count; i++)
            {
                string filename = (string)checkedListBoxClusterFiles.Items[i];

                if (!File.Exists(filename))
                   checkedListBoxClusterFiles.SetItemCheckState(i,CheckState.Unchecked);
            }
        }



        private void RemoveFromClusterFilesList()
        {
            int idx = checkedListBoxClusterFiles.SelectedIndex;

            if (idx >= 0)
               checkedListBoxClusterFiles.Items.RemoveAt(idx);
        }


        private void buttonAdd_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog openFileDialog = new OpenFileDialog())
            {
                openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                openFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";
                if (openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    AddToClusterFilesList(openFileDialog.FileName);
                }
            }
        }


        private void buttonRemove_Click(object sender, EventArgs e)
        {
            RemoveFromClusterFilesList();
        }


        private void FormOptionen_Load(object sender, EventArgs e)
        {
            LoadSettings();
            SetCheckedStateForExistingFiles();
        }

        private void buttonBrowseForBrowser_Click(object sender, EventArgs e)
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
            openFileDialog.Filter = "Ausfhrbare Dateien (*.exe)|*.exe|Alle Dateien (*.*)|*.*";
            if (openFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                textBoxBrowser.Text = openFileDialog.FileName;
            }
        }

        private void cbUseSpecialBrowser_CheckedChanged(object sender, EventArgs e)
        {
            if (cbUseSpecialBrowser.Checked)
            {
                textBoxBrowser.Enabled = true;
                buttonBrowseForBrowser.Enabled = true;
            }
            else
            {
                textBoxBrowser.Enabled = false;
                buttonBrowseForBrowser.Enabled = false;
            }
        }

        private void buttonEditFileHistory_Click(object sender, EventArgs e)
        {
            Configuration c = Configuration.GetInstance();

            FormEditStringList f = new FormEditStringList();
            f.Input = c.FileHistorySettings.History;
            f.ShowDialog();
            c.FileHistorySettings.History = f.Output;
        }

        private void buttonEditDirectoryHistory_Click(object sender, EventArgs e)
        {
            Configuration c = Configuration.GetInstance();

            FormEditStringList f = new FormEditStringList();
            f.Input = c.Directories;
            f.ShowDialog();
            c.Directories = f.Output;
        }

        private void buttonEditQueryHistory_Click(object sender, EventArgs e)
        {
            Configuration c = Configuration.GetInstance();

            FormEditStringList f = new FormEditStringList();
            f.Input = c.History;
            f.ShowDialog();
            c.History = f.Output;

        }

    }
}