//------------------------------------------------------------------------------
// Purpose: 
// Date   : 09. Nov. 2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Text.RegularExpressions;
using Fusym;
using System.Threading;
using System;

namespace Unarch5
{
    public partial class FormPathToHere : Form
    {
        private string m_Query;


        private IDocument m_LineRecord;

        public IDocument lrec
        {
            get { return m_LineRecord;; }
            set { m_LineRecord = value; }
        }


        public string Query
        {
            get { return m_Query; }
            set { m_Query = value; }
        }


        public FormPathToHere()
        {
            InitializeComponent();
        }


        private void FormPathToHere_Load(object sender, EventArgs e)
        {
            textBoxQuery.Text = m_Query;
        }

        private List<string> GetQuerySymbols()
        {
            List<string> result = new List<string>();

            if (Query == null)
                return result;

            Regex r = new Regex(@"[^\s]+");
            MatchCollection m = r.Matches(Query);
            foreach (Match ma in m)
            {
                string sym = ma.ToString();
                result.Add(sym);
            }

            return result;
        }



        private void LoadClusterFiles(SemanticNetwork cm)
        {
            Configuration c = Configuration.GetInstance();
            List<string> activeFiles = c.ActiveClusterFiles;

            foreach (string filename in activeFiles)
            {
                //statusStrip.Text = filename;
                Application.DoEvents();
                try
                {
                    cm.LoadFromFile(filename);
                }
                catch
                {
                }
            }

            Application.DoEvents();
        }




        private string FindSymbolOfDocumentWithMinimumDistance(SemanticNetwork cm, string querySymbol)
        {
            string result = "";
            int minDistance = -1;

            string s = m_LineRecord.GetFirstSymbol().ToUpper();
            while (s != "")
            {
                string start = querySymbol;
                string end = s;
                List<Node> thePath = new List<Node>();
                if (cm.FindPath(start, end, thePath))
                {
                    if ((thePath.Count < minDistance) || (minDistance < 0))
                    {
                        minDistance = thePath.Count;
                        result = s;
                    }
                }
                s = m_LineRecord.GetNextSymbol().ToUpper();
            }

            return result;
        }





        private void EnumeratePaths()
        {
            SemanticNetwork cm = new SemanticNetwork(Configuration.encoding);
            LoadClusterFiles(cm);

            List<string> inputSymbols = GetQuerySymbols();

            textBoxResult.Clear();

            for (int i = 0; i < inputSymbols.Count; i++)
                {
                    string start = inputSymbols[i];
                    string end = FindSymbolOfDocumentWithMinimumDistance(cm, start);
                    List<Node> thePath = new List<Node>();
                    if (cm.FindPath(start, end, thePath))
                    {
                        for (int k = 0; k < thePath.Count; k++)
                            textBoxResult.AppendText(thePath[k] + " " + thePath[k].Neighbours.Count.ToString() + System.Environment.NewLine);
                        textBoxResult.AppendText(System.Environment.NewLine);
                    }
                }
        }


        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }


        private void DoIt()
        {
            Cursor oldCursor        = Cursor.Current;
            bool   oldUseWaitCursor = Application.UseWaitCursor;


            Cursor.Current            = Cursors.WaitCursor;
            Application.UseWaitCursor = true;

            try
            {
              EnumeratePaths();
            }
            finally
            {
                Cursor.Current            = oldCursor;
                Application.UseWaitCursor = oldUseWaitCursor;
            }
        }

        private void FormPathToHere_Shown(object sender, EventArgs e)
        {
            DoIt();
        }
    }
}