using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fusym;

namespace Unarch5
{
    public partial class FormScrollBars : Form
    {
        private List<ScrollBarForSymbol> m_ScrollBars = new List<ScrollBarForSymbol>();
        private LinearCombinationOfSymbols m_LinearCombination;

        public FormScrollBars()
        {
            InitializeComponent();
        }

        private void textBoxQuery_TextChanged(object sender, EventArgs e)
        {

        }

        private void FormScrollBars_Load(object sender, EventArgs e)
        {

        }

        private void textBoxQuery_KeyUp(object sender, KeyEventArgs e)
        {
            for (int i = 0; i < m_ScrollBars.Count; i++)
            {
                ScrollBarForSymbol sb0 = m_ScrollBars[i];
                m_ScrollBars[i] = null;
                sb0.Dispose();
            }


            m_ScrollBars.Clear();
            List<string> strings = StringUtils.GetQuerySymbols(textBoxQuery.Text);
            for (int i = 0; i < strings.Count; i++)
            {
                string s = strings[i];
                ScrollBarForSymbol sb = new ScrollBarForSymbol();
                sb.Width = panel.Width - 30;
                sb.Top = i * sb.Height;
                sb.Parent = panel;
                sb.Text = s;
                sb.Value = 50;
                m_ScrollBars.Add(sb);
                sb.ValueChanged += trackBar_ValueChanged;
            }





            GlobalClusterManager gcm = GlobalClusterManager.GetInstance();
            SemanticNetwork sn = gcm.SemanticNetwork;
            m_LinearCombination = new LinearCombinationOfSymbols(sn, strings);
        }


        private void trackBar_ValueChanged(object sender, EventArgs e)
        {
            int sum = 0;
            for (int i = 0; i < m_ScrollBars.Count; i++)
            {
                ScrollBarForSymbol sb0 = m_ScrollBars[i];
                sum += sb0.Value;
            }
            Text = sum.ToString();

            CalculateOutput();
        }

        private void CalculateOutput()
        {
            if (m_LinearCombination == null)
                return;

            List<int> vector = Vector();
            List<NodeAndMark> list = m_LinearCombination.Evaluate(vector);
            StringBuilder sb = new StringBuilder();

            int i = 0;
            foreach (NodeAndMark nm in list)
            {
                sb.AppendLine(nm.Node.ToString());
                i++;
                if (i >= 30)
                    break;
            }
            textBoxOutput.Text = sb.ToString();
        }




        private List<int> Vector()
        {
            List<int> result = new List<int>();

            for (int i = 0; i < m_ScrollBars.Count; i++)
            {
                ScrollBarForSymbol sb0 = m_ScrollBars[i];
                result.Add(sb0.Value);
            }

            return result;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            Close();
        }
    }


    public class LinearCombinationOfSymbols
    {
        private Dictionary<Node, EvaluationVector> m_Dict = new Dictionary<Node, EvaluationVector>();

        ///---------------------------------------------------------------------
        /// <summary>
        /// Constructor.
        /// </summary>
        ///---------------------------------------------------------------------
        public LinearCombinationOfSymbols(SemanticNetwork semanticNetwork, List<string> queryStrings)
        {

            for (int i = 0; i < queryStrings.Count; i++)
            {
                string s = queryStrings[i];
                List<Node> list = semanticNetwork.Circle(s, 100000, 9);

                foreach (Node n in list)
                {
                    EvaluationVector vector;
                    if (!m_Dict.TryGetValue(n, out vector))
                    {
                        vector = new EvaluationVector(queryStrings.Count, 9);
                        m_Dict[n] = vector;
                    }

                    vector[i] = n.Mark;
                }
            }
        }


        public List<NodeAndMark> Evaluate(List<int> factors)
        {
            int count = factors.Count;

            List<NodeAndMark> result = new List<NodeAndMark>();
            foreach (KeyValuePair<Node, EvaluationVector> pair in m_Dict)
            {
                EvaluationVector vector = pair.Value;

                int sum = 0;
                for (int i = 0; i < count; i++)
                    sum += vector[i] * factors[i];


                NodeAndMark nm = new NodeAndMark();
                nm.Node = pair.Key;
                nm.Mark = sum;
                result.Add(nm);
            }

            result.Sort(NodeAndMark.CmpNodeAndMarkAsc);
            return result;
        }
    }
}