//------------------------------------------------------------------------------
// Purpose: Search for filenames by its parts.
// Date   : 03.Dec.2009
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Diagnostics;
using Fusym;
using System.Threading;

namespace Unarch5
{
    public partial class FormSearchFilename : Form
    {
        private ValuedDocList docList;
        private DataTable dt;
        private string m_Keywords = "";
        private string m_DirectoryRoot = "";

        public FormSearchFilename()
        {
            InitializeComponent();
        }


        public FormSearchFilename(string keywords, string directoryRoot)
        {
            InitializeComponent();

            m_Keywords      = keywords;
            m_DirectoryRoot = directoryRoot;

            labelAction.Text = "Verzeichnis wird gelesen.";
            labelQuery.Text         = keywords;
            labelDirectoryRoot.Text = directoryRoot;
            Application.DoEvents();
        }






        private void buttonClose_Click(object sender, EventArgs e)
        {
            backgroundWorker.CancelAsync();
            while (backgroundWorker.IsBusy)
                Application.DoEvents();

            Close();
        }





        ///---------------------------------------------------------------------
        /// <summary>
        /// Loop through m_ValuedDocList. Copy some data to the table dt.
        /// </summary>
        ///---------------------------------------------------------------------
        private void CreateTable(ValuedDocList doclist)
        {
            progressBar.Minimum = 0;
            progressBar.Maximum = doclist.Count;
            labelAction.Text    = "Tabelle wird gefllt.";
            Application.DoEvents();


            dt = null;
            dt = new DataTable();

            dt.Clear();
            dt.Columns.Clear();

            dt.Columns.Add(new DataColumn("Id"      , typeof(int)));
            dt.Columns.Add(new DataColumn("Filename", typeof(string)));
            dt.Columns.Add(new DataColumn("Keywords", typeof(string)));
            dt.Columns.Add(new DataColumn("Value"   , typeof(double)));
            dt.Columns.Add(new DataColumn("Date"    , typeof(DateTime)));
            //dt.Columns.Add(new DataColumn("Mark"    , typeof(bool)));

            int j = 0;

            for (int i = 0; i < doclist.Count; i++)
            {
                j = i;

                progressBar.Value = i;
                progressBar.Update();


                ValuedDocument vdoc = doclist[i];
                object o = vdoc.Object;
                DateTime d; 

                /*
                if (o is IRecordWithDate)
                    d = (o as IRecordWithDate).Date();
                else
                    d = DateTime.MinValue;
                */

                StructuredString s = (StructuredString)vdoc.Object;

                FileInfo fi = new FileInfo(s.Text);
                d = fi.LastWriteTime;

                AddNewRow(
                          dt,
                          i,
                          s.Text,
                          vdoc.theString,
                          vdoc.Value,
                          d,
                          vdoc.Mark
                         );
            }


            dataGridView.DataSource = dt;
            dataGridView.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;

            dataGridView.Columns["Id"   ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Date" ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            //dataGridView.Columns["Mark" ].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Value"].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            dataGridView.Columns["Value"].DefaultCellStyle.Format = "N3";

            progressBar.Value = progressBar.Maximum;
            progressBar.Update();
            progressBar.Value = 0;
            progressBar.Update();
            labelAction.Text = "";
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Add a new row.
        /// </summary>
        ///---------------------------------------------------------------------
        private void AddNewRow(
                               DataTable dt,
                               int id,
                               string filename,
                               string keywords,
                               double value,
                               DateTime d,
                               bool mark
                              )
        {
            DataRow dr = dt.NewRow();


            dr[0] = id;
            dr[1] = filename;
            dr[2] = keywords;
            dr[3] = value;
            dr[4] = d;
        //  dr[5] = mark;

            try
            {
                dt.Rows.Add(dr);
            }
            catch (Exception e)
            {
                MessageBox.Show("Main: " + e.Message);
            }
        }





        private void backgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            progressBar.Minimum = 0;
            progressBar.Maximum = 100;


            //-------------------------------------------------

            FilenameDocuments fd = new FilenameDocuments(m_DirectoryRoot);

            docList = new ValuedDocList();

            FilenameSearchCustomization f = new FilenameSearchCustomization(
                                                                            fd,
                                                                            docList,
                                                                            m_Keywords
                                                                           );

            f.Search();
        }

        private void FormSearchFilename_Load(object sender, EventArgs e)
        {
            backgroundWorker.RunWorkerAsync();
            LoadSettings();
        }

        private void backgroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressBar.Value = e.ProgressPercentage;
        }

        private void backgroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            CreateTable(docList);
        }




        private string FieldAsString(string fieldName)
        {
            if (dataGridView.DataSource == null)
                return "";

            CurrencyManager cm = (CurrencyManager)this.BindingContext[dataGridView.DataSource, dataGridView.DataMember];

            if (cm.Position < 0)
                return "";

            DataRowView rv = (DataRowView)cm.Current;
            DataRow r = rv.Row;

            Object o = r[fieldName];

            if (o is System.DBNull)
                return "";

            string result = (string)o;
            return result;
        }


        

        private void buttonOpenFolder_Click(object sender, EventArgs e)
        {
            string path;

            try
            {
                path = Path.GetDirectoryName(FieldAsString("Filename"));
            }
            catch
            {
                path = "";
            }


            if (Directory.Exists(path))
                Process.Start(path);
            else
                MessageBox.Show(path + " kann nicht geffnet werden.");
        }

        private void buttonOpen_Click(object sender, EventArgs e)
        {
            string file = FieldAsString("Filename");
            if (File.Exists(file))
                Process.Start(file);
            else
                MessageBox.Show(file + " kann nicht geffnet werden.");
        }

  

        private void dataGridView_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            string file = FieldAsString("Filename");



            if (File.Exists(file))
                Process.Start(file);
            else
            {
                string path;

                try
                {
                    path = Path.GetDirectoryName(file);
                }
                catch
                {
                    path = "";
                }

                if (Directory.Exists(path))
                    Process.Start(path);
                else
                    MessageBox.Show(file + " kann nicht geffnet werden.");
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load settings from the config file
        /// </summary>
        ///---------------------------------------------------------------------
        private void LoadSettings()
        {
            Configuration c = Configuration.GetInstance();
            this.Location = c.FormSearchFilenameSettings.Location;
            this.Size     = c.FormSearchFilenameSettings.Size;
        }


        private void StoreSettings()
        {
            Configuration c = Configuration.GetInstance();
            c.FormSearchFilenameSettings.Location = this.Location;
            c.FormSearchFilenameSettings.Size = this.Size;
        }

        private void FormSearchFilename_FormClosed(object sender, FormClosedEventArgs e)
        {
            StoreSettings();
        }

        private void FormSearchFilename_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

    }
}