//------------------------------------------------------------------------------
// Purpose: Search for symbols (rather than for documents)
// Date   : 26.Apr.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fusym;
using System.Text.RegularExpressions;
using System.Reflection;
using System.IO;
using System.Runtime.InteropServices;

namespace Unarch5
{



    public partial class FormSearchSymbol : Form, IDistributionProvider, ILoadingProgressReceiver
    {
        private ValuedDocList   m_ValuedDocList = new ValuedDocList();
        private int             m_DistributionProviderIndex;
        private AssociationFiles m_AssociationFiles;

        [DllImport("user32.dll")]
        public static extern bool LockWindowUpdate(IntPtr hWndLock); 

        public FormSearchSymbol()
        {
            InitializeComponent();

            Configuration c = Configuration.GetInstance();
            List<string> activeFiles = c.ActiveClusterFiles;

            m_AssociationFiles = new AssociationFiles(activeFiles, Configuration.encoding, this);
        }



        /*
        private EvaluationMethod StringToEvaluationMethod(string s)
        {
            if (s == "Sum")
               return EvaluationMethod.Sum;
            if (s == "SquareSum")
               return EvaluationMethod.SquareSum;
            if (s == "Fusym")
               return EvaluationMethod.Fusym;
           throw new Exception("FormSearchSymbol.StringToEvaluationMethod: illegal argument <" + s + ">");
        } */     




        
        private class MyComparer : IComparer
        {
            int IComparer.Compare(object x, object y)
            {
                if ((double)x > (double)y)
                    return 1;
                else
                    if ((double)x < (double)y)
                        return -1;
                    else
                        return 1; // Trick
            }
        }










        private void btnSearch_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            tbResult.Clear();
            tbResult.Update();

            if ((cbQuery.Items.IndexOf(cbQuery.Text) < 0) && (cbQuery.Text.Length > 0))
                cbQuery.Items.Insert(0, cbQuery.Text); // History

            int maxSym = 10000;
            int maxDepth = 3;


            try
            {
                maxSym = Int32.Parse(tbMaxSym.Text);
            }
            catch
            {
                // Keep Default
            }



            try
            {
                maxDepth = (int)numMaxDepth.Value;
            }
            catch
            {
                // Keep Default
            }


            bool bShowEval = cbBewertung.Checked;
            bool bShowClusterCoefficient = cbClusterCoefficient.Checked;
            bool bGroupChange = cbGruppenwechsel.Checked;



            EvaluationMethod evm = (EvaluationMethod)cbMethod.SelectedIndex;

            DoIt(maxDepth, maxSym, evm, bShowEval, bShowClusterCoefficient, bGroupChange);
            Cursor.Current = Cursors.Default;
        }








        private void buttonSearchSymbolRandomWalk_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            tbResult.Clear();
            tbResult.Update();

            if ((cbQuery.Items.IndexOf(cbQuery.Text) < 0) && (cbQuery.Text.Length > 0))
                cbQuery.Items.Insert(0, cbQuery.Text); // History

            int maxSym = 1000000;
            int maxDepth = 10;


            try
            {
                maxSym = Int32.Parse(tbMaxSym.Text);
            }
            catch
            {
                // Keep Default
            }



            try
            {
                maxDepth = (int)numMaxDepth.Value;
            }
            catch
            {
                // Keep Default
            }


            bool bShowEval    = cbBewertung.Checked;
            bool bGroupChange = cbGruppenwechsel.Checked;


            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);

            RandomWalk rw = new RandomWalk(semanticNetwork);
            List<Node> nodes = rw.SearchSymbol(inputSymbols, maxSym, maxDepth);

            if (tabControl.SelectedTab == tabPageMemo)
                FillTextBox(nodes);
            else
                FillCheckboxes(nodes);

            Cursor.Current = Cursors.Default;
        }






















        private void DoIt(
                          int maxDepth,
                          int maxSym,
                          EvaluationMethod evm,
                          bool bShowEval,
                          bool bShowClusterCoefficient,
                          bool bGroupChange
                         )
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);
            IDocumentEvaluator de = semanticNetwork.NewDocumentEvaluator(inputSymbols, maxDepth, maxSym, evm, false);



            if (tabControl.SelectedTab == tabPageCheckboxes)
            {
                FillCheckboxes(de);
                return;
            }
            


            
            
            
            List<string> associatedSymbols = de.AssociatedSymbols;


            m_ValuedDocList.Clear();

            for (int i = 0; i < associatedSymbols.Count; i++)
            {
//                OneStringDocument doc = new OneStringDocument(associatedSymbols[i], semanticNetwork);
                OneStringDocument doc = new OneStringDocument(associatedSymbols[i]);
                double value = de.Evaluate(doc);
                m_ValuedDocList.Add(doc, value);
            }








            StringBuilder sb = new StringBuilder("");

            object oldEvaluation = "";
            tbResult.SuspendLayout();
            for (int i = 0; i < m_ValuedDocList.Count; i++)
            {
                ValuedDocument vdoc = m_ValuedDocList[i];

                double evaluation = vdoc.Value;
                string symbol     = ((OneStringDocument)vdoc.Object).Text;

                if (bGroupChange && (oldEvaluation.ToString() != evaluation.ToString())
                   )
                {
                    if (i != 0)
                       sb.Append(System.Environment.NewLine);

                    oldEvaluation = evaluation;
                }

                string s;
                if (bShowEval)
                    s = evaluation.ToString("N3") + " - " + symbol;
                else
                    s = symbol;

                if (bShowClusterCoefficient)
                {
                    Node n = semanticNetwork.AddSymbol(symbol);
                    s += " " + n.ClusterCoefficient().ToString("N3");
                }

                sb.AppendLine(s);
            }


            tbResult.AppendText(sb.ToString());
            tbResult.ResumeLayout();

            tbResult.SelectionStart = 0;
            tbResult.ScrollToCaret();
        }







        private void FormSearchSymbol_Load(object sender, EventArgs e)
        {
            numMaxDepth.Value = 3;
            tbMaxSym.Text     = "10000";
            LoadSettings();
            ActiveControl = cbQuery;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormSearchSymbol_FormClosed(object sender, FormClosedEventArgs e)
        {
            WindowState = FormWindowState.Normal;
            StoreSettings();
        }

        private void LoadSettings()
        {
            Configuration c               = Configuration.GetInstance();
            this.Location                 = c.FormSearchSymbolSettings.Location;
            this.Size                     = c.FormSearchSymbolSettings.Size;
            this.numMaxDepth.Value        = c.FormSearchSymbolSettings.MaxDepth;
            this.tbMaxSym.Text            = c.FormSearchSymbolSettings.MaxSym.ToString();
            this.cbBewertung.Checked      = c.FormSearchSymbolSettings.DisplayValue;
            this.cbNeighbourCount.Checked = c.FormSearchSymbolSettings.NeighbourCount;
            this.cbGruppenwechsel.Checked = c.FormSearchSymbolSettings.GroupChange;
            this.cbMethod.SelectedIndex   = c.FormSearchSymbolSettings.EvaluationMethod;
            this.cbQuery.Items.Clear();
            this.cbQuery.Items.AddRange(c.History.ToArray());
        }


        private void StoreSettings()
        {
            int maxSym = 10000;
            try
            {
                maxSym = Int32.Parse(tbMaxSym.Text);
            }
            catch
            {
                // Keep Default
            }



            Configuration c = Configuration.GetInstance();
            c.FormSearchSymbolSettings.Location         = this.Location;
            c.FormSearchSymbolSettings.Size             = this.Size;
            c.FormSearchSymbolSettings.MaxDepth         = (int)this.numMaxDepth.Value;
            c.FormSearchSymbolSettings.MaxSym           = maxSym;
            c.FormSearchSymbolSettings.DisplayValue     = this.cbBewertung.Checked;
            c.FormSearchSymbolSettings.NeighbourCount   = this.cbNeighbourCount.Checked;
            c.FormSearchSymbolSettings.GroupChange      = this.cbGruppenwechsel.Checked;
            c.FormSearchSymbolSettings.EvaluationMethod = this.cbMethod.SelectedIndex;
            c.History.Clear();
            AddToStringList(c.History, cbQuery.Items);
            c.RestrictArrayListLength(c.History, c.MaxQueryHistory);

        }


        private void AddToStringList(List<string> list, ComboBox.ObjectCollection objects)
        {
            foreach (string s in objects)
            {
                list.Add(s);
            }

            return;
        }


        private void cbQuery_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Char.IsLetter(e.KeyChar))
               e.KeyChar = Char.ToUpper(e.KeyChar);
        }


        public void Progress(string item, int percent)
        {
            toolStripStatusLabel.Text = item;
            Application.DoEvents();
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// Generate all paths between each of the input symbols
        /// </summary>
        ///---------------------------------------------------------------------
        private void EnumeratePaths()
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);


            Application.DoEvents();
            LockWindowUpdate(tbResult.Handle);
            tbResult.Clear();
            for (int i = 0; i < inputSymbols.Count - 1; i++)
                for (int j = i + 1; j < inputSymbols.Count; j++)
                {
                    string start = inputSymbols[i];
                    string end   = inputSymbols[j];
                    List<Node> thePath = new List<Node>();
                    if (semanticNetwork.FindPath(start, end, thePath))
                    {
                        for (int k = 0; k < thePath.Count; k++)
                        {
                            string neighbourCount = "";
                            if (cbNeighbourCount.Checked)
                               neighbourCount = " " + thePath[k].Neighbours.Count.ToString();
                        
                        
                            tbResult.AppendText(thePath[k] + neighbourCount + System.Environment.NewLine);
                        }
                        tbResult.AppendText(System.Environment.NewLine);
                    }
                }
            LockWindowUpdate(IntPtr.Zero);
        }


        private void EnumeratePathsEdgeSum()
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            SemanticDistanceByEdgeNumber s = new SemanticDistanceByEdgeNumber(semanticNetwork);
            
            /*
            Node start = cm.GetNode("BUCH");
            Node dest  = cm.GetNode("EIS");
            
            s.CreateAscendingValueAssignment(start, dest);
             */





            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);

            Application.DoEvents();

            LockWindowUpdate(tbResult.Handle);
            tbResult.Clear();
            for (int i = 0; i < inputSymbols.Count - 1; i++)
                for (int j = i + 1; j < inputSymbols.Count; j++)
                {
                    string start = inputSymbols[i];
                    string end   = inputSymbols[j];
                    
                    
                    Node startNode = semanticNetwork.GetNode(start);
                    Node destNode  = semanticNetwork.GetNode(end);
                    
                    //s.CreateAscendingValueAssignment(startNode, destNode);
                    
                    
                    List<Node> thePath = new List<Node>();
                    if (s.FindPath(start, end, thePath))
                    {
//                        for (int k = 0; k < thePath.Count; k++)
  //                          tbResult.AppendText(thePath[k] + " " + thePath[k].Neighbours.Count.ToString() + System.Environment.NewLine);
                            
                        for (int k = 0; k < thePath.Count; k++)
                        {
                            string neighbourCount = "";
                            if (cbNeighbourCount.Checked)
                               neighbourCount = " " + thePath[k].Neighbours.Count.ToString();
                        
                        
                            tbResult.AppendText(thePath[k] + neighbourCount + System.Environment.NewLine);
                        }
                            
                            
                        tbResult.AppendText(System.Environment.NewLine);
                    }
                }
            LockWindowUpdate(IntPtr.Zero);
        }








        ///---------------------------------------------------------------------
        /// <summary>
        /// Determine distances between each pair of symbols.
        /// </summary>
        ///---------------------------------------------------------------------
        private void EnumerateDistances()
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);

            tbResult.Clear();
            for (int i = 0; i < inputSymbols.Count - 1; i++)
                for (int j = i + 1; j < inputSymbols.Count; j++)
                {
                    string start = inputSymbols[i];
                    string end   = inputSymbols[j];
                    List<Node> thePath = new List<Node>();
                    if (semanticNetwork.FindPath(start, end, thePath))
                    {
                        for (int k = 0; k < thePath.Count; k++)
                        {
                            string neighbourCount = "";
                            if (cbNeighbourCount.Checked)
                               neighbourCount = " " + thePath[k].Neighbours.Count.ToString();
                        
                        
                            tbResult.AppendText(thePath[k] + neighbourCount + System.Environment.NewLine);
                        }
                        tbResult.AppendText(System.Environment.NewLine);
                    }
                }
        }












        private void buttonPath_Click(object sender, EventArgs e)
        {
            Cursor oldCursor = Cursor.Current;
            Cursor.Current = Cursors.WaitCursor;
       
            EvaluationMethod evm = (EvaluationMethod)cbMethod.SelectedIndex;
            
                if (evm != EvaluationMethod.EdgeSum)
                   EnumeratePaths();
                else
                   EnumeratePathsEdgeSum();                
                
            Cursor.Current = oldCursor;
        }

        private void cbQuery_Enter(object sender, EventArgs e)
        {
            this.AcceptButton = btnSearch;
        }

        private void cbQuery_Leave(object sender, EventArgs e)
        {
            this.AcceptButton = null;
        }




        private void FillCheckboxes(IDocumentEvaluator de)
        {
            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);
            List<string> associatedSymbols = de.AssociatedSymbols;



            ValuedDocList vdl = new ValuedDocList();

            foreach (string s in associatedSymbols)
            {
                OneStringDocument doc = new OneStringDocument(s);
                double value = de.Evaluate(doc);
                vdl.Add(doc, value);
            }

            LockWindowUpdate(checkedListBoxSearchSymbols.Handle);
            checkedListBoxSearchSymbols.Items.Clear();
            for (int i = 0; i < vdl.Count; i++)
            {
                ValuedDocument vdoc = vdl[i];
                checkedListBoxSearchSymbols.Items.Add(((OneStringDocument)vdoc.Object).Text);
            }



            //--- Set all strings of the query checked ---
            foreach (string s in inputSymbols)
            {
                int idx = checkedListBoxSearchSymbols.FindStringExact(s);
                if (idx >= 0)
                    checkedListBoxSearchSymbols.SetItemChecked(idx, true);
            }
            LockWindowUpdate(IntPtr.Zero);
        }





        private void FillQueryText(ItemCheckEventArgs e)
        {
            cbQuery.Text = "";

            for (int i = 0; i < checkedListBoxSearchSymbols.Items.Count; i++)
            {
                if (
                    (
                     checkedListBoxSearchSymbols.GetItemChecked(i) &&
                     (((e.Index != i)) || (e.NewValue == CheckState.Checked)) // is not going to be unchecked
                    )

                    ||

                    ((e.Index == i) && (e.NewValue == CheckState.Checked)) // is going to be checked
                   )
                    cbQuery.Text += checkedListBoxSearchSymbols.Items[i] + " ";
            }
        }




        private void checkedListBoxSearchSymbols_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            FillQueryText(e);
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Things to do when the tabpage is changed.
        /// </summary>
        ///---------------------------------------------------------------------
        private void ChangeTabIndex()
        {
            tbResult.Clear();
            checkedListBoxSearchSymbols.Items.Clear();

            groupBoxDisplay         . Visible = (tabControl.SelectedTab == tabPageMemo);
            buttonPath              . Visible = (tabControl.SelectedTab == tabPageMemo);
            buttonDistances         . Visible = (tabControl.SelectedTab == tabPageMemo);
            buttonSearchSimilarWords. Visible = (tabControl.SelectedTab == tabPageMemo);
        }

        private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
        {
            ChangeTabIndex();
        }

        public double GetFirstDistributionValue()
        {
            m_DistributionProviderIndex = 0;
            return GetNextDistributionValue();
        }
        
        public double GetNextDistributionValue()
        {
            if (m_DistributionProviderIndex < m_ValuedDocList.Count) 
               return m_ValuedDocList[m_DistributionProviderIndex++].Value;
            else 
               return -1.0;
        }



        private void buttonDistances_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            tbResult.Clear();
            tbResult.Update();

            if ((cbQuery.Items.IndexOf(cbQuery.Text) < 0) && (cbQuery.Text.Length > 0))
               cbQuery.Items.Insert(0, cbQuery.Text); // History

            int maxSym = 10000;
            int maxDepth = 3;


            try
            {
                maxSym = Int32.Parse(tbMaxSym.Text);
            }
            catch
            {
                // Keep Default
            }



            try
            {
                maxDepth = (int)numMaxDepth.Value;
            }
            catch
            {
                // Keep Default
            }


            EvaluationMethod evm = (EvaluationMethod)cbMethod.SelectedIndex;

            ShowDistances(maxDepth, maxSym, evm);
            Cursor.Current = Cursors.Default;
        }






        ///---------------------------------------------------------------------
        /// <summary>
        /// Show the semantic distances between all pairs of the query smbols.
        /// </summary>
        ///---------------------------------------------------------------------
        private void ShowDistances(
                                   int maxDepth,
                                   int maxSym,
                                   EvaluationMethod evm
                                 )
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);
            IDocumentEvaluator de = semanticNetwork.NewDocumentEvaluator(inputSymbols, maxDepth, maxSym, evm, false);

            if (de is DocumentEvaluatorRandomWalk)
                return; // zu-tun

            List<DistanceElement> list = (de as DocumentEvaluator).DistanceManager.DistanceElements();
            
            
            foreach (DistanceElement d in list)
            {
                tbResult.AppendText(d.Symbol1 + " " + d.Symbol2 + " " + d.Distance.ToString() + System.Environment.NewLine);
            }
            
            
            
            
            
            
            /*
            List<string> associatedSymbols = de.AssociatedSymbols;


            OneStringDocument doc = new OneStringDocument();
            m_ValuedDocList.Clear();

            foreach (string s in associatedSymbols)
            {
                doc.Text = s;
                double value = de.Evaluate(doc);
                m_ValuedDocList.Add(s, value);
            }







            StringBuilder sb = new StringBuilder("");

            tbResult.SuspendLayout();
            for (int i = 0; i < m_ValuedDocList.Count; i++)
            {
                ValuedDocument vdoc = m_ValuedDocList[i];

                object evaluation = vdoc.Value;
                object symbol     = (string)vdoc.Object;

                sb.Append(symbol.ToString() + System.Environment.NewLine);
            }


            tbResult.ResumeLayout();
            tbResult.AppendText(sb.ToString());

            tbResult.SelectionStart = 0;
            tbResult.ScrollToCaret();
             * 
             */
        }

        private void buttonSearchSimilarWords_Click(object sender, EventArgs e)
        {
            string s = cbQuery.Text;

           SemanticNetwork semanticNetwork = m_AssociationFiles.Get();


            List<string> list = new List<string>();
            List<Node> nodes = semanticNetwork.NodeList;

            int count = nodes.Count;
            for (int i = 0; i < count; i++)
            {
                Node n = nodes[i];
                n.Mark = LevenshteinDistance.Compute(s, n.ToString());
            }

            semanticNetwork.SortByMark();

            StringBuilder sb = new StringBuilder();

            int oldValue = 0;
            bool b1 = cbGruppenwechsel.Checked;
            bool b2 = cbBewertung.Checked;

            for (int i = 0; i < count; i++)
            {
                Node n = nodes[i];

                if (b1 && (n.Mark != oldValue))
                   sb.AppendLine();

                oldValue = n.Mark;

                if (b2)
                    sb.AppendLine(n.ToString() + " " + n.Mark.ToString());
                else
                    sb.AppendLine(n.ToString());
            }

            tbResult.Text = sb.ToString();
        }

        private void buttonRandomCircle_Click(object sender, EventArgs e)
        {
            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            int maxSym = 10000;
            int maxDepth = 3;


            try
            {
                maxSym = Int32.Parse(tbMaxSym.Text);
            }
            catch
            {
                // Keep Default
            }



            try
            {
                maxDepth = (int)numMaxDepth.Value;
            }
            catch
            {
                // Keep Default
            }

            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);


            /*
            DocumentEvaluator de = new DocumentEvaluator(semanticNetwork, inputSymbols, maxDepth, maxSym, EvaluationMethod.Sum, false);
            List<Node> nodes = new List<Node>();
            de.GetCircleRandomWalk(inputSymbols, nodes);
            nodes.Sort(SemanticNetwork.cmpMarksAscending);
            */


            RandomWalk rw = new RandomWalk(semanticNetwork);

            List<Node> nodes;

            if (radioButton1.Checked)
                nodes = rw.GetCircleRandomWalkBeta(inputSymbols, maxSym, maxDepth, cbNormalized.Checked);
            else
                nodes = rw.GetCircleRandomWalkAlpha(inputSymbols, maxSym, maxDepth, cbNormalized.Checked);







            if (tabControl.SelectedTab == tabPageMemo)
                FillTextBox(nodes);
            else
                FillCheckboxes(nodes);
        }





        private void FillTextBox(List<Node> nodes)
        {
            StringBuilder sb = new StringBuilder();

            int oldValue = 0;
            bool b1 = cbGruppenwechsel.Checked;
            bool b2 = cbBewertung.Checked;



            int count = nodes.Count;
            for (int i = 0; i < count; i++)
            {
                Node n = nodes[i];

                if (b1 && (n.Mark != oldValue))
                    sb.AppendLine();

                oldValue = n.Mark;

                if (b2)
                    sb.AppendLine(n.ToString() + " " + n.Mark.ToString());
                else
                    sb.AppendLine(n.ToString());
            }

            tbResult.Text = sb.ToString();
        }



        private void FillCheckboxes(List<Node> nodes)
        {
            List<string> inputSymbols = StringUtils.GetQuerySymbols(cbQuery.Text);

            checkedListBoxSearchSymbols.BeginUpdate();
            LockWindowUpdate(checkedListBoxSearchSymbols.Handle);

            checkedListBoxSearchSymbols.Items.Clear();
            foreach (Node n in nodes)
                checkedListBoxSearchSymbols.Items.Add(n.ToString());



            //--- Set all strings of the query checked ---
            foreach (string s in inputSymbols)
            {
                int idx = checkedListBoxSearchSymbols.FindStringExact(s);
                if (idx >= 0)
                    checkedListBoxSearchSymbols.SetItemChecked(idx, true);
            }
            checkedListBoxSearchSymbols.EndUpdate();
            LockWindowUpdate(IntPtr.Zero);
        }

        private void cbQuery_TextChanged(object sender, EventArgs e)
        {
            int idx = cbQuery.SelectedIndex;
            cbQuery.Text = cbQuery.Text.ToUpper();
            cbQuery.SelectedIndex = idx;
        }

        private void buttonClusterPaths_Click(object sender, EventArgs e)
        {
            Cursor.Current = Cursors.WaitCursor;
            tbResult.Clear();
            tbResult.Update();

            if ((cbQuery.Items.IndexOf(cbQuery.Text) < 0) && (cbQuery.Text.Length > 0))
                cbQuery.Items.Insert(0, cbQuery.Text); // History
           


            bool bShowEval    = cbBewertung.Checked;
            bool bGroupChange = cbGruppenwechsel.Checked;


            SemanticNetwork semanticNetwork = m_AssociationFiles.Get();

            SymFndRndBetweenness2 s = new SymFndRndBetweenness2(semanticNetwork);
            List<Node> nodes = s.SearchNodes(cbQuery.Text);

            if (tabControl.SelectedTab == tabPageMemo)
                FillTextBox(nodes);
            else
                FillCheckboxes(nodes);

            Cursor.Current = Cursors.Default;
        }











    }
}