//------------------------------------------------------------------------------
// Purpose: 
// Date   : 13.Jul.2006
// Author : Joachim Pimiskern
// Status : Freeware
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Fusym;
using System.Threading;
using System.Runtime.InteropServices;

namespace Unarch5
{
    public enum EditorType {All,Notizen,URLs,Cluster}

    public partial class FormStandard : Form, IDistributionProvider
    {
        DataSetState m_State;
        LineRecordFile lrf = new LineRecordFile(Configuration.encoding);


        private FormUniversalEditor  m_FormUniversalEditor;
        private IFileHistoryReceiver m_FileHistoryReceiver;
        private FormGrid formGrid;
        private Random m_Rnd = new Random();

        private ValuedDocList m_ValuedDocList = new ValuedDocList();
        private int           m_Current       = 0; // Refers to the position in m_ValuedDocList
        private bool          m_Modified;
        private string        m_Filename;
        private bool          m_GotoLastRecord;
        private EditorType    m_EditorType = EditorType.All;
        private string        m_Query;
        
        private int           m_DistributionProviderIndex;
        private DistributionBar m_DistributionBar;


        public int Current
        {
            get 
            {
                return m_Current; 
            }
            set
            {
                m_Current = value;
                ApplyCurrent();
            }
        }


        public string Query
        {
            get { return m_Query; }
            set {
                 m_Query = value.Trim();
                  toolStripStatusLabel.Text = "<" + m_Query + ">";
                }
        }
	


        public bool GotoLastRecord
        {
            get { return m_GotoLastRecord; }
            set { m_GotoLastRecord = value; }
        }


        public bool Modified
        {
            get { return m_Modified; }
            set
            {
                if (value == true)
                {
                    buttonStoreFile.Enabled = true;
                }
                else
                {
                    buttonStoreFile.Enabled = false;
                }

                m_Modified = value;
            }
        }


        public IFileHistoryReceiver FileHistoryReceiver
        {
            get { return m_FileHistoryReceiver; }
            set { m_FileHistoryReceiver = value; }
        }



        protected void Init(string filename)
        {
            InitializeComponent();
            m_FormUniversalEditor = new FormUniversalEditor();
            m_FormUniversalEditor.TopLevel = false;
            m_FormUniversalEditor.ParentForm = this;
            m_FormUniversalEditor.Parent = panelEditor;
            m_FormUniversalEditor.Visible = true;
            m_FormUniversalEditor.Dock = DockStyle.Fill;
            m_Filename = filename;
            Text = filename;


            formGrid = new FormGrid(this);

        }


        public FormStandard(string filename)
        {
            Init(filename);
        }


        public FormStandard(string filename, EditorType editortype)
        {
            Init(filename);
            m_EditorType = editortype;
        }


        [DllImport("user32.dll")]
        public static extern bool LockWindowUpdate(IntPtr hWndLock); 



        private void FormStandardEditor_Load(object sender, EventArgs e)
        {
            m_DistributionBar = new DistributionBar(this,panelDistributionBar);
            
            Configuration c = Configuration.GetInstance();
            panelDistributionBar.Visible = c.DisplayDistribution;
            
            
            LockWindowUpdate(this.Handle);
            try
            {
                Query = "";
                Modified = false;
                LoadSettings();
                LoadFile();
                SetState(DataSetState.dsBrowsing);
            }
            finally 
            {
                LockWindowUpdate(IntPtr.Zero);
            }
            
            
            
        }


        private void SetState(DataSetState state)
        {
            m_FormUniversalEditor.SetState(state);

            m_State = state;
            if (m_State == DataSetState.dsBrowsing)
            {
                buttonSearchAssociative.Enabled = true;
                buttonSearchExact.Enabled = true;
                buttonSelectAll.Enabled = true;
                buttonFirst.Enabled = true;
                buttonPrev.Enabled = true;
                buttonNext.Enabled = true;
                buttonLast.Enabled = true;
                buttonEdit.Enabled = true;
                buttonDelete.Enabled = true;
                buttonStore.Enabled = false;
                buttonCancel.Enabled = false;
                buttonNotiz.Enabled = true;
                buttonURL.Enabled = true;
                buttonCluster.Enabled = true;
                checkNavigationButtons();
                checkEditorType();
            } 
            else
            if ((m_State == DataSetState.dsEditing) || (m_State == DataSetState.dsInsert))
            {
                buttonSearchAssociative.Enabled = false;
                buttonSearchExact.Enabled = false;
                buttonSelectAll.Enabled = false;
                buttonFirst.Enabled = false;
                buttonPrev.Enabled = false;
                buttonNext.Enabled = false;
                buttonLast.Enabled = false;
                buttonEdit.Enabled = false;
                buttonDelete.Enabled = false;
                buttonStore.Enabled = true;
                buttonCancel.Enabled = true;
                buttonNotiz.Enabled = false;
                buttonURL.Enabled = false;
                buttonCluster.Enabled = false;
            }
        }

        ///---------------------------------------------------------------------
        /// <summary>
        /// This function must be called in browsing state only.
        /// </summary>
        ///---------------------------------------------------------------------
        private void checkNavigationButtons()
        {
            if (m_Current <= 0)
            {
                buttonFirst.Enabled = false;
                buttonPrev.Enabled = false;
            }
            else
            {
                buttonFirst.Enabled = true;
                buttonPrev.Enabled = true;
            }

            if (m_Current >= m_ValuedDocList.Count - 1)
            {
                buttonNext.Enabled = false;
                buttonLast.Enabled = false;
            }
            else
            {
                buttonNext.Enabled = true;
                buttonLast.Enabled = true;
            }
        }

        private void checkEditorType()
        {
            switch (m_EditorType)
            {
                case EditorType.All:
                    buttonNotiz.Enabled = true;
                    buttonURL.Enabled = true;
                    buttonCluster.Enabled = true;
                    break;
                case EditorType.Notizen:
                    buttonNotiz.Enabled = true;
                    buttonURL.Enabled = false;
                    buttonCluster.Enabled = false;
                    break;
                case EditorType.URLs:
                    buttonNotiz.Enabled = false;
                    buttonURL.Enabled = true;
                    buttonCluster.Enabled = false;
                    break;
                case EditorType.Cluster:
                    buttonNotiz.Enabled = false;
                    buttonURL.Enabled = false;
                    buttonCluster.Enabled = true;
                    break;
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Load the record that corresponds to the current index
        /// </summary>
        ///---------------------------------------------------------------------
        private void ApplyCurrent()
        {
            if ((m_Current >= 0) && (m_Current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[m_Current];
                LineRecord lrec = (LineRecord)vdoc.Object;

                labelValue.Text  = vdoc.Value.ToString("0.00");
                textboxPath.Text = vdoc.theString;

                m_FormUniversalEditor.Record = lrec;
                m_FormUniversalEditor.RecordToScreen();
                checkBoxExport.Checked = vdoc.Mark;
                labelNr.Text = (m_Current + 1).ToString() + " von " + m_ValuedDocList.Count.ToString();
            }
            else
            {
                m_FormUniversalEditor.Clear();
                checkBoxExport.Checked = false;
                labelNr.Text = "?";
            }

            checkNavigationButtons();
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Make sure that all records of lrf are in m_ValuedDocList. Value = index
        /// </summary>
        ///---------------------------------------------------------------------
        private void SelectAll()
        {
            m_ValuedDocList.Clear();

            for (int i = 0; i < lrf.Count; i++)
                m_ValuedDocList.Add(lrf.Records[i], i);

            m_Current = 0;
            ApplyCurrent();
        }

        private void SelectRandomOrder()
        {
            m_ValuedDocList.Clear();

            for (int i = 0; i < lrf.Count; i++)
                m_ValuedDocList.Add(lrf.Records[i], m_Rnd.NextDouble());

            m_Current = 0;
            ApplyCurrent();
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Let only marked records be in the current selection
        /// </summary>
        ///---------------------------------------------------------------------
        private void RestrictToMarked()
        {
            List<LineRecord> list = new List<LineRecord>();
            LineRecord lrec;

            // Copy alle marked elements to list
            for (int i = 0; i < m_ValuedDocList.Count; i++)
            {
                ValuedDocument vdoc = m_ValuedDocList[i];
                if (vdoc.Mark)
                {
                    lrec = (LineRecord)vdoc.Object;
                    list.Add(lrec);
                }
            }

            m_ValuedDocList.Clear();
            for (int i = 0; i < list.Count; i++)
            {
                ValuedDocument vdoc;
                m_ValuedDocList.Add(list[i], i);
                vdoc = m_ValuedDocList[i];
                vdoc.Mark = true;
            }

            m_Current = 0;
            ApplyCurrent();
        }




        private void buttonFirst_Click(object sender, EventArgs e)
        {
            m_Current = 0;
            ApplyCurrent();
        }

        private void buttonPrev_Click(object sender, EventArgs e)
        {
            m_Current--;
            if (m_Current < 0)
                m_Current = 0;
            ApplyCurrent();
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            m_Current++;
            if (m_Current >= m_ValuedDocList.Count)
                m_Current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }

        private void buttonLast_Click(object sender, EventArgs e)
        {
            m_Current = m_ValuedDocList.Count - 1;
            ApplyCurrent();
        }



        private void LoadFile()
        {
            if ((m_Filename == null) || (m_Filename.Length == 0))
                return;

            Cursor.Current = Cursors.WaitCursor;
            try
            {
                lrf.LoadFromFile(m_Filename, true);
                m_FileHistoryReceiver.AddFileToHistory(m_Filename);
            }
            catch (IOException)
            {
                MessageBox.Show("Datei " + m_Filename + " kann nicht geffnet werden.");
                m_Filename = "";
                Text = "";
            }

            SelectAll();

            if (this.GotoLastRecord)
               buttonLast_Click(null,null);

            SetState(DataSetState.dsBrowsing);

            Cursor.Current = Cursors.Default;
        }


        public void Edit()
        {
            if ((m_Current >= 0) && (m_Current < m_ValuedDocList.Count))
            {
                SetState(DataSetState.dsEditing);
                m_FormUniversalEditor.SetActiveControl();
            }
        }

        private void buttonEdit_Click(object sender, EventArgs e)
        {
            Edit();
        }


        private void buttonDelete_Click(object sender, EventArgs e)
        {
            Configuration c = Configuration.GetInstance();

            if (c.AskBeforeDelete)
            {
                DialogResult dialogResult = MessageBox.Show("Datensatz lschen", "Frage",
                                MessageBoxButtons.YesNo,
                                MessageBoxIcon.Question,
                                MessageBoxDefaultButton.Button1
                               );
                    if (dialogResult == System.Windows.Forms.DialogResult.No)
                    {
                        return;
                    }
            }

            if ((m_Current >= 0) && (m_Current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[m_Current];
                LineRecord lrec = (LineRecord)vdoc.Object;

                lrec.Delete();
                m_FormUniversalEditor.Clear();

                Modified = true;
            }
        }

        private void buttonInsert_Click(object sender, EventArgs e)
        {
            SetState(DataSetState.dsInsert);
            m_FormUniversalEditor.Clear();
            m_FormUniversalEditor.SetActiveControl();
        }



        private void buttonStore_Click(object sender, EventArgs e)
        {
            m_FormUniversalEditor.ScreenToRecord();
            if (m_State == DataSetState.dsEditing)
            {
                ValuedDocument vdoc = m_ValuedDocList[m_Current];
                if (vdoc != null)
                {
                    ((LineRecord)(vdoc.Object)).Delete();
                    vdoc.Object = m_FormUniversalEditor.Record;
                }

                lrf.Add(m_FormUniversalEditor.Record);
            }
            else
                if (m_State == DataSetState.dsInsert)
                {
                    lrf.Add(m_FormUniversalEditor.Record);
                    SelectAll();
                }

            Modified = true;
            SetState(DataSetState.dsBrowsing);
        }


        private void buttonCancel_Click(object sender, EventArgs e)
        {
            SetState(DataSetState.dsBrowsing);
            //--- Restore old content ---
            ApplyCurrent();
        }


        private void Search(string searched)
        {
            searched = searched.ToUpper().Trim();

            m_ValuedDocList.Clear();
            for (int i = 0; i < lrf.Count; i++)
            {
                LineRecord lrec = lrf.Records[i];

                string s = lrec.GetFirstSymbol().ToUpper();
                while (s != "")
                {
                    if (s == searched)
                    {
                        m_ValuedDocList.Add(lrec, searched, i);
                        s = ""; // End inner loop. This record is qualified.
                    }
                    else
                        s = lrec.GetNextSymbol().ToUpper();
                }
            }

            m_Current = 0;
            ApplyCurrent();
        }




        private void buttonSearchExact_Click(object sender, EventArgs e)
        {
            using (FormSearchExact f = new FormSearchExact())
            {
                f.ShowDialog();


                if (f.Result == true)
                {
                    Cursor.Current = Cursors.WaitCursor;
                    Query = f.SearchString;
                    Search(f.SearchString);
                    Cursor.Current = Cursors.Default;
                }
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonSelectAll_Click(object sender, EventArgs e)
        {
            SelectAll();
            Query = "";
        }

        private void buttonSearchAssociative_Click(object sender, EventArgs e)
        {
            using (FormSearchAssociative f = new FormSearchAssociative(m_Filename, lrf, lrf, m_ValuedDocList))
            {
                f.ShowDialog();
                Query = f.Query;
                m_Current = 0;
                ApplyCurrent();
                panelDistributionBar.Invalidate();
            }
        }

        private void FormStandard_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!TryToSave(true))
                e.Cancel = true;
        }


        private void FormStandardEditor_FormClosed(object sender, FormClosedEventArgs e)
        {
            WindowState = FormWindowState.Normal; 
            StoreSettings();

            formGrid.Hide();
            formGrid.Close();
            formGrid = null;
        }


        private void LoadSettings()
        {
            Configuration c = Configuration.GetInstance();
            this.Location       = c.FormStandardSettings.Location;
            this.Size           = c.FormStandardSettings.Size;
            this.GotoLastRecord = c.GotoLastRecord;
        }


        private void StoreSettings()
        {
            Configuration c = Configuration.GetInstance();
            c.FormStandardSettings.Location = this.Location;
            c.FormStandardSettings.Size = this.Size;
        }

        private void buttonNotiz_Click(object sender, EventArgs e)
        {
            m_FormUniversalEditor.Record = new LineRecordNotiz("", "", "", new List<string>(), new List<string>());
            m_FormUniversalEditor.Clear();
            m_FormUniversalEditor.SetActiveControl();
            SetState(DataSetState.dsInsert);
        }

        private void buttonURL_Click(object sender, EventArgs e)
        {
            m_FormUniversalEditor.Record = new LineRecordUrl("", "", "", "", "", new List<string>());
            m_FormUniversalEditor.Clear();
            m_FormUniversalEditor.SetActiveControl();
            SetState(DataSetState.dsInsert);
        }

        private void buttonCluster_Click(object sender, EventArgs e)
        {
            m_FormUniversalEditor.Record = new LineRecordCluster(new List<string>());
            m_FormUniversalEditor.Clear();
            m_FormUniversalEditor.SetActiveControl();
            SetState(DataSetState.dsInsert);
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Check if the user begun editing and has not cancelled or stored yet.
        /// </summary>
        /// <returns>false, if the user must press a button, true otherwise.</returns>
        ///---------------------------------------------------------------------
        private bool EditStateIsOk()
        {
            if ((m_State == DataSetState.dsEditing) || (m_State == DataSetState.dsInsert))
            {
                MessageBox.Show("Datensatz wurde editiert." + System.Environment.NewLine +
                                "Bitte drcken Sie entweder 'Speichern' oder 'Abbrechen'");
                return false;
            }

            return true;
        }


        private void buttonStoreFile_Click(object sender, EventArgs e)
        {
            TryToSave(false);
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Tries to save the LineRecordFile to disk.
        /// </summary>
        /// <returns>true, if it was successful, false otherwise</returns>
        ///---------------------------------------------------------------------
        public bool TryToSave(bool doAsk)
        {
            DialogResult dialogResult;

            if (!EditStateIsOk())
                return false;

            if (!Modified)
                return true;


            if (doAsk)
                dialogResult = MessageBox.Show("Aktuelle nderungen speichern", "Frage",
                                             MessageBoxButtons.YesNoCancel,
                                             MessageBoxIcon.Question,
                                             MessageBoxDefaultButton.Button1
                                            );
            else
                dialogResult = DialogResult.Yes;


            if (dialogResult == DialogResult.Yes)
            {
            }
            else
            if (dialogResult == DialogResult.No)
               {
                // User want to leave without storing.
                Modified = false;
                return true;
               }
            else
               {
                return false;
               }


            // Ask for filename, if modified and filename is not known.
            if ((m_Filename == null) || (m_Filename == ""))              
            {
                using (SaveFileDialog saveFileDialog = new SaveFileDialog())
                {
                    saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                    saveFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";
                    if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                        m_Filename = saveFileDialog.FileName;
                    else
                        return false;
                }
            }

            if ((m_Filename != null) && (m_Filename.Length > 0))
               {
                lrf.SaveToFile(m_Filename);
                m_FileHistoryReceiver.AddFileToHistory(m_Filename);
               }

            Modified = false;
            return true;
        }

// lschen zu-tun
        private void buttonExportUnarch_Click(object sender, EventArgs e)
        {

            string filename;

            using (SaveFileDialog saveFileDialog = new SaveFileDialog())
            {
                saveFileDialog.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.Personal);
                saveFileDialog.Filter = "Unarch-Dateien (*.una)|*.una|Alle Dateien (*.*)|*.*";

                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                    filename = saveFileDialog.FileName;
                else
                    return;
            }



            // Construct a big string consisting of all records of the current selection.
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < m_ValuedDocList.Count; i++)
            {
                ValuedDocument vdoc = m_ValuedDocList[i];
                LineRecord lrec = (LineRecord)vdoc.Object;
                sb.AppendLine(lrec.Text);
                sb.AppendLine();
            }





           try
           {
               FileStream IniStream = new FileStream(filename, FileMode.Create);
               if (!IniStream.CanWrite)
               {
                   IniStream.Close();
                   return;
               }
               StreamWriter writer = new StreamWriter(IniStream,Configuration.encoding);
               writer.Write(sb.ToString());
               writer.Flush();
               writer.Close();
               IniStream.Close();
               return;
           }
           catch
           {
               return;
           }
        }






        private void LoadClusterFiles(SemanticNetwork cm)
        {
            Configuration c = Configuration.GetInstance();
            List<string> activeFiles = c.ActiveClusterFiles;

            foreach (string filename in activeFiles)
            {
                //statusStrip.Text = filename;
                labelFilenames.Text = filename;
                Application.DoEvents();
                try
                {
                    cm.LoadFromFile(filename);
                }
                catch
                {
                    // Da sollte man was tun.
                }
            }

            labelFilenames.Text = "";
            Application.DoEvents();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="lrec"></param>
        /// <returns></returns>
        private string GetFirstStichwort(LineRecord lrec)
        {
            if (lrec is IKeywordProvider)
                return (lrec as IKeywordProvider).GetFirstKeyword();
            else
               return "";
        }

        private string GetNextStichwort(LineRecord lrec)
        {
            if (lrec is IKeywordProvider)
                return (lrec as IKeywordProvider).GetNextKeyword();
            else
                return "";
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Goes through all records and all entries of the STICHWORTE field
        /// and determines the unknown ones, ie. those which are not in the associative net.
        /// </summary>
        ///---------------------------------------------------------------------
        private void DetermineUnknownKeywords(ArrayList listOfWords)
        {
            SemanticNetwork cm = new SemanticNetwork(Configuration.encoding);
            Hashtable alreadySeen = new Hashtable();

            // Load from the calling document
            labelFilenames.Text = m_Filename;
            Application.DoEvents();
            cm.LoadFromClusterProvider(lrf);

            LoadClusterFiles(cm);


            progressBarUnbekannteStichworte.Maximum = lrf.Count;

            for (int i = 0; i < lrf.Count; i++)
            {
                progressBarUnbekannteStichworte.Value = i;

                string theSymbol;

                LineRecord lrec = lrf.Records[i];

                theSymbol = GetFirstStichwort(lrec);
                while (theSymbol.Length > 0)
                {
                    theSymbol = theSymbol.ToUpper();
                    if ((!alreadySeen.ContainsKey(theSymbol)) && (!cm.HasAssociations(theSymbol)))
                       {
                           listOfWords.Add(theSymbol);
                        alreadySeen.Add(theSymbol,null);
                       }
                    theSymbol = GetNextStichwort(lrec);
                }
            }
            progressBarUnbekannteStichworte.Value = 0;
        }

    







        private void buttonUnbekannteStichwoerter_Click(object sender, EventArgs e)
        {
            ArrayList listOfWords = new ArrayList();

            Cursor.Current = Cursors.WaitCursor;
            try
            {
                DetermineUnknownKeywords(listOfWords);
            }
            finally
            {
                Cursor.Current = Cursors.Default; 
            }

            using (FormUnknownKeywords f = new FormUnknownKeywords(listOfWords))
            { 
                f.ShowDialog(); 
            }
        }

        private void buttonPfad_Click(object sender, EventArgs e)
        {
            if ((m_Current >= 0) && (m_Current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[m_Current];
                LineRecord lrec = (LineRecord)vdoc.Object;

                using (FormPathToHere f = new FormPathToHere())
                {
                    f.lrec = lrec;
                    f.Query = m_Query;
                    f.ShowDialog();
                }
            }



        }



        private void buttonExport_Click(object sender, EventArgs e)
        {
            using (FormExport f = new FormExport())
            {
                f.Query   = m_Query;
                f.DocList = m_ValuedDocList;
                f.ShowDialog();
            }
        }

        private void checkBoxExport_CheckedChanged(object sender, EventArgs e)
        {
            if ((m_Current >= 0) && (m_Current < m_ValuedDocList.Count))
            {
                ValuedDocument vdoc = m_ValuedDocList[m_Current];
                if (sender == checkBoxExport)
                    vdoc.Mark = (sender as CheckBox).Checked;
            }

        }



 


        private void buttonSelectMarked_Click(object sender, EventArgs e)
        {
            RestrictToMarked();
        }

        private void buttonGrid_Click(object sender, EventArgs e)
        {
            formGrid.SetValuedDocList(m_ValuedDocList);
            formGrid.Show();
        }

        public double GetFirstDistributionValue()
        {
            m_DistributionProviderIndex = 0;
            return GetNextDistributionValue();
        }
        
        public double GetNextDistributionValue()
        {
            if (m_DistributionProviderIndex < m_ValuedDocList.Count) 
               return m_ValuedDocList[m_DistributionProviderIndex++].Value;
            else 
               return -1.0;
        }

        private void panelDistributionBar_Paint(object sender, PaintEventArgs e)
        {
            if (panelDistributionBar.Visible) 
               m_DistributionBar.Paint(e);
        }


        private void buttonRandom_Click(object sender, EventArgs e)
        {
            SelectRandomOrder();
        }

    }

}