// 26.Jun.2010 Joachim Pimiskern
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Fusym;
using UnarchControls;
using System.Text.RegularExpressions;

namespace Unarch5
{
    public partial class FormSymbolBits : Form
    {
        private SymbolAsBitsManager m_SymbolAsBitsManager;
        private bool m_Stop;
        private CheckBox[] m_CheckBoxes;


        public FormSymbolBits()
        {
            InitializeComponent();
            m_SymbolAsBitsManager = new SymbolAsBitsManager(GlobalClusterManager.GetInstance().SemanticNetwork, 32);
            CreateCheckboxes();
        }

        private void CreateCheckboxes()
        {
            string s = "";
            int count = m_SymbolAsBitsManager.BitsPerSymbol;
            m_CheckBoxes = new CheckBox[count];

            for (int i = 0; i < count; i++)
            {
                s += i.ToString() + " ";

                CheckBox cb = new CheckBox();
                cb.Parent = this.panelScroll;
                cb.Tag = i;
                cb.Size = cbDemo.Size;
                cb.Location = new Point(i * cbDemo.Width + 2, 10);
                cb.Text = i.ToString();
                cb.Click += new System.EventHandler(this.cbDemo_Click);
                cb.Visible = true;
                m_CheckBoxes[i] = cb;
            }
        }

        private void buttonTraining_Click(object sender, EventArgs e)
        {

            m_Stop = false;
            int i = 0;
            while (m_Stop == false)
            {
                string s = m_SymbolAsBitsManager.TrainingStep();

                if (i++ % 100 == 0)
                {
                    Application.DoEvents();
                    textBoxOutput.Text = s;
                    label1.Text = m_SymbolAsBitsManager.ErrorAvg().ToString("N5");
                }

                // Remove same codes
                // if (i % 1000 == 0)
                  // m_SymbolAsBitsManager.AvoidCollisions();
            }

            // Finally make sure there are no collisions.
           // m_SymbolAsBitsManager.AvoidCollisions();
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void buttonStop_Click(object sender, EventArgs e)
        {
            m_Stop = true;
        }

        private void buttonStore_Click(object sender, EventArgs e)
        {
            m_SymbolAsBitsManager.Save();
        }

        private List<string> GetQuerySymbols(string s)
        {
            List<string> result = new List<string>();

            s = s.ToUpper();

            Regex r = new Regex(@"[^\s]+");
            MatchCollection m = r.Matches(s);
            foreach (Match ma in m)
            {
                string sym = ma.ToString();
                result.Add(sym);
            }

            return result;
        }



        private void buttonSearch_Click(object sender, EventArgs e)
        {
            List<String> query = GetQuerySymbols(textBoxInput.Text);

            List<string> list = m_SymbolAsBitsManager.SearchByString(query, true);
            DisplayOutput(list);
        }

        private void cbDemo_Click(object sender, EventArgs e)
        {
            string s = "";
            for (int i = 0; i < m_CheckBoxes.Length; i++)
            {
                s += m_CheckBoxes[i].Checked ? "1" : "0";
            }

            textBoxInput.Text = s;
            List<string> list = m_SymbolAsBitsManager.SearchByBits(s, true);
            DisplayOutput(list);
        }

        private void DisplayOutput(List<string> list)
        {
            textBoxOutput.SuspendLayout();
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < list.Count; i++)
                sb.AppendLine(list[i]);
            textBoxOutput.Text = sb.ToString();
            textBoxOutput.ResumeLayout();
        }

        private void buttonDistribution_Click(object sender, EventArgs e)
        {
            List<string> list = m_SymbolAsBitsManager.Distribution(textBoxInput.Text);
            DisplayOutput(list);

        }
    }
}