//------------------------------------------------------------------------------
// Contains multiple editor pages, one for each LineRecord type.
//
// Joachim Pimiskern, 07.May.2007, Freeware.
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections;
using System.Diagnostics;
using System.Runtime.InteropServices;
using UnarchControls;

namespace Unarch5
{
    public partial class FormUniversalEditor : Form
    {
        private LineRecord m_Record;
        public FormStandard ParentForm = null;

        public LineRecord Record
        {
            get 
            {
             return m_Record; 
            }
            set 
            {
             m_Record = value;
             ApplyType();
            }
        }
	

        public FormUniversalEditor()
        {
            InitializeComponent();
        }

        [DllImport("user32.dll")]
        public static extern bool LockWindowUpdate(IntPtr hWndLock); 






        private void ActivatePanel(Panel p)
        {
            foreach (Control c in this.Controls)
            {
                if (c is Panel)
                   {
                    c.TabStop = false;
                    c.Enabled = false;
                   }
            }


            p.BringToFront();
            p.TabStop = true;
            p.Enabled = true;
            foreach (Control c in p.Controls)
            {
                if (c.Tag is string)
                    try
                    {
                        int i = int.Parse((string)c.Tag);
                        c.TabIndex = i;
                    }
                    catch (Exception)
                    {
                    }
            }
        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Dependent on the type of the document, choose a tabpage as editor
        /// </summary>
        ///---------------------------------------------------------------------
        private void ApplyType()
        {
            if (m_Record == null)
               return;

            LockWindowUpdate(this.Handle);
            try
            {
                Type documentType = m_Record.GetType();

                if (documentType == typeof(LineRecord))
                    ActivatePanel(panelStandard);
                else
                if (documentType == typeof(LineRecordCluster))
                   ActivatePanel(panelCluster);
                else
                if (documentType == typeof(LineRecordNotiz))
                   ActivatePanel(panelNotizen);
                else
                if (documentType == typeof(LineRecordUrl))
                   ActivatePanel(panelURLs);
                else
                   throw new Exception("FormUniversalEditor.ApplyType: unexpected document type");
            }
            finally
            {
                LockWindowUpdate(IntPtr.Zero);
            }
        }


        ///---------------------------------------------------------------------
        /// <summary>
        /// Dependent on the record type, set the focus.
        /// </summary>
        ///---------------------------------------------------------------------
        public void SetActiveControl()
        {
            if (m_Record == null)
                return; // Man sollte drueber nachdenken, ob dieser Fall sinnvoll ist. zu-tun


            Type documentType = m_Record.GetType();

            if (documentType == typeof(LineRecord))
                ActiveControl = textBoxText;
            else
            if (documentType == typeof(LineRecordCluster))
                ActiveControl = textBoxCluster;
            else
            if (documentType == typeof(LineRecordNotiz))
                ActiveControl = textBoxInfo;
            else
            if (documentType == typeof(LineRecordUrl))
                ActiveControl = textBoxUrl;
            else
                throw new Exception("FormUniversalEditor.SetActiveControl: unexpected document type");
        }




        ///---------------------------------------------------------------------
        /// <summary>
        /// Show or hide some controls, dependent from the editing state of a dataset
        /// </summary>
        /// <param name="state">The dataset state</param>
        ///---------------------------------------------------------------------
        public void SetState(DataSetState state)
        {

            LockWindowUpdate(this.Handle);

            if (state == DataSetState.dsBrowsing)
            {
                //--------------------------------------------------------------
                textBoxText.ReadOnly             = true;
                textBoxText.BackColor            = Color.White;
                //--------------------------------------------------------------
                textBoxCluster.ReadOnly          = true;
                textBoxCluster.BackColor         = Color.White;
                //--------------------------------------------------------------
                textBoxTagNotiz.ReadOnly         = true;
                textBoxTagNotiz.BackColor        = Color.White;
                textBoxMonatNotiz.ReadOnly       = true;
                textBoxMonatNotiz.BackColor      = Color.White;
                textBoxJahrNotiz.ReadOnly        = true;
                textBoxJahrNotiz.BackColor       = Color.White;
                textBoxInfo.ReadOnly             = true;
                textBoxInfo.BackColor            = Color.White;
                textBoxStichworteNotiz.ReadOnly  = true;
                textBoxStichworteNotiz.BackColor = Color.White;
                //--------------------------------------------------------------
                textBoxTagUrl.ReadOnly           = true;
                textBoxTagUrl.BackColor          = Color.White;
                textBoxMonatUrl.ReadOnly         = true;
                textBoxMonatUrl.BackColor        = Color.White;
                textBoxJahrUrl.ReadOnly          = true;
                textBoxJahrUrl.BackColor         = Color.White;
                textBoxUrl.ReadOnly              = true;
                textBoxUrl.BackColor             = Color.White;
                textBoxTitle.ReadOnly            = true;
                textBoxTitle.BackColor           = Color.White;
                textBoxStichworteUrl.ReadOnly    = true;
                textBoxStichworteUrl.BackColor   = Color.White;
                //--------------------------------------------------------------
            }
            else
            if (
                (state == DataSetState.dsEditing) || 
                (state == DataSetState.dsInsert)
               )
             {
                 //--------------------------------------------------------------
                 textBoxText.ReadOnly            = false;
                 //--------------------------------------------------------------
                 textBoxCluster.ReadOnly         = false;
                 //--------------------------------------------------------------
                 textBoxTagNotiz.ReadOnly        = false;
                 textBoxMonatNotiz.ReadOnly      = false;
                 textBoxJahrNotiz.ReadOnly       = false;
                 textBoxInfo.ReadOnly            = false;
                 textBoxStichworteNotiz.ReadOnly = false;
                 //--------------------------------------------------------------
                 textBoxTagUrl.ReadOnly          = false;
                 textBoxMonatUrl.ReadOnly        = false;
                 textBoxJahrUrl.ReadOnly         = false;
                 textBoxUrl.ReadOnly             = false;
                 textBoxTitle.ReadOnly           = false;
                 textBoxStichworteUrl.ReadOnly   = false;
                 //--------------------------------------------------------------
             }

         LockWindowUpdate(IntPtr.Zero);

        }



        ///---------------------------------------------------------------------
        /// <summary>
        /// Put the content of the record to the input fields on the form.
        /// </summary>
        ///---------------------------------------------------------------------
        public void RecordToScreen()
        {
            // Do nothing if record is deleted
            if (m_Record.Deleted)
               {
                Clear();
                return;
               }

            Type documentType = m_Record.GetType();

            if (documentType == typeof(LineRecord))
               {
                   LineRecord lrec  = m_Record;
                   textBoxText.Text = lrec.Text;
               }
            else
            if (documentType == typeof(LineRecordCluster))
               {
                   LineRecordCluster lrec = (LineRecordCluster)m_Record;
                   textBoxCluster.Text = lrec.Text;

                   textBoxCluster.MarkAllUnknownWords();
               }
            else
            if (documentType == typeof(LineRecordNotiz))
               {
                   LineRecordNotiz lrec = (LineRecordNotiz)m_Record;
                   textBoxTagNotiz.Text        = lrec.Tag;
                   textBoxMonatNotiz.Text      = lrec.Monat;
                   textBoxJahrNotiz.Text       = lrec.Jahr;
                   textBoxInfo.Text            = lrec.Info;
                   textBoxStichworteNotiz.Text = lrec.Stichworte;

                   textBoxStichworteNotiz.MarkAllUnknownWords();
               }
            else
            if (documentType == typeof(LineRecordUrl))
               {
                   LineRecordUrl lrec        = (LineRecordUrl)m_Record;
                   textBoxTagUrl.Text        = lrec.Tag;
                   textBoxMonatUrl.Text      = lrec.Monat;
                   textBoxJahrUrl.Text       = lrec.Jahr;
                   textBoxUrl.Text           = lrec.URL;
                   textBoxTitle.Text         = lrec.Titel;
                   textBoxStichworteUrl.Text = lrec.Stichworte;

                   textBoxTitle         . MarkAllUnknownWords();
                   textBoxStichworteUrl . MarkAllUnknownWords();
               }
            else
               throw new Exception("FormUniversalEditor.RecordToScreen: unexpected document type");
        }










        ///---------------------------------------------------------------------
        /// <summary>
        /// Takes the current Record for type information.
        /// Replaces the current Record by a new one of the same type.
        /// </summary>
        ///---------------------------------------------------------------------
        public void ScreenToRecord()
        {
            if (m_Record == null)
                return;

            Type documentType = m_Record.GetType();

            
            if (documentType == typeof(LineRecord))
               {
                    m_Record = new LineRecord();
                    m_Record.Text = textBoxText.Text;
               }
            else
            if (documentType == typeof(LineRecordCluster))
               {
                    m_Record = new LineRecordCluster(textBoxCluster.Text);
               }
            else
            if (documentType == typeof(LineRecordNotiz))
               {
                   List<string> info       = new List<string>();
                   List<string> stichworte = new List<string>();

                   LineRecord.StringToStringList(textBoxInfo.Text, info);
                   LineRecord.StringToStringList(textBoxStichworteNotiz.Text, stichworte);
                   m_Record = new LineRecordNotiz(textBoxTagNotiz.Text,
                                                  textBoxMonatNotiz.Text,
                                                  textBoxJahrNotiz.Text,
                                                  info,
                                                  stichworte
                                                 );

               }
            else                    
            if (documentType == typeof(LineRecordUrl))
               {
                   List<string> stichworte = new List<string>();

                   LineRecord.StringToStringList(textBoxStichworteUrl.Text, stichworte);
                   m_Record = new LineRecordUrl(textBoxTagUrl.Text,
                                                textBoxMonatUrl.Text,
                                                textBoxJahrUrl.Text,
                                                textBoxUrl.Text,
                                                textBoxTitle.Text,
                                                stichworte
                                               );

              }
            else
              throw new Exception("FormUniversalEditor.ScreenToRecord: unexpected document type");
        }


















        ///---------------------------------------------------------------------
        /// <summary>
        /// Clear the input fields
        /// </summary>
        ///---------------------------------------------------------------------
        public void Clear()
        {
            //--------------------------------------------------------------
            textBoxText.Clear();
            //--------------------------------------------------------------
            textBoxCluster.Clear();
            //--------------------------------------------------------------
            textBoxTagNotiz.Text   = DateTime.Today.Day.ToString();
            textBoxMonatNotiz.Text = DateTime.Today.Month.ToString();
            textBoxJahrNotiz.Text  = DateTime.Today.Year.ToString();
            textBoxInfo.Clear();
            textBoxStichworteNotiz.Clear();
            //--------------------------------------------------------------
            textBoxTagUrl.Text      = DateTime.Today.Day.ToString();
            textBoxMonatUrl.Text    = DateTime.Today.Month.ToString();
            textBoxJahrUrl.Text     = DateTime.Today.Year.ToString();
            textBoxUrl.Clear();
            textBoxTitle.Clear();
            textBoxStichworteUrl.Clear();
            //--------------------------------------------------------------
        }

        private void textBoxCluster_KeyPress(object sender, KeyPressEventArgs e)
        {
            e.KeyChar = Char.ToUpper(e.KeyChar);

            if ((e.KeyChar == ' ') || (e.KeyChar == '\t'))
            {
                e.Handled = true;
            }

        }


        private void OpenUrl(string url)
        {

            Configuration c = Configuration.GetInstance();
            if ((c.UseSpecialBrowser) && (url.StartsWith("http:")))
            {
                Process.Start(c.HttpBrowserExe, url); 
            }
            else
                Help.ShowHelp(this, url);
        }



        private void buttonOpen_Click(object sender, EventArgs e)
        {
            try
            {
                OpenUrl(textBoxUrl.Text);
            }
            catch
            {
                MessageBox.Show("Start nicht mglich.");
            }
        }




        private void Command(AssociationRichTextBox a, string command)
        {
            if (command == "selectall")
                a.SelectAll();
            else
            if (command == "copy")
               a.Copy();
            else
            if (command == "undo")
               a.Undo();
            else
            if (command == "delete")
               a.SelectedText = "";
            else
            if (command == "cut")
               a.Cut();
            else
            if (command == "paste")
               a.Paste();
        }

        private void toolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (!(sender is ToolStripMenuItem))
                return;

            object o = (sender as ToolStripMenuItem).Tag;

            if (!(o is string))
                return;

            string tag = (string)o;

            Command(textBoxCluster, tag);
        }

        private void toolStripMenuItem5_Click(object sender, EventArgs e)
        {
            if (!(sender is ToolStripMenuItem))
                return;

            object o = (sender as ToolStripMenuItem).Tag;

            if (!(o is string))
                return;

            string tag = (string)o;

            Command(textBoxStichworteNotiz, tag);
        }

        private void toolStripMenuItem11_Click(object sender, EventArgs e)
        {
            if (!(sender is ToolStripMenuItem))
                return;

            object o = (sender as ToolStripMenuItem).Tag;

            if (!(o is string))
                return;

            string tag = (string)o;

            Command(textBoxStichworteUrl, tag);

        }

        private void toolStripMenuItem17_Click(object sender, EventArgs e)
        {
            if (!(sender is ToolStripMenuItem))
                return;

            object o = (sender as ToolStripMenuItem).Tag;

            if (!(o is string))
                return;

            string tag = (string)o;

            Command(textBoxTitle, tag);

        }
    }
}