// 7.Juli 2006

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Text.RegularExpressions;

namespace Unarch5
{
    public interface IKeywordProvider
    {
        string GetFirstKeyword();
        string GetNextKeyword();
    }


    //--------------------------------------------------------------------------
    public class LineRecordNotiz : LineRecord, IKeywordProvider, IRecordWithDate
    {
        StringBuilder sb = new StringBuilder();
        
        private string m_Tag;
        private string m_Monat;
        private string m_Jahr;
        private List<string> m_Info = new List<string>();
        private List<string> m_Stichworte = new List<string>();

        private string[] listOfAllWords;
        private int current;

        private string[] listOfAllWords_Stichworte;
        private int current_Stichworte;

        public LineRecordNotiz(
                               string tag,
                               string monat,
                               string jahr,
                               List<string> info,
                               List<string> stichworte
                              )
        {
            m_Tag   = tag;
            m_Monat = monat;
            m_Jahr  = jahr;
            m_Info.AddRange(info);
            m_Stichworte.AddRange(stichworte);
        }


        public string Tag
        {
            get { return m_Tag; }
            set { m_Tag = value; }
        }

        public string Monat
        {
            get { return m_Monat; }
            set { m_Monat = value; }
        }

        public string Jahr
        {
            get { return m_Jahr; }
            set { m_Jahr = value; }
        }

        public DateTime Date()
        {
            DateTime result;

            try
            {
                result = new DateTime(int.Parse(m_Jahr), int.Parse(m_Monat), int.Parse(m_Tag));
            }
            catch
            {
                result = DateTime.MinValue;
            }


            return result;
        }


        public string Info
        {
            get { return LineRecord.StringListToString(m_Info); }
            set { LineRecord.StringToStringList(value, m_Info); }
        }

        public string Stichworte
        {
            get { return LineRecord.StringListToString(m_Stichworte); }
            set { LineRecord.StringToStringList(value, m_Stichworte); }
        }

        public override string Text
        {
            get
            {
                return GetText();
            }
        }



        private string GetText()
        {
            sb.Length = 0;
            
            sb.Append("TAG ");
            sb.AppendLine(m_Tag);
            sb.Append("MONAT ");
            sb.AppendLine(m_Monat);
            sb.Append("JAHR ");
            sb.AppendLine(m_Jahr);

            for (int i = 0; i < m_Info.Count; i++)
            {
                sb.Append("INFO-");
                sb.Append(i);
                sb.Append(" ");
                sb.AppendLine(m_Info[i]);
            }

            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append("STICHWORTE-");
                sb.Append(i);
                sb.Append(" ");
                sb.AppendLine(m_Stichworte[i]);
            }

            return sb.ToString();
        }


        public override string GetFirstSymbol()
        {
            sb.Length = 0;
            
            sb.Append(m_Tag);
            sb.Append(" ");
            sb.Append(m_Monat);
            sb.Append(" ");
            sb.Append(m_Jahr);

            for (int i = 0; i < m_Info.Count; i++)
            {
                sb.Append(" ");
                sb.Append(m_Info[i]);
            }

            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append(" ");
                sb.Append(m_Stichworte[i]);
            }


            listOfAllWords = Regex.Split(sb.ToString(), m_WordlistSplitRegex);
            current = 0;
            return GetNextSymbol();
        }




        public override string GetNextSymbol()
        {
            while (current <= listOfAllWords.GetUpperBound(0))
            {
                string s = listOfAllWords[current++];
                if (s.Length == 0)
                    continue;
                return s;
            }
            return "";
        }


        public string GetFirstKeyword()
        {
            sb.Length = 0;
            
            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append(" ");
                sb.Append(m_Stichworte[i]);
            }

            listOfAllWords_Stichworte = Regex.Split(sb.ToString(), m_WordlistSplitRegex);

            current_Stichworte = 0;
            return GetNextKeyword();
        }


        public string GetNextKeyword()
        {
            int upperBound = listOfAllWords_Stichworte.GetUpperBound(0);

            while (
                   (current_Stichworte <= upperBound) &&
                   (listOfAllWords_Stichworte[current_Stichworte].Length <= 0)
                  )
                current_Stichworte++;
            if (current_Stichworte <= listOfAllWords_Stichworte.GetUpperBound(0))
                return listOfAllWords_Stichworte[current_Stichworte++];
            else
                return "";
        }

    }

    //--------------------------------------------------------------------------
    public class LineRecordUrl : LineRecord, IKeywordProvider, IRecordWithDate
    {
        StringBuilder sb = new StringBuilder();
    
        private string m_Tag;
        private string m_Monat;
        private string m_Jahr;
        private string m_Url;
        private string m_Titel;
        private List<string> m_Stichworte = new List<string>();

        private string[] listOfAllWords;
        private int current;

        private string[] listOfAllWords_Stichworte;
        private int current_Stichworte;

        public LineRecordUrl(
                             string tag,
                             string monat,
                             string jahr,
                             string url,
                             string titel,
                             List<string> stichworte
                            )
        {
            m_Tag   = tag;
            m_Monat = monat;
            m_Jahr  = jahr;
            m_Url   = url;
            m_Titel = titel;
            m_Stichworte.AddRange(stichworte);
        }


        public string Tag
        {
            get { return m_Tag; }
            set { m_Tag = value; }
        }

        public string Monat
        {
            get { return m_Monat; }
            set { m_Monat = value; }
        }

        public string Jahr
        {
            get { return m_Jahr; }
            set { m_Jahr = value; }
        }

        public DateTime Date()
        {
            DateTime result;

            try
            {
                result = new DateTime(int.Parse(m_Jahr), int.Parse(m_Monat), int.Parse(m_Tag));
            }
            catch
            {
                result = DateTime.MinValue;
            }


            return result;
        }


        public string URL
        {
            get { return m_Url; }
            set { m_Url = value; }
        }

        public string Titel
        {
            get { return m_Titel; }
            set { m_Titel = value; }
        }

        public string Stichworte
        {
            get { return LineRecord.StringListToString(m_Stichworte); }
            set { LineRecord.StringToStringList(value, m_Stichworte); }
        }

        public override string Text
        {
            get
            {
                return GetText();
            }
        }

        private string GetText()
        {
            sb.Length = 0;
            
            sb.Append("TAG ");
            sb.AppendLine(m_Tag);
            sb.Append("MONAT ");
            sb.AppendLine(m_Monat);
            sb.Append("JAHR ");
            sb.AppendLine(m_Jahr);
            sb.Append("URL ");
            sb.AppendLine(m_Url);
            sb.Append("TITEL ");
            sb.AppendLine(m_Titel);


            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append("STICHWORTE-");
                sb.Append(i);
                sb.Append(" ");

                if (i < m_Stichworte.Count - 1)
                   sb.AppendLine(m_Stichworte[i]);
                else
                   sb.Append(m_Stichworte[i]);
           }

            return sb.ToString();
        }




        public override string GetFirstSymbol()
        {
            sb.Length = 0;
            
            sb.Append(m_Tag);
            sb.Append(" ");
            sb.Append(m_Monat);
            sb.Append(" ");
            sb.Append(m_Jahr);
            sb.Append(" ");
            sb.Append(m_Url);
            sb.Append(" ");
            sb.Append(m_Titel);


            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append(" ");
                sb.Append(m_Stichworte[i]);
            }


            listOfAllWords = Regex.Split(sb.ToString(), m_WordlistSplitRegex);
            current = 0;
            return GetNextSymbol();
        }

        public override string GetNextSymbol()
        {
            int upperBound = listOfAllWords.GetUpperBound(0);
            while (current <= upperBound)
            {
                string s = listOfAllWords[current++];
                if (s.Length == 0)
                    continue;
                return s;
            }
            return "";
        }



        public string GetFirstKeyword()
        {
            sb.Length = 0;
            
            for (int i = 0; i < m_Stichworte.Count; i++)
            {
                sb.Append(" ");
                sb.Append(m_Stichworte[i]);
            }

            listOfAllWords_Stichworte = Regex.Split(sb.ToString(), m_WordlistSplitRegex);

            current_Stichworte = 0;
            return GetNextKeyword();
        }

        public string GetNextKeyword()
        {
            // Overread separators
            int upperBound = listOfAllWords_Stichworte.GetUpperBound(0);
            while (
                   (current_Stichworte <= upperBound) &&
                   (listOfAllWords_Stichworte[current_Stichworte].Length <= 0)
                  )
                  current_Stichworte++;

            if (current_Stichworte <= listOfAllWords_Stichworte.GetUpperBound(0))
                return listOfAllWords_Stichworte[current_Stichworte++];
            else
                return "";
        }


    }


    //--------------------------------------------------------------------------
    public class LineRecordCluster : LineRecord
    {

        List<string> m_Cluster = new List<string>();

        private string[] listOfAllWords;
        private int current;


        public LineRecordCluster(List<string> cluster)
        {
            m_Cluster.AddRange(cluster);
        }

        public LineRecordCluster(string text)
        {
            Text = text;
        }


        public new List<string> Lines
        {
            get
            {
                return m_Cluster;
            }
        }


        public override string Text
        {
            get
            {
                return StringListToString(m_Cluster);
            }

            set
            {
                StringToStringList(value, m_Cluster);
            }
        }




        public override string GetFirstSymbol()
        {
            listOfAllWords = m_Cluster.ToArray();
            current = 0;
            return GetNextSymbol();
        }

        public override string GetNextSymbol()
        {
            int upperBound = listOfAllWords.GetUpperBound(0);
            while (current <= upperBound)
            {
                string s = listOfAllWords[current++];
                if (s.Length == 0)
                    continue;
                return s;
            }
            return "";
        }


    }





    public interface IRecordWithDate
    {
        DateTime Date();
    }







    //--------------------------------------------------------------------------
    public class LineRecordParser
    {
        static Regex m_RegexTag        = new Regex(@"^TAG\s+(?<tag>\d*)\s*$", RegexOptions.Compiled);
        static Regex m_RegexMonat      = new Regex(@"^MONAT\s+(?<monat>\d*)\s*$", RegexOptions.Compiled);
        static Regex m_RegexJahr       = new Regex(@"^JAHR\s+(?<jahr>\d*)\s*$", RegexOptions.Compiled);
        static Regex m_RegexInfo       = new Regex(@"^INFO-\d+\s(?<info>.*)$", RegexOptions.Compiled);
        static Regex m_RegexStichworte = new Regex(@"^STICHWORTE-\d+\s(?<stichworte>.*)$", RegexOptions.Compiled);
        static Regex m_RegexUrl        = new Regex(@"^URL\s+(?<url>.*)$", RegexOptions.Compiled);
        static Regex m_RegexTitel      = new Regex(@"^TITEL\s+(?<titel>.*)$", RegexOptions.Compiled);
        static Regex m_RegexSymbol     = new Regex(@"^\s*(?<symbol>\S+)\s*$", RegexOptions.Compiled);

        //--- All that might be relevant for any type ---
        string    tag;
        string    monat;
        string    jahr;
        string    url;
        string    titel;
        List<string> info = new List<string>();
        List<string> stichworte = new List<string>();
        List<string> cluster = new List<string>();


        public LineRecord GetSpecializedType(LineRecord lrec)
        {
            int state = 0;
            List<string> lines = lrec.Lines;

            tag   = null;
            monat = null;
            jahr  = null;
            url   = null;
            titel = null;
            info.Clear();
            stichworte.Clear();
            cluster.Clear();



            for (int i = 0; i < lines.Count; i++)
            {
                string line = lines[i];
                Match m;

                //--------------------------------------------------------------
                if (state == 0)
                {
                    m = m_RegexTag.Match(line);
                    if (m.Success)
                    {
                        tag = m.Groups["tag"].Value;
                        state = 100;
                    }
                    else
                    {
                        m = m_RegexSymbol.Match(line);
                        if (m.Success)
                        {
                            cluster.Add(m.Groups["symbol"].Value);
                            state = 700;
                        }
                        else
                        {
                            //--- Anything else ---
                            state = 1000;
                        }
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 100)
                {
                    m = m_RegexMonat.Match(line);
                    if (m.Success)
                    {
                        monat = m.Groups["monat"].Value;
                        state = 200;
                    }
                    else
                    {
                        //--- Anything else ---
                        state = 1000;
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 200)
                {
                    m = m_RegexJahr.Match(line);
                    if (m.Success)
                    {
                        jahr = m.Groups["jahr"].Value;
                        state = 300;
                    }
                    else
                    {
                        //--- Anything else ---
                        state = 1000;
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 300)
                {
                    m = m_RegexInfo.Match(line);
                    if (m.Success)
                    {
                        info.Add(m.Groups["info"].Value);
                        state = 400;
                    }
                    else
                    {
                        m = m_RegexUrl.Match(line);
                        if (m.Success)
                        {
                            url = m.Groups["url"].Value;
                            state = 600;
                        }
                        else
                        {
                            //--- Anything else ---
                            state = 1000;
                        }
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 400)
                {
                    m = m_RegexInfo.Match(line);
                    if (m.Success)
                    {
                        info.Add(m.Groups["info"].Value);
                        state = 400;
                    }
                    else
                    {
                        m = m_RegexStichworte.Match(line);
                        if (m.Success)
                        {
                            stichworte.Add(m.Groups["stichworte"].Value);
                            state = 500;
                        }
                        else
                        {
                            //--- Anything else ---
                            state = 1000;
                        }
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 500)
                {
                    m = m_RegexStichworte.Match(line);
                    if (m.Success)
                    {
                        stichworte.Add(m.Groups["stichworte"].Value);
                        state = 500;
                    }
                    else
                    {
                        //--- Anything else ---
                        state = 1000;
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 600)
                {
                    m = m_RegexTitel.Match(line);
                    if (m.Success)
                    {
                        titel = m.Groups["titel"].Value;
                        state = 500;
                    }
                    else
                    {
                        //--- Anything else ---
                        state = 1000;
                    }
                }
                //--------------------------------------------------------------
                else
                if (state == 700)
                {
                    m = m_RegexSymbol.Match(line);
                    if (m.Success)
                    {
                        cluster.Add(m.Groups["symbol"].Value);
                        state = 700;
                    }
                    else
                    {
                        //--- Anything else ---
                        state = 1000;
                    }
                }
            }


            if (info.Count > 0)
                return new LineRecordNotiz(tag,
                                           monat,
                                           jahr,
                                           info,
                                           stichworte
                                          );
            else
            if ((url != null) && (url.Length > 0))
                return new LineRecordUrl(tag,
                                         monat,
                                         jahr,
                                         url,
                                         titel,
                                         stichworte
                             );
            else
            if (state == 700)
               {
                   return new LineRecordCluster(cluster);
               }
            else
               return lrec;
        }

    }
}
