// Joachim Pimiskern, 5.12.2009
using System;
using System.Collections.Generic;
using System.Text;
using Fusym;
using System.Text.RegularExpressions;

namespace Unarch5
{
    public class StructuredString : IDocument
    {
        public string Text;
        private List<string> m_Parts = new List<string>();
        private int m_Idx = 0;

        public StructuredString(string s, SemanticNetwork semanticNetwork)
        {
            Text = s;
//            m_Parts.AddRange(SplitRaw(s));

            string[] array = SplitRaw(s);
            foreach (string s1 in array)
            {
                List<string> list = SplitFine(s1);
                m_Parts.AddRange(list);
            }

        }

        private string[] SplitRaw(string s)
        {
            string[] result = Regex.Split(s, @"(_|\.|-|\,|\;|\(|\)|\s|\/|\\)+", RegexOptions.ExplicitCapture);
            return result;
        }


        private bool IsLetter(char c)
        {
            if (IsLowcase(c) || IsUpcase(c))
                return true;
            else
                return false;
        }


        private bool IsUpcase(char c)
        {
            if (
                ((c >= 'A') && (c <= 'Z')) ||
                ((c == '') || (c == '') || (c == ''))
               )
                return true;
            else
                return false;
        }


        private bool IsLowcase(char c)
        {
            if (
                ((c >= 'a') && (c <= 'z')) ||
                ((c == '') || (c == '') || (c == '') || (c == ''))
                )
                return true;
            else
                return false;
        }




        private List<string> SplitFine(string s)
        {
            List<string> result = new List<string>();
            string buffer = "";

            char oldChar = '?';

            for (int i = 0; i < s.Length; i++)
            {
                char c = s[i];

                if (
                    (IsLowcase(oldChar) && !IsLowcase(c)) ||
                    (IsLetter(oldChar) && ! IsLetter(c))   ||
                    (! IsLetter(oldChar) && IsLetter(c))
                   )
                {
                    if (buffer.Length > 0)
                    {
                        result.Add(buffer);
                        buffer = "";
                    }
                }

                buffer += c;


                oldChar = c;
            }

            if (buffer.Length > 0)
                result.Add(buffer);

            return result;
        }



        public string GetFirstSymbol()
        {
            m_Idx = 0;
            return GetNextSymbol();
        }

        public string GetNextSymbol()
        {
            string result;

            if (m_Idx < m_Parts.Count)
                result = m_Parts[m_Idx++];
            else
                result = "";

            return result;
        }
    }
}
