﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using UnarchControls;
using Fusym;
using System.Text.RegularExpressions;
using System.Runtime.InteropServices;

namespace UnarchControls
{
    public partial class AssociationRichTextBox : RichTextBox
    {
        protected static string m_WordlistSplitRegex = @"(?:[;,´‘`'’""“”„()\n\s])"; // Evtl. zentralisieren. Existiert aktuell zweimal
                                                        
        Regex regexpSeparator = new Regex(m_WordlistSplitRegex);
        GlobalClusterManager gcm = GlobalClusterManager.GetInstance();


        
        public AssociationRichTextBox()
        {
            InitializeComponent();
        }

        /*
        protected override void WndProc(ref Message m)
        {
            Console.WriteLine(m.Msg);
             switch(m.Msg)
            {

                case 0x302: //WM_PASTE
                {
                    Console.WriteLine("Paste");
                    MessageBox.Show("Paste");
                    break;
                }
                default:
                    base.WndProc(ref m);
                    break;
            }
        }

         */


        bool ctrlDown = false; 
        bool shiftDown = false;


        private CharacterCasing m_CharacterCasing = CharacterCasing.Normal;

        [
         Browsable(true),
         CategoryAttribute("Appearance"),
         Description("Uppercase, normal or lowercase characters"),
//         System.ComponentModel.RefreshProperties(RefreshProperties.Repaint)
        ]
        public CharacterCasing CharacterCasing 
        {
            get { return m_CharacterCasing; }
            set { m_CharacterCasing = value; }
        }

        [DllImport("user32.dll")]
        public static extern bool LockWindowUpdate(IntPtr hWndLock);
  

      protected override void WndProc(ref System.Windows.Forms.Message m) 
      { 
       bool handledMessage = false;  // used to decide if call to base WndProc needed. 
       switch (m.Msg) 
       { 
        case 0x0100 : 
         // WM_KEYDOWN 
         // VK_CTRL 
         if((int)m.WParam == 0x0011) {ctrlDown = true; break;} 
         // VK_SHIFT 
         if((int)m.WParam == 0x0010) {shiftDown = true; break;} 
         // ctrlDown and VK_V 
         if((ctrlDown == true) && ((int)m.WParam == 0x0056)) {handledMessage = performPaste(); break;} 
         // ctrlDown and VK_X 
         if((ctrlDown == true) && ((int)m.WParam == 0x0058)) {handledMessage = performCut(); break;}   
         // ctrlDown and VK_C 
         if((ctrlDown == true) && ((int)m.WParam == 0x0043)) {handledMessage = performCopy(); break;} 
         // shiftDown and VK_INSERT 
         if((shiftDown == true) && ((int)m.WParam == 0x002D)) {handledMessage = performPaste(); break;} 
         // shiftDown and VK_DELETE 
         if((shiftDown == true) && ((int)m.WParam == 0x002E)) {handledMessage = performCut(); break;} 
         // ctrlDown and VK_INSERT 
         if((ctrlDown == true) && ((int)m.WParam == 0x002D)) {handledMessage = performCopy();break;}     
         break; 


        case 0x0101 :
         MarkWordsAtCaretPosition();
       

         // WM_KEYUP 
         // VK_CTRL 
         if((int)m.WParam == 0x0011) {ctrlDown = false; break;} 
         // VK_SHIFT 
         if((int)m.WParam == 0x0010) {shiftDown = false; break;} 
         break; 


        case 0x0300 : 
         // WM_CUT 
         handledMessage = performCut(); 
         break; 
        case 0x0301 : 


         // WM_COPY 
         handledMessage = performCopy(); 
         break; 
        case 0x0302 : 


         // WM_PASTE 
         handledMessage = performPaste(); 
         break; 
        default : 
         break; 
       } 

       if (handledMessage == true) 
          { 
           m.Result = new IntPtr(0); 
          } 
       else 
          { 
           base.WndProc(ref m); 
          } 
      }//------------------------------------------------------------------------­-- 
      private bool performPaste() 
      { 
       bool result = false; 
       int oldSelectionStart = this.SelectionStart;
       IDataObject clipData = Clipboard.GetDataObject();
       string newText;

       if (clipData.GetDataPresent(DataFormats.Text)) 
          {
           LockWindowUpdate(this.Handle);
           newText = clipData.GetData(DataFormats.Text).ToString();

           if (m_CharacterCasing == CharacterCasing.Lower)
               newText = newText.ToLower();
           else
           if (m_CharacterCasing == CharacterCasing.Upper)
               newText = newText.ToUpper();

           this.SelectedText = newText;
           MarkRange(StartOfWord(this.Text,oldSelectionStart),
                     EndOfWord(this.Text,SelectionStart + newText.Length)
                    );
           this.SelectionStart  = oldSelectionStart + newText.Length;
           this.SelectionLength = 0;
           LockWindowUpdate(IntPtr.Zero);

           
           result = true; 
          } 
       return result; 
      }//------------------------------------------------------------------------­-- 
      private bool performCut() 
      { 
       bool result = false; 

       if (this.SelectedText != "") 
          { 
           string plainText = this.SelectedText; 
           Clipboard.SetDataObject(plainText, true); 
           this.SelectedText = ""; 
           result = true; 
          } 

       return result; 
      }//------------------------------------------------------------------------­-- 
      private bool performCopy() 
      { 
       bool result = false; 
       if(this.SelectedText != "") 
       { 
        string plainText = this.SelectedText; 
        Clipboard.SetDataObject(plainText, true); 
        result = true; 
       } 
       return result; 
      }//------------------------------------------------------------------------­-- 
 



 














        protected override void OnPaint(PaintEventArgs pe)
        {
            // TODO: Benutzerdefinierten Zeichnungscode hier einfügen

            // OnPaint-Basisklasse wird aufgerufen
            base.OnPaint(pe);
        }


        private bool IsKnownWord(string s)
        {
            SemanticNetwork cm = gcm.SemanticNetwork;
            return cm.HasAssociations(s);
        }



        private int minI(int a, int b)
        {
            return a < b ? a : b;
        }

        private int maxI(int a, int b)
        {
            return a > b ? a : b;
        }


        private Color GetColor(bool known)
        {
            if (gcm.Loaded)
            {
                if (known)
                    return Color.Black;
                else
                    return Color.Red;
            }
            else
                return Color.Blue;
        }


        private void setAllToBlack(int start, int end)
        {
            int oldSelectionStart = this.SelectionStart;
            try
            {
                this.Select(maxI(start, 0), minI(end + 1, this.Text.Length));
                this.SelectionColor = System.Drawing.Color.Black;
            }
            finally
            {
                this.SelectionStart = oldSelectionStart;
            }
        }




        private bool IsSeparator(char c)
        {
            return regexpSeparator.IsMatch(c.ToString());
        }


        public void MarkAllUnknownWords()
        {
            MarkRange(0, this.Text.Length);
        }

        
        public void MarkRange(int start, int end)
        {
            string s = this.Text;
            string collector = "";

            for (int i = maxI(start,0); i < minI(end+1,s.Length); i++)
            {
                char c = s[i];

                if (IsSeparator(c))
                {
                    if (collector.Length != 0)
                    {
                        if (!IsKnownWord(collector))
                        {
                            this.Select(i - collector.Length, collector.Length);
                            this.SelectionColor = System.Drawing.Color.Red;
                        }
                        else
                        {
                            this.Select(i - collector.Length, collector.Length);
                            this.SelectionColor = System.Drawing.Color.Black;

                        }
                        collector = "";
                    }
                }
                else
                    collector += c;
            }



            if (collector.Length != 0)
            {
                if (!IsKnownWord(collector))
                {
                    this.Select(s.Length - collector.Length, collector.Length);
                    this.SelectionColor = System.Drawing.Color.Red;
                }
                else
                {
                    this.Select(s.Length - collector.Length, collector.Length);
                    this.SelectionColor = System.Drawing.Color.Black;

                }

            }
        }



        private int StartOfWord(string text, int pos)
        {
            int result = pos-1;

            if (text.Length == 0)
               return 0;

            while ((result < text.Length) && (result >= 0))
            {
                char c = text[result];

                if (!IsSeparator(c))
                    result--;
                else
                    return result;

            }

            return result;
        }

        private int EndOfWord(string text, int pos)
        {
            int result = pos;

            while (result < text.Length)
            {
                char c = text[result];

                if (!IsSeparator(c))
                    result++;
                else
                    return result;
            }

            return result;
        }





        public void MarkWordsAtCaretPosition()
        {
            int oldSelectionStart  = this.SelectionStart;
            int oldSelectionLength = this.SelectionLength;
            string s  = this.Text;
            int start = StartOfWord(this.Text,this.SelectionStart);
            int end   = EndOfWord(this.Text,this.SelectionStart);

            LockWindowUpdate(this.Handle);
            MarkRange(start,end);
            this.SelectionStart  = oldSelectionStart;
            this.SelectionLength = oldSelectionLength;
            LockWindowUpdate(IntPtr.Zero);
        }

    }

}
